<!-- An XSL style sheet for creating MATHsAiD prefix syntax from OMDoc
     Initial version 20051004 by Juergen Zimmer, 
     send bug-reports, patches, suggestions to jzimmer@ags.uni-sb.de

     Copyright (c) 2005 Juergen Zimmer, 

     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) any later version.

     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.

     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:om="http://www.openmath.org/OpenMath"
  xmlns:dc="http://purl.org/dc/elements/1.1/"
  xmlns:omdoc="http://www.mathweb.org/omdoc"
  version="1.0">

<!--    <xsl:output method="xml" indent="yes" encoding="iso-8859-1"/> -->
<!--   <xsl:output method="xml" indent="yes" encoding="UTF-8"/> -->

  <!-- commont template definitions -->
  <xsl:template name="replace-string">
    <xsl:param name="text"/>
    <xsl:param name="replace"/>
    <xsl:param name="with"/>
    <xsl:choose>
      <xsl:when test="contains($text,$replace)">
        <xsl:value-of select="substring-before($text,$replace)"/>
        <xsl:value-of select="$with"/>
        <xsl:call-template name="replace-string">
	  <xsl:with-param name="text" select="substring-after($text,$replace)"/>
          <xsl:with-param name="replace" select="$replace"/>
          <xsl:with-param name="with" select="$with"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$text"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- <xsl:import href="omdoc2sys.xsl"/> -->
<xsl:strip-space elements="*"/>
<xsl:output method="text"/>


<xsl:template match="omdoc:CMP"><xsl:apply-templates/></xsl:template>

<xsl:template match="omdoc:FMP">
  <xsl:if test="om:OMOBJ"></xsl:if>
  <xsl:apply-templates/>
  <xsl:if test="om:OMOBJ"></xsl:if>
</xsl:template>

<xsl:template match="omdoc:omtext">
  <xsl:call-template name="print-ccmp"/>
  <xsl:text>&#xA;&#xA;</xsl:text>
</xsl:template>

<!-- #################### Theory Elements ##################### -->
 
<xsl:template match="omdoc:theory"> 
  <xsl:variable name="Theory"><xsl:value-of select="@xml:id"/></xsl:variable>
<!--   <xsl:if test="count(descendant::omdoc:imports)>0"> -->
<!--     <xsl:apply-templates/> -->
<!--   </xsl:if> -->
  <xsl:if test="$Theory!='hol' and $Theory!='Mathsaid'">
    <xsl:apply-templates/>
  </xsl:if>
</xsl:template>

<xsl:template match="omdoc:axiom[omdoc:FMP[om:OMOBJ[om:OMA[om:OMS[@cd='Mathsaid']]]]]">
  <xsl:variable name="Id"><xsl:value-of select="@xml:id"/></xsl:variable>
<!--   <xsl:text>(th~axiom </xsl:text><xsl:value-of select="@xml:id"/><xsl:text>&#xA;</xsl:text> -->
  <xsl:text>MetaAxiom(</xsl:text>
  <xsl:value-of select="@xml:id"/>
  <xsl:text>): </xsl:text>
  <xsl:apply-templates select="omdoc:FMP"/>
  <xsl:text>&#xA;</xsl:text>
</xsl:template>

<xsl:template match="omdoc:axiom">
  <xsl:variable name="Id"><xsl:value-of select="@xml:id"/></xsl:variable>
<!--   <xsl:text>(th~axiom </xsl:text><xsl:value-of select="@xml:id"/><xsl:text>&#xA;</xsl:text> -->
  <xsl:text>Axiom(</xsl:text>
  <xsl:value-of select="@xml:id"/>
  <xsl:text>) Then: </xsl:text>
  <xsl:apply-templates select="omdoc:FMP"/><xsl:text>&#xA;</xsl:text>
</xsl:template>

<xsl:template match="omdoc:assertion[@status='theorem']">
  <xsl:variable name="Id"><xsl:value-of select="@xml:id"/></xsl:variable>
  <xsl:text>Theorem(</xsl:text>
  <xsl:value-of select="@xml:id"/>
  <xsl:text>) Then: </xsl:text>
  <xsl:apply-templates select="omdoc:FMP[omdoc:conclusion]"/><xsl:text>&#xA;</xsl:text>
</xsl:template>

<xsl:template match="omdoc:assertion">
  <xsl:variable name="Id"><xsl:value-of select="@xml:id"/></xsl:variable>
  <xsl:text>Conjecture(</xsl:text>
  <xsl:value-of select="@xml:id"/>
  <xsl:text>) Then: </xsl:text>
  <xsl:apply-templates select="omdoc:FMP[omdoc:conclusion]"/><xsl:text>&#xA;</xsl:text>
</xsl:template>

<!-- ################ Symbols ################ -->

<xsl:template match="omdoc:type">
  <xsl:apply-templates select="om:OMOBJ"/>
</xsl:template>

<xsl:template match="omdoc:symbol[@scope='local']">
<!--   <xsl:text>(</xsl:text> -->
<!--   <xsl:choose> -->
<!--     <xsl:when test="omdoc:metadata/dc:subject"> -->
<!--       <xsl:value-of select="omdoc:metadata/dc:subject"/> -->
<!--     </xsl:when> -->
<!--     <xsl:otherwise><xsl:value-of select="@name"/></xsl:otherwise> -->
<!--   </xsl:choose> -->
<!-- <xsl:text> </xsl:text> -->
<!--   <xsl:apply-templates select="omdoc:type"/> -->
<!--   <xsl:text>) </xsl:text> -->
</xsl:template>

<xsl:template match="omdoc:symbol[@kind='object' and @scope='global']">
  <xsl:variable name="Id">
    <xsl:call-template name="normalise-name">
      <xsl:with-param name="name" select="@name"/>
    </xsl:call-template>
  </xsl:variable>
  
  <!--   <xsl:number value="count(descendant::om:OMA)"/>    -->
  <!--   <xsl:variable name="funtypes" value="count(descendant::om:OMA)"/> -->
  <!--   <xsl:value-of select="$funtypes"/> -->
  <xsl:if test="count(descendant::om:OMA)=0">
    <xsl:text>MetaAxiom(constant_</xsl:text>
    <xsl:value-of select="$Id"/>
    <xsl:text>): constant(</xsl:text>
    <xsl:value-of select="$Id"/>
    <xsl:text>)&#xA;</xsl:text>
  </xsl:if>
  <xsl:if test="count(descendant::om:OMA)>0">
    <xsl:text>MetaAxiom(operator_</xsl:text>
    <xsl:value-of select="$Id"/>
    <xsl:text>): operator(</xsl:text>
    <xsl:value-of select="$Id"/>
    <xsl:text>)&#xA;</xsl:text>
  </xsl:if>
</xsl:template>


<!-- ################ OpenMath ################-->

<xsl:template match="om:OMOBJ"><xsl:apply-templates/></xsl:template>

<!-- set(A) & set(B) ==> forall([x], x fadfasdfkj  -->

<xsl:template match="om:OMA">
  <xsl:apply-templates select="*[1]"/>  
  <xsl:text>(</xsl:text>
    <xsl:for-each select="*">
      <xsl:if test="position()!=1">
        <xsl:apply-templates select="."/>
      </xsl:if>
      <xsl:if test="position()!=last() and position()!=1">
        <xsl:text>, </xsl:text>
      </xsl:if>
    </xsl:for-each>
  <xsl:text>)</xsl:text>
</xsl:template>

<xsl:template name="normalise-name">
  <xsl:param name="name"/>
<!--   <xsl:variable name="name0"> -->
<!--     <xsl:call-template name="replace-string"> -->
<!--       <xsl:with-param name="text" select="$name"/> -->
<!--       <xsl:with-param name="replace" select="'+'"/> -->
<!--       <xsl:with-param name="with" select="'esc_plus_'"/> -->
<!--     </xsl:call-template> -->
<!--     <xsl:value-of select="translate($name, '-', 'esc_usc_')"/> -->
<!--   </xsl:variable> -->
  <xsl:variable name="name1">
    <xsl:call-template name="replace-string">
      <xsl:with-param name="text" select="$name"/>
      <xsl:with-param name="replace" select="'-'"/>
      <xsl:with-param name="with" select="'esc_hyph_'"/>
    </xsl:call-template>
    <!--     <xsl:value-of select="translate($name, '-', 'esc_usc_')"/> -->
  </xsl:variable>
<!--   <xsl:variable name="name2"> -->
<!--     <xsl:call-template name="replace-string"> -->
<!--       <xsl:with-param name="text" select="$name1"/> -->
<!--       <xsl:with-param name="replace" select="'*'"/> -->
<!--       <xsl:with-param name="with" select="'esc_star_'"/> -->
<!--     </xsl:call-template> -->
<!--   </xsl:variable> -->
  <xsl:variable name="name3">
    <xsl:call-template name="replace-string">
      <xsl:with-param name="text" select="$name1"/>
      <xsl:with-param name="replace" select="':'"/>
      <xsl:with-param name="with" select="'esc_col_'"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="name4">
    <xsl:call-template name="replace-string">
      <xsl:with-param name="text" select="$name3"/>
      <xsl:with-param name="replace" select="'('"/>
      <xsl:with-param name="with" select="'esc_lpar_'"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="name5">
    <xsl:call-template name="replace-string">
      <xsl:with-param name="text" select="$name4"/>
      <xsl:with-param name="replace" select="')'"/>
      <xsl:with-param name="with" select="'esc_rpar_'"/>
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="name6">
    <xsl:call-template name="replace-string">
      <xsl:with-param name="text" select="$name5"/>
      <xsl:with-param name="replace" select="'!'"/>
      <xsl:with-param name="with" select="'esc_excl_'"/>
    </xsl:call-template>
<!--     <xsl:value-of select="translate($name2, ':', 'esc_col_')"/> -->
  </xsl:variable>
  <xsl:value-of select="$name6"/>
 </xsl:template>


<xsl:template match="om:OMV">
<!--   <xsl:value-of select="@name"/> -->
<!--   <xsl:value-of select="translate(@name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', -->
            <!--   'abcdefghijklmnopqrstuvwxyz')"/> -->
  <xsl:call-template name="normalise-name">
     <xsl:with-param name="name" select="@name"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMS[@name='eq' and @cd='relation1']">
  <xsl:text>=</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='eq' and @cd='hol']">
  <xsl:text>=</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='Eq' and @cd='hol']">
  <xsl:text>=</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='equal' and @cd='hol']">
  <xsl:text>=</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='implies' and @cd='pl0']">
  <xsl:text>==></xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='Impl' and @cd='hol']">
  <xsl:text>==></xsl:text>
</xsl:template>


<xsl:template match="om:OMS[@name='equiv' and @cd='pl0']">
  <xsl:text>&lt;==></xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='Eqv' and @cd='hol']">
  <xsl:text>&lt;==></xsl:text>
</xsl:template>


<xsl:template match="om:OMS[@name='equivalent' and @cd='pl0']">
  <xsl:text>&lt;==></xsl:text>
</xsl:template>


<xsl:template match="om:OMS[@name='and' and @cd='pl0']">
<!--   <xsl:text>&amp;</xsl:text> -->
  <xsl:text>and</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='And' and @cd='hol']">
<!--   <xsl:text>&amp;</xsl:text> -->
  <xsl:text>and</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='or' and @cd='pl0']">
  <xsl:text>or</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='Or' and @cd='hol']">
  <xsl:text>or</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='not' and @cd='pl0']">
<!--   <xsl:text>&#xAC;</xsl:text> -->
  <xsl:text>not</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='Not' and @cd='hol']">
  <xsl:text>not</xsl:text>
</xsl:template>


<xsl:template match="om:OMS[@name='welldef' and @cd='Mathsaid']">
  <xsl:text>welldef</xsl:text>
</xsl:template>

<xsl:template match="om:OMS[@name='undefterm' and @cd='Mathsaid']">
  <xsl:text>undefterm</xsl:text>
</xsl:template>

<xsl:template match="om:OMS">
<!--   <xsl:value-of select="@name"/> -->
  <xsl:call-template name="normalise-name">
     <xsl:with-param name="name" select="@name"/>
  </xsl:call-template>
<!--   <xsl:value-of select="translate(@name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')"/> -->
</xsl:template>

<xsl:template match="om:OMBVAR">
<!--   <xsl:value-of select="@name"/> -->
  <xsl:call-template name="normalise-name">
     <xsl:with-param name="name" select="@name"/>
  </xsl:call-template>
<!--   <xsl:value-of select="translate(@name, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', 'abcdefghijklmnopqrstuvwxyz')"/> -->
</xsl:template>

<xsl:template match="om:OMI"><xsl:value-of select="."/><xsl:text> </xsl:text></xsl:template>

<!-- <xsl:template match="om:OMBIND[om:OMS[@name='all-types' and @cd='POST']]"> -->
<!--   <xsl:text>(all-types </xsl:text> -->
<!--     <xsl:apply-templates select="om:OMBVAR/*"/> -->
<!--     <xsl:apply-templates select="*[3]"/> -->
<!--   <xsl:text>)</xsl:text> -->
<!-- </xsl:template> -->

<xsl:template name="lambda">
  <xsl:param name="quantor"/>
  <xsl:param name="varlist"/>
  <xsl:param name="body"/>
  <xsl:for-each select="$varlist">
    <xsl:value-of select="$quantor"/>
    <xsl:text>([</xsl:text>
    <xsl:apply-templates select="." />
    <xsl:text>], </xsl:text>
  </xsl:for-each>
<!--   <xsl:text>&#xa;</xsl:text> -->
  <xsl:apply-templates select="$body" />
  <xsl:for-each select="$varlist">
    <xsl:text>)</xsl:text>
  </xsl:for-each>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='forall' and @cd='quant1']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'forall'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='forall' and @cd='sthol']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'forall'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='forall' and @cd='pl1']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'forall'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMA[om:OMS[@name='PI' and @cd='hol']]">
  <!-- [om:OMBIND[om:OMS[@name='lambda' and @cd='fns1']]] -->
  <xsl:for-each select="om:OMBIND[om:OMS[@name='lambda' and @cd='fns1']]">
<!--     <xsl:call-template name="lambda"> -->
<!--       <xsl:with-param name="quantor" select="'forall'"/> -->
<!--       <xsl:with-param name="varlist" select="om:OMBVAR/*"/> -->
<!--       <xsl:with-param name="body" select="*[3]"/> -->
<!--     </xsl:call-template> -->
    <xsl:call-template name="lambda">
      <xsl:with-param name="quantor" select="'forall'"/>
      <xsl:with-param name="varlist" select="om:OMBVAR/om:OMV|om:OMBVAR/om:OMATTR/om:OMV"/>
      <xsl:with-param name="body" select="*[3]"/>
    </xsl:call-template>
  </xsl:for-each>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='exists' and @cd='quant1']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'exists'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='exists' and @cd='pl1']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'exists'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMBIND[om:OMS[@name='exists' and @cd='sthol']]">
  <xsl:call-template name="lambda">
     <xsl:with-param name="quantor" select="'exists'"/>
     <xsl:with-param name="varlist" select="om:OMBVAR/*"/>
     <xsl:with-param name="body" select="*[3]"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="om:OMA[om:OMS[@name='funtype' and @cd='mltt']]">
<!--   <xsl:text>(</xsl:text> -->
    <xsl:apply-templates select="*[3]"/>
<!--     <xsl:text> </xsl:text> -->
<!--     <xsl:apply-templates select="*[2]"/> -->
<!--   <xsl:text>)</xsl:text> -->
</xsl:template>

<xsl:template match="om:OMA[om:OMS[@name='funtype' and @cd='simpletypes']]">
<!--   <xsl:text>(</xsl:text> -->
<!--     <xsl:apply-templates select="*[3]"/> -->
<!--     <xsl:text> </xsl:text> -->
<!--     <xsl:apply-templates select="*[2]"/> -->
<!--   <xsl:text>)</xsl:text> -->
</xsl:template>

<xsl:template match="om:OMATTR[om:OMATP[om:OMS[@name='type' and @cd='mltt']]]">
<!--  <xsl:text>(</xsl:text> -->
  <xsl:apply-templates select="*[2]"/>
<!--   <xsl:text> </xsl:text> -->
<!--   <xsl:apply-templates select="om:OMATP/*[2]"/> -->
<!--   <xsl:text>)</xsl:text> -->
</xsl:template>

<xsl:template match="om:OMATTR[om:OMATP[om:OMS[@name='type' and @cd='simpletypes']]]">
<!--  <xsl:text>(</xsl:text> -->
<!--   <xsl:apply-templates select="*[2]"/> -->
<!--   <xsl:text> </xsl:text> -->
<!--   <xsl:apply-templates select="om:OMATP/*[2]"/> -->
<!--   <xsl:text>)</xsl:text> -->
</xsl:template>


<xsl:template name="print-cmp">
  <xsl:choose>
    <xsl:when test="CMP[xml:lang='en']">
      <xsl:apply-templates select="CMP[xml:lang='en']"/>
    </xsl:when>
    <xsl:when test="CMP[not(xml:lang)]">
      <xsl:apply-templates select="CMP[not(xml:lang)]"/>
    </xsl:when>
  </xsl:choose>
</xsl:template>

<xsl:template name="print-ccmp">
  <xsl:if test="CMP[not(xml:lang) or xml:lang='en']">
    <xsl:text>#|</xsl:text>
    <xsl:call-template name="print-cmp"/>
    <xsl:text>|#&#xA;</xsl:text>
  </xsl:if>
</xsl:template>

<xsl:template name="bcomment"><xsl:text>;;; </xsl:text></xsl:template>

</xsl:stylesheet>





