// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/
package org.mathweb.atp;

import java.io.*;
import java.util.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import javax.xml.namespace.QName;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

import org.mindswap.query.ValueMap;

import org.mathweb.owls.OWLSParser;
import org.mathweb.onto.OWLJDom;
import org.mathweb.serv.ReasoningService;
import org.mathweb.util.Java2WSDL;
import org.mathweb.util.TSTPCreator;

public class SpassClient
{

  public String readFile(String fileName){
    try {
      FileInputStream isFile = new FileInputStream(fileName);
      int nChar;
      StringBuffer sbRead = new StringBuffer();
      nChar = isFile.read();
      while (nChar > -1) {
	sbRead.append((char) nChar);
	nChar = isFile.read();
      }
      isFile.close();
      return sbRead.toString();
    } catch (IOException ioe) {
      return null;
    }
  }

  public static void main(String [] args) {
    String msHome = System.getProperty("mathserv.home");
    OWLJDom owlJDom = OWLJDom.getInstance();
    TSTPCreator creator = new TSTPCreator();
    String timeInd = owlJDom.createTimeIndi(8, 8);
    try {
      String hostname;
      if ((args.length > 0) && (!args[0].trim().equals(""))) 
	hostname = args[0];
      else 
	hostname = "localhost";
      
      OWLSParser owlsParser = new OWLSParser();
      
      SpassClient c = new SpassClient();
      
      String fofProb = c.readFile(msHome + "/examples/owl/fof2.owl");
      fofProb = creator.tstp2OWLProblem("/home/jzimmer/fuer_Juergen.tptp");

      String wsdl = null;

      Vector ms = new Vector();
      ms.add("proveProblem");
      ms.add("proveProblemXml");
      wsdl = (new Java2WSDL()).java2WSDL(hostname, "SpassService", "org.mathweb.atp.Spass" , ms);
      //      System.out.println(" wsdl:  " + wsdl);

      ReasoningService service = owlsParser.parseServiceFile(msHome + "/etc/services/owls/1.1/SpassService.owl", wsdl);
      //System.out.println("Service " + service);

      Hashtable map = new Hashtable();
      map.put("tstp_problem", fofProb);
      map.put("time_res", timeInd);
      ValueMap resTab = service.invokeOWLS(map);
      System.out.println("Sent problem and got '" + resTab + "'");
    } catch (Exception e) {
      e.printStackTrace();
      System.err.println(e.toString());
    }
  }
}
