// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/* For testing:
run org.mathweb.atp.TPTPClassifier /home/jzimmer/casc/unsolved-solved-by-vampire-7 /home/jzimmer/casc
run org.mathweb.atp.TPTPClassifier /home/jzimmer/casc/ForJZ-orig /home/jzimmer/casc/Active
*/

package org.mathweb.atp;

import java.io.*;
import java.net.URI;
//import java.util.StringTokenizer;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Enumeration;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import dk.capgemini.tc.JProf;

import org.mathweb.util.TSTPCreator;

import org.mathweb.core.BasicService;
import org.mathweb.onto.OWLJDom;

//import monet.openmath.lang.*;
//import monet.openmath.lang.*;

import org.mathweb.core.BasicService;

/**
@author Juergen Zimmer University of Edinburgh
@version $Date: 2006/08/24 13:40:31 $ $Revision: 1.28 $
*/
public class TPTPClassifier extends BasicService {
  
  public String TPTP_HOME = getSystemProperty("tptp.home.dir", "TPTP.HOME.DIR");
  public String CLASSIFIER_HOME = getSystemProperty("tptp.classifier.home", "TPTP_CLASSIFIER_HOME");
  public String[] classifier_envs = new String[] {"TPTP_HOME="+TPTP_HOME, "TPTP="+TPTP_HOME+sep+"TPTP"};
  public String classifierExecutable = findSystemExecutable(CLASSIFIER_HOME, "SPCForProblem", "SPCForProblem.exe");
  public String acceptedConcept = "TstpProblem";

  private OWLJDom owlJDom;

  //  private HashMap classifier_map = new HashMap();
  
  private static Logger logger = Logger.getLogger("Classifier");
  private static String logFileName = getSystemProperty("tptp.classifier.log.file", "TPTP_CLASSIFIER_LOG_FILE");
  private static FileAppender appender = newAppender(logFileName);

  private static int maxInputLength = 100000;
  
  public TPTPClassifier()
  { //THM_RFO_EQU_FOF
    //mkTempDir("tptp_classifier");
    //    System.out.println("Temporary dir is "+ tempDir);
    
//     classifier_map.put("EPR","essentiallyProp");
//     classifier_map.put("RFO","realFirstOrder");
//     classifier_map.put("FOF","fofForm");
//     classifier_map.put("CNF","cnfForm");
//     classifier_map.put("HRN","Horn");
//     classifier_map.put("NHN","notHorn");
//     classifier_map.put("EQU","equality");
//     classifier_map.put("NEQ","noEquality");
//     classifier_map.put("SEQ","someEquality");
//     classifier_map.put("PEQ","pureEquality");
//     classifier_map.put("UEQ","unitPureEquality");
//     classifier_map.put("NUE","nonUnitPureEquality");
//     classifier_map.put("THM","theorem");
//     classifier_map.put("SAT","satisfiable");
//     classifier_map.put("NKS","notKnownSatisfiable");
//     classifier_map.put("NKC","notKnownCounterSatisfiable");
    
    owlJDom = OWLJDom.getInstance();
    
    logger.setLevel(Level.ALL);
    logger.setLevel(Level.OFF);
    Enumeration enu = logger.getAllAppenders();
//     for( ; enum.hasMoreElements() ;) {
//       System.out.println("appender: " +enum.nextElement());
//     }
    if (!enu.hasMoreElements()) logger.addAppender(appender);
//     FileAppender appender = null;
//     try {
//       appender = new FileAppender(new SimpleLayout(), logFileName, false);
//     } catch(Exception e) {}
//     logger.addAppender(appender);
  }
  

  
  public String analyseProblem(String provingProblem)
  {
    long startTime = System.currentTimeMillis();
    long cpuStart = JProf.getCurrentThreadCpuMillis();
    String probID = null;
    boolean valid = false; 
    Element probRoot = null;
    File problemFile = null;
    
    //    logger.info("[Classify]: " + provingProblem);
    
    try{
      if ((provingProblem != null) && (provingProblem.length() > maxInputLength)){
        //      System.out.println("monster problem " + probID);
        //        return owlJDom.createFailure(probID, startTime, cpuStart, "Problem (of size " + provingProblem.length() + " bytes) is too big to be handled by MathServe at the moment.");
        Document probDoc = owlJDom.parseLargeDocument(provingProblem);
        StringWriter writer = new StringWriter();
        new XMLOutputter().output(probDoc,writer);
        /* The new XML document with huge problem descriptions written in a file */
        probRoot = probDoc.getRootElement();
      } else {
        probRoot = owlJDom.getRootElement(provingProblem);
      }

      String xmlBase = probRoot.getAttributeValue("base", Namespace.XML_NAMESPACE);
      
      logger.info("... parsing, took me " + (System.currentTimeMillis() - startTime) + " msecs");
      logger.debug("[Classify] : " + probRoot + " " + owlJDom);
      //        List cs = probRoot.getChildren();
      //        for(int i=0; i<cs.size();i++) logger.debug(cs.get(i));
      List children = probRoot.getChildren();
      Element rightChild = null;
      for(int i=0; i<children.size(); i++){
          Element c = (Element) children.get(i);
          if ((c.getName()).indexOf("Problem") > -1) {
              String potentialID = owlJDom.valueOf(owlJDom.RDF_ID, c);
              if (potentialID != null) {
                  rightChild = c;
                  probID = xmlBase + potentialID;
                  break;
              }
          }
      }
      if (probID == null) 
        return owlJDom.createFailure(xmlBase, startTime, cpuStart, "Problem has to be of class " + acceptedConcept + "!");
      logger.info("[Classify] prob ID: " + probID);
 //      valid = reasoner.tellIndividual(provingProblem);
//       if (!valid)
//         return owlJDom.createFailure(probID, startTime, cpuStart, "Problem has to be valid RDF description of an OWL individual of class TstpProblem!");
      Element formal = (Element) OWLJDom.PROVING_PROBLEM_FORMAL_DESCRIPTIONS.selectSingleNode(rightChild);
      logger.info("formal: " + formal);
      Attribute resource = formal.getAttribute("resource", OWLJDom.RDF_NS);
      if (resource != null){
        problemFile = new File(new URI(resource.getValue()));
        logger.info("formal file: " + problemFile.getAbsolutePath());
      } else {
        String content = formal.getText();
        logger.info("formal: " + content);
        problemFile = File.createTempFile("tstp", ""); 
        logger.info("file: " + problemFile.getAbsolutePath());
        writeFile(problemFile, content);
      }
      
      //      Element formal = (Element) OWLJDom.PROVING_PROBLEM_FORMAL_DESCRIPTIONS.selectSingleNode(rightChild);
      if (problemFile == null) 
        return owlJDom.createFailure(probID, startTime, cpuStart, "Problem has to be valid RDF description of an OWL individual of class TstpProblem!");
      //      String problem = formal.getTextTrim();
      
      URI probURI = URI.create(probID);
      //       String problem = reasoner.valueOf("formalDescription", probURI);
      //      logger.info("[Classify] prob: " + problem);
      /*-----------------------------------------------------*/
      /*  Call the SCP classifier on the formal description */
      /*-----------------------------------------------------*/
      String[] result = classifyProblem(problemFile);

      /* now we can delete the input file */
      if (problemFile != null) {
        problemFile.delete();
        problemFile = null;
      }

      String pClass = result[0];
      String failure = result[1];
      if (failure != null){
        return owlJDom.createFailure(probID, startTime, cpuStart, failure);
      }
      
      logger.info("[Classify] SPC: " + pClass);
      if (pClass == null){
        return owlJDom.createFailure(probID, startTime, cpuStart, "Could not classify problem " + probID);
      }
      
      return createResult(probID, startTime, cpuStart, pClass);
    } catch(Exception e){
      return owlJDom.createFailure(probID, startTime, cpuStart, e.getMessage());
    } finally{
      if (problemFile != null) {
        problemFile.delete();
        problemFile = null;
      }
    }
  }
  
  public String createResult(String probID, long startTime, long cpuStart, String pClass) {
    try{
      Element rdf = new Element("RDF", OWLJDom.RDF_NS);
      rdf.addNamespaceDeclaration(OWLJDom.MW_NS);
      rdf.setAttribute("base", OWLJDom.MW_NS.getURI(), Namespace.XML_NAMESPACE);
      Element classE = new Element("FOProblemClass",OWLJDom.MW_NS);
      classE.setAttribute("about", OWLJDom.MW_NS_STR+pClass, OWLJDom.RDF_NS);

      Element timeres = OWLJDom.createTimeResource(startTime, cpuStart);

      rdf.addContent(classE); 
      rdf.addContent(timeres); 
      return owlJDom.newXMLDocument(rdf);
    }
    catch(Exception e){
      return owlJDom.createFailure(probID, startTime, cpuStart, e.getMessage());
    }
  }
  
  public String[] classifyProblem(String problem){
    
    File inputFile = null;
    try {
      inputFile = File.createTempFile("classifier", ".in");
      return classifyProblem(inputFile);
    } catch(Exception ex){
      ex.printStackTrace();
      return null;
    } finally{
      if (inputFile != null) inputFile.delete();
    }
  }
  
  /* NOTE: SPCForProblem cannot classify axioms of the form 
    fof(ax_11,axiom,
      ( ! [X] : 
      ( ? [Y] : 
        ~(hates(U, Y))))).
  */

  public String[] classifyProblem(File inputFile){
    
    File outputFile = null;
    try {
      outputFile = File.createTempFile("classifier", ".out");
      String inputFileName = inputFile.getAbsolutePath();
      String outputFileName = outputFile.getAbsolutePath();
      String command = classifierExecutable + " " + inputFileName + "> " +outputFileName + " 2>&1";
      //      System.out.println("prob: " + command);
      
      // write the TPTP problem into the file
      //      this.writeFile(inputFile, problem);
      
      System.out.println("command: " + command);
      // call the SPCForProblem classifier 
      String[] commands = {"sh", "-c", command};
      this.exec(commands, classifier_envs, false, true);
      String result = readFile(outputFile);
      if (result == null) 
        return new String[] {null, null};
      System.out.print(result + "\n");
      if (result.indexOf("ERROR") > -1){
        String problem = readFile(inputFile);
        return new String[] {null, result + "\n in problem: \n----------8<----------\n" + problem};
      }
      else 
        return new String[] {result.trim(), null };
      
      //        System.out.println("[TPTPClassifier] problem classified as: " + result);
      //         StringTokenizer st = new StringTokenizer(result, "_");
      //         HashSet feature_set = new HashSet();
      //         String token;
      //         while (st.hasMoreTokens()){
      //           token = st.nextToken().trim();
      //           //TODO handle EQU properly
      //           if (!(token.equals("THM") || token.equals("SAT") || token.equals("EQU")))
      //             feature_set.add(classifier_map.get(token));
      //         }
      //         Iterator iter = feature_set.iterator();
      //         //        System.out.println("features: ");
      //         //         while (iter.hasNext()) {
      //         //           System.out.println(iter.next());
      //         //         }
      //         if (inputFile != null) inputFile.delete();
      //         if (outputFile != null) outputFile.delete();
      //         return feature_set;
      //       }
      //       return null;      
      
    } catch(Exception e){
      e.printStackTrace();
      return null;
    } finally{
      if (outputFile != null) outputFile.delete();
    }
  }

  public String[] classifyFile(String inputFileName)
  {
    //    System.out.print(inputFileName + " " );
    String problem = this.readFile(inputFileName);
    return classifyProblem(problem);
  }


  public void classifyFileList(String listFileName, String prefix)
  {
    FileOutputStream outputFile = null; 
    
    String spcFileName = listFileName;
    Hashtable spcHash = new Hashtable();
    Hashtable fileHash = new Hashtable();

    String[] spcLines = readFile(spcFileName).split("\n");
    int problems = spcLines.length;
    StringBuffer output = new StringBuffer();
    
    for(int i = 0; i< problems; i++){
      String line = spcLines[i].trim();
      String[] parts = line.split("\t");
      String fileName = prefix + "/" + parts[0].trim();
      //      System.out.println("file: " + fileName);
      String spc = null;
      String[] result;
      
      try{
        result = classifyFile(fileName);
        spc = result[0];
      }catch(Exception e) {
        e.printStackTrace();
      } 
      output.append(line);
      output.append("\t");
      //      System.out.println(spc);
      if (spc != null) {
        output.append(spc);
        if (spcHash.containsKey(spc)) {
          Integer pre = (Integer) spcHash.get(spc);
          spcHash.put(spc, new Integer(pre.intValue() + 1));
        } else {
          spcHash.put(spc, new Integer(1));
        }
        if (fileHash.containsKey(spc)) {
          String fileNames = (String) fileHash.get(spc);
          fileHash.put(spc, fileNames + parts[0].trim()  + "\n");
        } else {
          fileHash.put(spc, parts[0].trim()  + "\n");
        }
      }
      output.append("\n");
      /* SPCs of 71 problems not solved by mathserve but by vampire 8.0 */
// CNF_NKS_RFO_PEQ_NUE 1
// CNF_NKS_RFO_NEQ_NHN 5
// CNF_NKS_RFO_SEQ_NHN 15
// FOF_NKC_RFO_NEQ 1
// FOF_NKC_RFO_EQU 23

/*
    CNF_NKS_RFO_SEQ_NHN & 15 & 1816 & {\eprover}\\\hline
    FOF_NKC_RFO_EQU      & 23 & 915  & {\eprover} \\\hline\hline
*/

//       || spc.equals("FOF_NKC_RFO_NEQ") 
//            || spc.equals("CNF_NKS_RFO_PEQ_NUE")
//            || spc.equals("CNF_NKS_RFO_NEQ_NHN") 

      if (   spc.equals("CNF_NKS_RFO_SEQ_NHN") 
          || spc.equals("FOF_NKC_RFO_EQU"))
        System.out.println(line);
    }
    for(Enumeration en= spcHash.keys(); en.hasMoreElements();){
        String key = (String) en.nextElement();
        this.writeFile("/home/jzimmer/casc/spcs/" + key, (String) fileHash.get(key));
    }

    //     System.out.println("SPCs for Problems");
//     System.out.println(output);

//     System.out.println("SPCs occuring");
//     for(Enumeration en= spcHash.keys(); en.hasMoreElements();){
//         String key = (String) en.nextElement();
//         System.out.println(key + " " + spcHash.get(key));
//     }

  }
  
  public void countSPCs(String listFileName)
  {
    String[] spcLines = readFile(listFileName).split("\n");
    int problems = spcLines.length;
    Hashtable spcs = new Hashtable();
    
    for(int i = 0; i< problems; i++){
        String line = spcLines[i].trim();
        String[] parts = line.split("   ");
        String spc = parts[1].trim();
        //       String[] parts = line.split("\t");
//       String spc = parts[0].trim();
      System.out.println(spc);
      if (spcs.containsKey(spc)) {
        Integer pre = (Integer) spcs.get(spc);
        spcs.put(spc, new Integer(pre.intValue() + 1));
      } else {
        spcs.put(spc, new Integer(1));
      }
    }
    System.out.println(spcs);
  }

  
  
  public static void main(String argv[]) throws Exception {
    String msHome = System.getProperty("mathserv.home");
    TPTPClassifier c = new TPTPClassifier();
    TSTPCreator creator = new TSTPCreator();
    String ind;
        
 //    if (argv.length>1) {
//       String fileList = argv[0];
//       String prefix = argv[1];
//       c.classifyFileList(fileList, prefix);
//       //      c.countSPCs(fileList);
//     } 
// 
     //                      /     if (argv.length>0) {
//       c.countSPCs(argv[0]);
//     } 
    //else {
//     String servF1;
//     servF1 = msHome + "/examples/owl/fof3.owl";
//     String ind = c.readFile(servF1);
//     //ind = creator.tstp2OWLProblem(msHome + "/thirdParty/TPTPWorld/TPTP/Problems/NLP/NLP061+1.p");
//    ind = creator.tstp2OWLProblem("/home/jzimmer/casc/Active/Problems/EPS/NoTransform/tptp/SYN/SYN853-1+short+ran.p");
//    ind = creator.tstp2OWLProblem("/home/jzimmer/fail1.tstp");
    ind = creator.tstp2OWLProblem("/home/jzimmer/core1.p");
        //     ind = creator.tstp2OWLProblem(msHome + "/thirdParty/TPTPWorld/TPTP/Problems/SYN/SYN880-1.p");
    
    String res = c.analyseProblem(ind);
    System.out.println(res);
    //    }
  }
  
 
} // end TPTPClassifier.java
    
      
      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

