// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

//   jade -gui atp:org.mathweb.atp.ATPAgent dummy0:jade.tools.DummyAgent.DummyAgent

package org.mathweb.atp;

import java.util.HashMap;
import java.io.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

//import org.tptp.xtstp.TSTP2XMLReader;
import net.sf.xff.tstp.TSTP2XMLReader;
//  import jade.gui.GuiAgent;
//  import jade.gui.GuiEvent;


import org.mathweb.core.BasicService;

/**
   @author Juergen Zimmer Universitaet des Saarlandes 
   @version $Date: 2006/08/25 14:29:28 $ $Revision: 1.59 $
*/
public class TSTP2X extends BasicService {
  
  static boolean verbose=true;
  
  private static String sep = File.separator;
  public String TPTP_HOME = getSystemProperty("tptp.home.dir", "TPTP.HOME.DIR");
  public String TPTP = TPTP_HOME + "/TPTP";
  public String SWI_HOME =  getSystemProperty("tptp.swi.prolog", "TPTP_SWI_PROLOG");
  public String YAP_HOME =  getSystemProperty("tptp.yap.prolog", "TPTP_YAP_PROLOG");
  public String[] tptp_envs = new String[] {"TPTP_HOME="+TPTP_HOME, "TPTP="+TPTP, "TPTPDirectory="+TPTP_HOME, "SWI_HOME="+SWI_HOME, "YAP_HOME="+YAP_HOME, "YAPLIBDIR="+YAP_HOME};
  public String tptp2XPath = TPTP_HOME + sep + "TPTP" + sep +"TPTP2X";
  public String tptp2xExecutable = findSystemExecutable(tptp2XPath, "tptp2X", "tptp2X.exe");
  public String tptp2PostXslFile = getSystemProperty("tptp.post.xsl", "TPTP_POST_XSL");
  public String tptp2OMDocXslFile = getSystemProperty("tptp.omdoc.xsl", "TPTP_OMDOC_XSL");

  public String TRANS_HOME = getSystemProperty("tstp.transform.home", "TSTP_TRANSFORM_HOME");
  public String swiExecutable = findSystemExecutable(SWI_HOME, "pl", "pl.exe");
  public String tstp2xtstpSWI = findSystemExecutable(TRANS_HOME, "tstp2xtstp.swi", "tstp2xtstp.swi");

  private static Logger logger = Logger.getLogger("TSTP2X");
  private static String logFileName = getSystemProperty("tstp2x.log.file", "TSTP_LOG_FILE");
  private static FileAppender appender = newAppender(logFileName);
  
  private int counter = 0;
  
  private Transformer transformer;
  
  public static final int defaultNice = 15;
  public Throwable exception;
  
//    TransformSuffix = unit('rm_equality:stfp+stdfof': "+rm_eq_stfp+stdfof"
//                           'rm_equality:rstfp':       "+rm_eq_rstfp"
//                           'rm_equality:stfp':        "+rm_eq_stfp"
//                           'cnf:otter':               "+cnf_otter"
//                           'otter':                   "+rm_eq_stfp"
//                           'dfg':                     "+rm_eq_rstfp"
//                           'waldmeister':             "+rm_eq_rstfp+cnf_otter"
//                          )
// %  problem+rm_eq_stfp+stdfof.in
// %   
//    GeneratedSuffix = unit(otter:   ".in"
//                           bliksem: ".blk"
//                           dfg:     ".dfg"
//                           tptp:    ".tptp"
//                           waldmeister: ".pr"
//                          )
   
  private HashMap transformSuffix;
  private HashMap systemSuffix;
    
  public TSTP2X()
  {
    transformSuffix = new HashMap();
    // for CNF problems:
    //transformSuffix.put("rm_equality:stfp+stdfof", "+rm_eq_stfp");
    transformSuffix.put("rm_equality:stfp+stdfof","+rm_eq_stfp+stdfof");
    transformSuffix.put("rm_equality:rstfp","+rm_eq_rstfp");
    transformSuffix.put("rm_equality:stfp","+rm_eq_stfp");
    transformSuffix.put("stdfof", new String[] {"+stdfof", ""});
    transformSuffix.put("add_equality:r", new String[] {"+eq_r", "+noeq"});
    transformSuffix.put("add_equality", new String[] {"+eq_r", "+noeq", "+eq_rstfp"});
    transformSuffix.put("cnf:otter","+cnf_otter");
    transformSuffix.put("cnf:otter+rm_equality:rstfp","+cnf_otter+rm_eq_rstfp");
    transformSuffix.put("otter","+rm_eq_stfp");
    transformSuffix.put("dfg","+rm_eq_rstfp");
    transformSuffix.put("waldmeister","+rm_eq_rstfp+cnf_otter");
    transformSuffix.put("none","");
    
    systemSuffix = new HashMap();
    systemSuffix.put("otter",".in");
    systemSuffix.put("mace4",".in");
    systemSuffix.put("prover9",".in");
    systemSuffix.put("\"otter:hypothesis:set(auto),clear(print_given)\"", ".in");
    systemSuffix.put("bliksem",".blk");
    systemSuffix.put("dfg",".dfg");
    systemSuffix.put("tptp",".tptp");
    systemSuffix.put("oldtptp",".oldtptp"); 
    systemSuffix.put("protein",".tme");
    systemSuffix.put("sem",".sem");
    systemSuffix.put("sem:all:1",".sem");
    systemSuffix.put("waldmeister",".pr");
//     System.out.println("[TPTP2X] "+tptp2xExecutable);
//     System.out.println("[TPTP2X] "+tptp_envs);
//     System.out.println("[TPTP2X] "+logFileName);

    logger.setLevel(Level.ALL);
    logger.setLevel(Level.OFF);
    logger.removeAllAppenders();
    logger.addAppender(appender);
  }

  public String tstp2X(String problem, String toSyntax, String transform) 
  {
    String outputFileName = null;
    File inputFile = null;
    File outputFile = null;
    try{
      mkTempDir("tptp2x");
      String tempDirPath = tempDir.getAbsolutePath();
      String inputFileName = tempDirPath + sep + "problem.p";
      inputFile = new File(inputFileName);
      //       inputFile = File.createTempFile("problem", ".tptp"); 
      //       String tempDirPath = inputFile.getParent();
      //       String inputFileName = inputFile.getAbsolutePath();
      
      this.writeFile(inputFile,problem.trim());
      String command = tptp2xExecutable + " -f " +  toSyntax + " -t " + transform + " -d " + tempDirPath + " " + inputFileName + " > " + msHome + sep + "logs" + sep + "tptp2x_prolog.out"; // + " > /dev/null";
      //      String command = tptp2xExecutable + " -f " +  toSyntax + " -t " + transform + " -d " + tempDirPath + " " + inputFileName; // + " > /dev/null";
      logger.debug("[TPTP2X] starting tptp2X...\n" + command);
      //      System.out.println("[TPTP2X] starting tptp2X...\n" + command);
      String[] commands = {"sh", "-c", command};
      this.exec(commands, tptp_envs, false, true);
      //this.exec(command, tptp_envs, true, true);
      //this.start(command, tptp_envs, defaultNice, 20);
      
      Object suffix1 = transformSuffix.get(transform);
      String suffix2 = (String) systemSuffix.get(toSyntax);
      
      if (suffix1 instanceof String[]){
        String[] suffixes = (String[]) suffix1;
        for(int i=0; i< suffixes.length; i++){
          logger.debug("[TPTP2X] suffix: " + suffixes[i]); 
          logger.debug("[TPTP2X] suffix2: " + suffix2); 
          outputFileName = tempDirPath + sep + "problem" + suffixes[i] + suffix2;
          outputFile = new File(outputFileName);
          logger.debug("[TPTP2X] reading possible output file " + outputFileName);
          if (outputFile.exists()){
            logger.debug("[TPTP2X] reading possible output file " + outputFileName);
            String result =  this.readFile(outputFile);
            logger.debug("[TPTP2X] result: " + result); 
            return result;
          } 
        }
        return null;
      } else {
        outputFileName = tempDirPath + sep + "problem" + suffix1 +suffix2;
        outputFile = new File(outputFileName);
        logger.debug("[TPTP2X] reading output file " + outputFileName);
        String result =  this.readFile(outputFile);
        logger.debug("[TPTP2X] result: " + result); 
        return result;
      }
    } catch(Exception ex){
      logger.debug("[TSTP2X] Exception " + ex);
      return null;
    } finally {
      if (inputFile !=null) { inputFile.delete();  }
      if (outputFile !=null){ outputFile.delete();  }
      removeTempDir();
    }
  }

  public File tstp2X(File inputFile, String toSyntax, String transform) 
  {
    String outputFileName = null;
    //    File inputFile = null;
    File outputFile = null;
    try{
      String tempDirPath = System.getProperty("java.io.tmpdir");
      
      String inputFilePath = inputFile.getAbsolutePath();
      String inputFileName = inputFile.getName();
      
      int dotIndex = inputFileName.lastIndexOf(".");
      if (dotIndex > 0) 
        inputFileName = inputFileName.substring(0, dotIndex);
      
      logger.debug("name " + inputFileName);
      logger.debug("index1 " + dotIndex);

      //tempDirPath + sep + "problem.tptp";
      //inputFile = new File(inputFilePath);
      //       inputFile = File.createTempFile("problem", ".tptp"); 
      //       String tempDirPath = inputFile.getParent();
      //       String inputFilePath = inputFile.getAbsolutePath();
      
      //      this.writeFile(inputFile,problem.trim());
      String command = tptp2xExecutable + " -f " +  toSyntax + " -t " + transform + " -d " + tempDirPath + " " + inputFilePath  + " > " + msHome + sep + "logs" + sep + "tptp2x_prolog.out";
      logger.debug("[TPTP2X] starting tptp2X...\n" + command);
      //      System.out.println("[TPTP2X] starting tptp2X...\n" + command);
      String[] commands = {"sh", "-c", command};
      this.exec(commands, tptp_envs, false, true);
      //this.exec(command, tptp_envs, true, true);
      //this.start(command, tptp_envs, defaultNice, 20);
      
      Object suffix1 = transformSuffix.get(transform);
      String suffix2 = (String) systemSuffix.get(toSyntax);
      
      if (suffix1 instanceof String[]){
        String[] suffixes = (String[]) suffix1;
        for(int i=0; i< suffixes.length; i++){
          logger.debug("[TPTP2X] suffix: " + suffixes[i]); 
          logger.debug("[TPTP2X] suffix2: " + suffix2); 
          outputFileName = tempDirPath + sep + inputFileName + suffixes[i] + suffix2;
          outputFile = new File(outputFileName);
          if (outputFile.exists()){
            return outputFile;
          } else 
            logger.info(" file does not exist: " +outputFile.getAbsolutePath());
        }
        return null;
      } else {
        outputFileName = tempDirPath + sep + inputFileName + suffix1 +suffix2;
        logger.debug("[TPTP2X] new file : " + outputFileName); 
        outputFile = new File(outputFileName);
        if (outputFile.exists()){
          return outputFile;
        } else {
          logger.debug("file not found : " + outputFileName); 
          return null;
        }
      }
      
    } catch(Exception ex){
      logger.debug("[tstp2xtstpx] Exception " + ex);
      return null;
    }
  }
  
  
  /*
    ../../swi-prolog/bin/pl -s tstp2xtstp.swi -g "tstp_to_xtstp('/home/jzimmer/full.tstp', '/home/jzimmer/test.xml')." -t "halt."  
  */
  public String tstp2XTstp(String tstpProblem) 
  {
    File inputFile = null;
    File outputFile = null;
    try{
      inputFile = File.createTempFile("tstp2xtstp", ".tstp");
      outputFile = File.createTempFile("tstp2xtstp", ".xtstp");
      
      writeFile(inputFile, tstpProblem);
      /* Hack: return the original problem if it's too big to be translated with SWIProlog */
      if (tstpProblem.length() > 80000) return tstpProblem;

      /* translate TSTP to XTSTP with SWIProlog program */
      String command = swiExecutable + " -G256m -s " + tstp2xtstpSWI + " -g \"tstp_to_xtstp('" + inputFile.getAbsolutePath() + "', '"+
        outputFile.getAbsolutePath() + "').\" -t \"halt.\" > /dev/null";
      logger.debug("[tstp2xtstp] command: " + command);
      String[] commands = {"sh", "-c", command};
      logger.debug("[tstp2xtstp] starting translation...");
      this.start(command, tptp_envs, defaultNice, 5);
      logger.debug("reading result..." + outputFile.getAbsolutePath());
      String result = this.readFile(outputFile);
      return result;
    } catch(Exception ex){
      logger.debug("[tstp2xtstpx] Exception " + ex);
      return null;
    } finally{
      try{ 
        if (inputFile != null) inputFile.delete();
        if (outputFile != null) outputFile.delete();
      } catch(Exception ex){ logger.debug("[tstp2xtstp] Exception " + ex); }
    }
  }
  
//   private class MyListener implements ErrorListener{
//     public void error(TransformerException exception)
//     {
//       logger.debug(exception);
//     }
//     public void fatalError(TransformerException exception)
//     {
//       logger.fatal(exception);
//     }
    
//     public void warning(TransformerException exception)
//     {
//       logger.info(exception);
//     }
//   }
//   public String tstp2XTstp(String tstpProblem) 
//   {
//     try{
//       StringReader reader = new StringReader(tstpProblem);
//       StringWriter writer = new StringWriter();
//       TSTP2XMLReader tstpReader = new TSTP2XMLReader();
//       MyListener listener = new MyListener();
//       tstpReader.tstp2XML(reader, writer, listener);    
//       return writer.toString();
//     } catch(Exception e){
//       e.printStackTrace();
//       return "Something went wrong with the tstp2XTstp transformation! "+ e;
//     }
//   }
  
    
  public String tstp2Post(String tstpProblem) throws TransformerException
  {
    //  System.out.println("[tstp2Post] XTSTP " + tstpProblem);
    writeFile(msHome + "/etc/data/tstp/tstp2x-tstp"+(++counter), tstpProblem);
    
    String tstpXml = tstp2XTstp(tstpProblem);
    //    System.out.println("[tstp2Post] XTSTP " + tstpXml);
    writeFile(msHome + "/etc/data/tstp/tstp2x-xtstp"+counter, tstpXml);    
    String tptp2PostXslStr = readFile(tptp2PostXslFile);
    StringReader tptp2PostXsl = new StringReader(tptp2PostXslStr);

    TransformerFactory tFactory = TransformerFactory.newInstance();
    transformer = tFactory.newTransformer(new StreamSource(tptp2PostXsl));
    StringWriter  writer = new StringWriter();
    transformer.transform(new StreamSource(new StringReader(tstpXml)), 
                          new StreamResult(writer));
    writeFile(msHome + "/etc/data/tstp/tstp2x-post"+counter, writer.toString());
    return writer.toString();
  }

  public String tstp2OMDoc(String tstpProblem) throws TransformerException
  {
    String tstpXml = tstp2XTstp(tstpProblem);
    logger.debug("[TPTP2OMDoc] XTSTP " + tstpXml);

    String tptp2OMDocXslStr = readFile(tptp2OMDocXslFile);
    StringReader tptp2OMDocXsl = new StringReader(tptp2OMDocXslStr);

    TransformerFactory tFactory = TransformerFactory.newInstance();
    transformer = tFactory.newTransformer(new StreamSource(tptp2OMDocXsl));
    StringWriter  writer = new StringWriter();
    transformer.transform(new StreamSource(new StringReader(tstpXml)), 
                          new StreamResult(writer));
    return writer.toString();
  }

  public void test(){
    String msHome = System.getProperty("mathserv.home");
    logger.info("msHome: " +  msHome ); 
    String fof;
    File fofFile = null;
    try{
      //      fof  = readFile(msHome + "/examples/owl/tptp/agatha-fof.tptp");
      fof  = readFile(msHome + "/examples/tptp/agatha-fof.tptp");
      fofFile = new File(msHome + "/examples/tptp/agatha-fof.tptp");
      fofFile = new File("/tmp/problem65149.p");
      //fof  = readFile(msHome + "/thirdParty/TPTPWorld/TPTP/Axioms/SET005+0.ax");
      //      fof  = readFile(msHome + "/thirdParty/TPTPWorld/TPTP/Axioms/GRP003+0.ax");
      //fof  = readFile(msHome + "/thirdParty/TPTPWorld/TPTP/Axioms/SET006+0.ax");
      //      fof  = readFile(msHome + "/thirdParty/TPTPWorld/TPTP/Axioms/HAL001+0.ax");
      //String n = tstp2X(fof, "\"otter:hypothesis:set(auto),clear(print_given)\"",
      //                        "add_equality:r");

      //       String n = tstp2X(fof, "otter",
//                         "add_equality:r");
      //  String n = tstp2X(fof, "tptp", "cnf:otter");
      String toSyntax= "dfg";
      String transform = "rm_equality:rstfp";
      String newProb;
      File newProbFile = null;
      
      logger.info("old problem: " + fofFile );
      newProbFile = tstp2X(fofFile, toSyntax, transform);
      newProb = readFile(newProbFile);
      logger.info("new problem: " + newProb );
      
//       String n = tstp2OMDoc(fof);
      
//       System.out.println("[TPTP2X]\n  " + n);
      //     } catch(Exception e){
      //       e.printStackTrace();
      //     }
      //     String ts = readFile(msHome + "/examples/owl/tptp/agatha-fof.tptp");
      //     try{
      //      String proof  = readFile(msHome + "/src/org/mathweb/atp/proof");
      //       System.out.println("proof\n " + proof);
      //       String tstpXml = tstp2XTstp(proof);
      //       System.out.println("xml " + tstpXml);
      //       System.out.println("[TPTP2OMDoc] omdoc " + omdoc);
      //       writeFile(msHome + "/examples/owl/omdoc/agatha-fof.omdoc", omdoc);
      //     } catch(Exception e){
      //       e.printStackTrace();
      //     }
      //       fof = readFile(msHome + "/examples/owl/fof2.owl");
      //       String tstpXml = tstp2XTstp(fof);
      //       System.out.println("[TPTP2Post] xml " + tstpXml);
      //     try{
//       String xml = tstp2XTstp(fof);
//       System.out.println("[TPTP2Post] xml " + xml );
//       String post = tstp2Post(fof);
//       String omdoc = tstp2OMDoc(fof);
      //      System.out.println("[TPTP2Post] post " + post );
      //            System.out.println("[TPTP2omdoc] omdoc " + omdoc );
      } catch(Exception e){
      e.printStackTrace();
    }
  }
  
  
  public static void main(String argv[]) throws Exception {
    String msHome = System.getProperty("mathserv.home");
//     String prob = "include('Axioms/EQU001+0.ax').\n"+
//       "include('Axioms/GRP004+0.ax').\n"+
//       "input_formula(conjecture118,conjecture,(\n"+
//       "! [B,C,D] :\n"+
//       "((equal(inverse(B),C) & equal(multiply(C,B),D) )\n"+
//       "<=> (equal(multiply(B,C),D) & equal(multiply(C,B),D) &\n"+
//       "                           equal(inverse(C),B))))).\n";
    
//     System.out.println("[TPTP2X] old " + prob);
    TSTP2X t = new TSTP2X();
    t.test();
    //     String n = t.tstp2X(prob, "tptp", "cnf:otter");
//     System.out.println("[TPTP2X] new " + n);
  }
  
}


/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

