// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

//   jade -gui atp:org.mathweb.atp.ATPAgent dummy0:jade.tools.DummyAgent.DummyAgent

package org.mathweb.atp;

import java.util.HashMap;
import java.io.*;
import javax.xml.transform.Transformer;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

//import org.tptp.xtstp.TSTP2XMLReader;
import org.mathweb.core.BasicService;

/**
   @Author Juergen Zimmer University of Edinburgh
   @version $Date: 2006/07/10 12:05:29 $ $Revision: 1.26 $
*/

public class X2TSTP extends BasicService {
  
  static boolean verbose=true;
  
  private static String sep = File.separator;
  public String TPTP_HOME = getSystemProperty("tptp.home.dir", "TPTP.HOME.DIR");
  public String[] tptp_envs = new String[] {"TPTP_HOME="+TPTP_HOME, "TPTPDirectory="+TPTP_HOME}; 
  private String x2tstpPath = TPTP_HOME + sep + "TSTP" + sep + "X2TPTP";
  private String x2tstpExecutable = findSystemExecutable(x2tstpPath, "X2tptpExtractor", "X2tptpExtractor.exe");
  public String omdoc2TstpXslFile = getSystemProperty("omdoc.tstp.xsl", "OMDOC_TSTP_XSL");

  private static Logger logger = Logger.getLogger("X2TSTP");
  private static String logFileName = getSystemProperty("x2tstp.log.file", "X2TSTP_LOG_FILE");
  private static FileAppender appender = newAppender(logFileName);

  private static final int defaultNice = 15;
  private Transformer transformer;
  private Throwable exception;
  
  public X2TSTP()
  {
    super();
    logger.setLevel(Level.ALL);
    logger.setLevel(Level.OFF);
    if (!logger.getAllAppenders().hasMoreElements()) logger.addAppender(appender);
  }
  
  private class MyListener implements ErrorListener{
    public void error(TransformerException exception)
    {
      logger.debug(exception);
    }
    public void fatalError(TransformerException exception)
    {
      logger.fatal(exception);
    }
    
    public void warning(TransformerException exception)
    {
      logger.info(exception);
    }
  }

  public String x2tstp(String output, String format )
  {
    File inputFile = null;
    File outputFile = null;
    try{
      inputFile = File.createTempFile("x2tstp", ".tstp");
      outputFile = File.createTempFile("x2tstp", ".out");
      String result;
      
      this.writeFile(inputFile, output);
      
      //       X2tstpExtractor Otter---3.2 < Otter---3.2-Raw | more
      String command = x2tstpExecutable + " " +  format +
        " < " + inputFile.getAbsolutePath() + " > " + outputFile.getAbsolutePath();
      String[] commands = {"sh", "-c", command};
      logger.info("[X2tstp] starting...\n" + command);      
      this.exec(commands, tptp_envs, false, true);
      //this.start(command, tptp_envs, defaultNice, 20);
      logger.info("[X2TSTP] reading file..." + outputFile.getAbsolutePath());
      result =  this.readFile(outputFile);
      //      System.out.println("[X2TSTP] ...done");
      return result;
    }
    catch(Exception e){
      e.printStackTrace();
      return output;
    } finally{
      try{ 
        if (inputFile != null) inputFile.delete();
        if (outputFile != null) outputFile.delete();
      } catch(Exception e){ logger.debug("[X2TSTP] Exception " + e); }
    }
  }

//   public String x2tstpXML(String output, String format )
//   {
//     String tstp = x2tstp(output, format);
//     String xml;
//     StringReader reader = new StringReader(tstp);
//     StringWriter writer = new StringWriter(20000);
//     TSTP2XMLReader tstpReader=new TSTP2XMLReader();
//     MyListener listener = new MyListener();
    
//     try {
//       tstpReader.tstp2XML(reader, writer,listener);    
//       //      System.out.println("result: " + writer.toString());
//       return writer.toString();
//     }
//     catch(Exception e){
//       e.printStackTrace();
//       return null;
//     }
//   }

  public String omdoc2TSTP(String omdocProblem) 
  {
    try{
      String omdoc2TstpXslStr = readFile(omdoc2TstpXslFile);
      StringReader omdoc2TstpXsl = new StringReader(omdoc2TstpXslStr);
      
      TransformerFactory tFactory = TransformerFactory.newInstance();
      transformer = tFactory.newTransformer(new StreamSource(omdoc2TstpXsl));
      StringWriter  writer = new StringWriter();
      transformer.transform(new StreamSource(new StringReader(omdocProblem)), 
                            new StreamResult(writer));
      return writer.toString();
    }
    catch(TransformerException ex){
      return ex.getMessageAndLocation();
    }
  }
  
  
  public static void main(String argv[]) throws Exception {
    String msHome = System.getProperty("mathserv.home");
    X2TSTP t = new X2TSTP();

//     String prob = t.readFile(msHome + "/thirdParty/TPTPWorld/TSTP/X2TPTP/Otter---3.2-Raw");
//     System.out.println("[X2TSTP] prob : " + prob);
//     String n = t.x2tstp(prob, "Otter---3.3");
//     System.out.println("[X2TSTP] result: " + n); 
//    String prob = t.readFile(msHome + "/examples/omdoc/core3.omdoc");
//    String prob = t.readFile(msHome + "/examples/omdoc/foo.xml");
//    String prob = t.readFile("/home/jzimmer/m.omdoc");
    String prob = t.readFile("/home/jzimmer/fail.omdoc");
    String n = t.omdoc2TSTP(prob);
    System.out.println(n); 
//    System.out.println("[X2TSTP] XML: " + n);
//     String prob = t.readFile(msHome + "/etc/data/omdoc/a.omdoc");
//     String n = t.omdoc2TSTP(prob);
    
  }
  
}
