// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/** For testing:
*/
package org.mathweb.broker;

import java.io.*;
import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.util.HashSet;
import java.util.Set;

import java.util.Enumeration;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

import org.jdom.input.*;
import org.jdom.output.*;

import salvo.jesus.graph.*;

//import monet.openmath.lang.*;
//import monet.openmath.lang.*;

import org.mathweb.core.BasicService;
import org.mathweb.core.OMUtils;

import org.mathweb.onto.*;

// import org.mathweb.broker.ServiceRegistry;
// import org.mathweb.broker.ServicePlanner;
// import org.mathweb.broker.BrokerGUI;
// import org.mathweb.broker.GuiEvent;

import org.mathweb.atp.TPTPClassifier;

import org.mathweb.serv.ReasoningService;
import org.mathweb.serv.ReasoningProblem;
import org.mathweb.serv.Query;
import org.mathweb.serv.Parameter;
import org.mathweb.serv.Input;
import org.mathweb.serv.PreCondition;

import org.mathweb.owls.OWLSParser;

/**
   @author Juergen Zimmer, Saarland University, Germany
   @version $Date: 2006/08/24 13:41:18 $ $Revision: 1.54 $
*/
public class Broker extends BasicService {

  private static Logger logger = Logger.getLogger("Broker");
  private String logFileName = getSystemProperty("broker.log.file", "BROKER_LOG_FILE");
  public  String SWI_HOME =  getSystemProperty("broker.swi.prolog", "BROKER_SWI_PROLOG");
  public  String[] swi_envs = new String[] {"SWI_HOME="+SWI_HOME};
  public  String swiExecutable = findSystemExecutable(SWI_HOME, "pl", "pl.exe");
  public  String GOLOG_HOME = getSystemProperty("broker.golog.dir", "BROKER_GOLOG_DIR");

  private String GRAPHVIZ_HOME = getSystemProperty("graphviz.home.dir", "BROKER_LOG_FILE");
  private String dotExecutable  = findSystemExecutable(GRAPHVIZ_HOME + sep + "bin", "dot", "Dot.exe");
  //  private String brokerEmptyImage = getSystemProperty("broker.empty.image", "BROKER_EMPTY_IMAGE");
  
  private ServiceRegistry serviceRegistry;
  //  private ServiceMatcher serviceMatcher = new ServiceMatcher();
  private ServicePlanner servicePlanner = new ServicePlanner();
  private GologExecutor gologExecutor = new GologExecutor();
  private Query currentQuery = null;

  private ServiceExecutor executor = new ServiceExecutor();
  private List executionPlans = new ArrayList();
  private ArrayList executionGraphs = new ArrayList();
  private ArrayList executionImages = new ArrayList();
  private int executionPlanIndex = -1;
  
  /**  protected Ontology brokerOntology = BrokerOntology.getInstance(); */
  protected OWLJDom owlJDom;

  private OWLSParser owlsParser = new OWLSParser();
  private TPTPClassifier tptpClassifier = new TPTPClassifier();
  private BrokerGUI gui = null;
  //  private OMUtils omUtils = new OMUtils();
 
  /** GuiEvent to start the agent activity **/
  final static int COMP_PLANS = 2;
  final static int PREV_PLAN = 3;
  final static int NEXT_PLAN = 4;
  final static int EXEC_PLAN = 5;
  final static int COMP_POLICY = 6;

  //  protected Ontology brokerOntology = BrokerOntology.getInstance();
  
  public Broker(boolean withGui) {
    owlJDom = OWLJDom.getInstance();
    serviceRegistry = new ServiceRegistry();

    serviceRegistry.loadDefaultOwlsServices(owlsParser);

    if (withGui){
      gui = new BrokerGUI(this, "MathServe Broker GUI");
      gui.setVisible(true);
      showServices(serviceRegistry.allNames());
    }
    
    logger.setLevel(Level.ALL);
    //    logger.setLevel(Level.OFF);
    FileAppender appender = null;
    try {
      appender = new FileAppender(new SimpleLayout(), logFileName, false);
    } catch(Exception e) {}
    logger.addAppender(appender);

  }

  public Broker()
  {
    this(false);
  }
  
  public String registerService(String description)  {
    try { 
      logger.info(" descr: "+description);
      ReasoningService service = owlsParser.parseService(description);
      logger.info(" parsed!!!");
      return this.registerService(service);
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return owlJDom.createFailure(null, System.currentTimeMillis(), cpuTime, "Broker could not interpret service!");
    }
  }

  public String deregisterService(String serviceName)  {
    try { 
      serviceRegistry.remove(serviceName);
      return owlJDom.createConfirmation("De-registration succeeded!");
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return owlJDom.createFailure(serviceName, System.currentTimeMillis(), cpuTime, "Broker could not interpret service!");
    }
  }

  protected String registerService(ReasoningService service)  {
    if (service.getName() == null){
      logger.info("The service has no name!!!");
    }
    if (serviceRegistry.put(service)){
      showServices(serviceRegistry.allNames());
      return owlJDom.createConfirmation("Registration succeeded!");
    } else {
      long cpuTime = 0;
      return owlJDom.createFailure(service.getName(), System.currentTimeMillis(), cpuTime, "Registration did NOT succeed!");
    }
  }
  
  public String brokerService(String queryXML, int time)  {
    Query query = owlsParser.parseAbstractQuery(queryXML);
    System.out.println(query);
    computePlans(query, -1);
    
    if (executionPlans.size() > 0){
      String policy = computeOptimalPolicy(query);
      String result = "";
      result = gologExecutor.executeProgram(policy, query, serviceRegistry, 10, false, null, false);
      return result;
    } else 
      return owlJDom.createFailure("Could not broker any atomic service for query " + query.uri);
  }
  
  public String recommendAtomicService(String queryXML, String categoriesStr)  {
    Query query = owlsParser.parseAbstractQuery(queryXML);
    //    System.out.println(query);
    computePlans(query, -1);
    
    if (! executionPlans.isEmpty()){
      String policy = computeOptimalPolicy(query);
      String[] reportCats = categoriesStr.split(" ");
      for(int i=0; i< reportCats.length; i++)
        reportCats[i] = reportCats[i].trim();
      
      String result = "";
      result  = gologExecutor.executeProgram(policy, query, serviceRegistry, 10, false, reportCats, true);
      return result;
    } else 
      return owlJDom.createFailure("Could not broker any atomic service for query " + query.uri);
  }

  public String findAtomicServices(String queryXML)  {
    Query query = owlsParser.parseQuery(queryXML);
    System.out.println(query);
    computePlans(query, 1);
    StringBuffer result = new StringBuffer();
    
    if (! executionPlans.isEmpty()){
      for (int i=0; i<executionPlans.size(); i++){        
        ServicePlan sp = (ServicePlan) executionPlans.get(i);
        result.append(servicePlanner.plan2ServiceURIs(sp));
        
        if (i < executionPlans.size() -1)
          result.append(", ");
      }
      return result.toString();
    } else 
      return owlJDom.createFailure("The MathServe broker could not any atomic service matching the query " + query.uri);
  }
  
  
  protected String computePlans(String description, int maxLength)  {
    try{    
      if (gui != null) gui.setQuery(description);
      Query query = owlsParser.parseQuery(description);
      return computePlans(query, maxLength);
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return owlJDom.createFailure("foo", System.currentTimeMillis(), cpuTime, "Broker could not interpret query description!");
    }
  }
  
  /** 
   * @param query an OWL-S query profile as a Query object.
   * @param maxLength the maximum length of the plans computed, if null no limit on the length is set.
   * @return A string naming the number of plans found. 
   */
  protected String computePlans(Query query, int maxLength)  {
    ReasoningProblem problem = query.getProblem();
    List inputs = problem.getInputs();
    List outputs = problem.getOutputs();
    ArrayList params = new ArrayList();
    params.addAll(inputs);
    params.addAll(outputs);
    

    try{    
      currentQuery = query;
      executionImages = new ArrayList();
      executionPlanIndex = -1;
      
      executionPlans = servicePlanner.findProdigyPlans(query, serviceRegistry, maxLength);
      servicePlanner.plansComputeGraphs(query, executionPlans);
      
      //      executionGraphs = servicePlanner.findPopPlans(query, serviceRegistry);
      
      /* throw away subsumed plans: 
       ============================ */
      List subsumed = new ArrayList();
      List duplicates = new ArrayList();
      for(int i=0; i< executionPlans.size(); i++){
        ServicePlan aPlan = (ServicePlan) executionPlans.get(i);
        DirectedGraph agraph = aPlan.getGraph();
        if (!duplicates.contains(aPlan)) 
          for(int j=0; j< executionPlans.size(); j++){
            if (i != j){
              ServicePlan bPlan = (ServicePlan) executionPlans.get(j);
              DirectedGraph bgraph = bPlan.getGraph();
              if (bgraph == null)
                duplicates.add(bPlan);
              else{
                if (servicePlanner.equal(agraph, bgraph))
                  duplicates.add(bPlan);
                else 
                  if (servicePlanner.subsumes(agraph, bgraph)){
                    //                   System.out.println("graph " + i + " subsumes graph " +j+ "?: " + 
                    //                                      subsumes);
                    subsumed.add(bPlan);
                  }
              }
            }
          }
      }
      
      executionPlans.removeAll(duplicates);
      executionPlans.removeAll(subsumed);
      
      if (executionPlans.size() >0) {
        executionPlanIndex = 0; 
        for(int l=0; l< executionPlans.size(); l++){
          ServicePlan currentPlan = (ServicePlan) executionPlans.get(l);
          DirectedGraph graph = (DirectedGraph) currentPlan.getGraph();
          executionGraphs.add(graph);
          if (gui != null) {
            BufferedImage image = graph2Image(graph, l);
            executionImages.add(image);
          }
        }
        if (gui != null)
          gui.showImage((BufferedImage) executionImages.get(executionPlanIndex));
        return "PRODIGY found " + executionPlans.size() + " plans";
      } else{
        long cpuTime = 0;
        return owlJDom.createFailure(query.uri, System.currentTimeMillis(), cpuTime, "Broker could not find any execution plans!");
      }
      
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return owlJDom.createFailure(query.uri, System.currentTimeMillis(), cpuTime, "Broker encountered a problem!\n" + e);
    }
  }
  
  protected String computeOptimalPolicy(String description)  {
    try{    
      if (gui != null) gui.setQuery(description);
      Query query = owlsParser.parseQuery(description);
      logger.info("The query: " + query);
      return computeOptimalPolicy(query);
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return owlJDom.createFailure("foo", System.currentTimeMillis(), cpuTime, "Broker could not interpret query description!");
    }
  }

  protected String computeOptimalPolicy(Query query)  {
    String result = null; 
    File swiFile = null; 
    File outFile = null; 
    ReasoningProblem problem = query.getProblem();
    List inputs = problem.getInputs();
    List outputs = problem.getOutputs();
    ArrayList params = new ArrayList();
    params.addAll(inputs);
    params.addAll(outputs);
       
    try{    
      
//       ServiceMerger merger = new ServiceMerger();
//       ArrayList merged = merger.merge(plans);
      String dtGologProgram = servicePlanner.plans2DTGologDomain(query, executionPlans, serviceRegistry);

      if (gui != null){
        gui.setGolog(dtGologProgram);
      }
      
      //System.out.println("domain : " + dtGologProgram);
      writeFile(msHome + "/etc/data/broker/dtgolog-domain.swi", dtGologProgram);
      
      String interpreter = readFile(GOLOG_HOME + sep + "interpreter.swi");
      String appendix = readFile(GOLOG_HOME + sep + "appendix.swi");
      
      swiFile = File.createTempFile("dtGolog", ".swi"); 
      outFile = File.createTempFile("policy", ".xml"); 
      
      writeFile(swiFile, interpreter + dtGologProgram + appendix);
      writeFile("/home/jzimmer/domain.swi", interpreter + dtGologProgram + appendix);
      

      String actString = query.getURI();
      String parStr = "";
      
      for(int p=0; p < params.size(); p++){
        Parameter par = (Parameter) params.get(p);
        parStr += par.getName().toUpperCase();
        if (p < params.size() - 1)
          parStr += ", ";
      }
      //ReasoningService.actionString(query.getURI(), params);
      System.out.println(actString + " " + parStr);
      
      String[] new_envs = new String[] {"SWI_HOME="+SWI_HOME, "GOLOG_PL="+swiExecutable, 
                                        "GOLOG_DOM="+swiFile.getAbsolutePath(), "GOLOG_ACT="+actString, 
                                        "GOLOG_PARS="+parStr, "GOLOG_OUT="+outFile.getAbsolutePath()};
      
      
      
      //       String command = "\"" + GOLOG_HOME + sep + "golog-bin " + swiExecutable + " " + swiFile.getAbsolutePath() + " " +
//         actString + " '" + parStr + "' " + outFile.getAbsolutePath() + "\"  > /dev/null";
      String command = GOLOG_HOME + sep + "golog-bin > /home/jzimmer/golog.out";
      
      System.out.println(command);
      String[] commands = {"sh", "-c", command};
      this.exec(commands, new_envs, true, true);
      //this.start(command, swi_envs, 15, 10);
      String policy = readFile(outFile);
      System.out.println("policy: " + policy);

      if (gui != null){
        String tidyPol = servicePlanner.gologXml2Math(policy);
        gui.setPolicy(tidyPol);
        System.out.println(policy);
        writeFile("/home/jzimmer/policy.xml", tidyPol);
      } else {
              writeFile("/home/jzimmer/policy.xml", policy);
      }
      
//       result = gologExecutor.executeProgram(policy, query, serviceRegistry, 40, false, null, false);
//       //      return policy;
      return policy;
      
    } catch(Exception e){
      e.printStackTrace();
      long cpuTime = 0;
      return null;
    } finally {
      if (swiFile != null) swiFile.delete();
      if (outFile != null) outFile.delete();
    }
  }
    
  private ReasoningProblem analyseProblem(ReasoningProblem problem){
    //     ArrayList inputs = problem.getInputs();
    //     Iterator it = inputs.iterator();
    //     Input in;
    //     ArrayList preConds = problem.getPreconds();
    
    //     while (it.hasNext()){
    //       in = (Input)it.next();
    //       logger.info("next input: " + in.toString());
      
    //       if (omUtils.equal(in.getType(), tptpOMStr)){
    //         OMString content = (OMString) in.getContent();
    //         HashSet features = tptpClassifier.classifyProblem(content.getString());
    //         Iterator featit = features.iterator();
    //         String precondStr = "";
    //         String inputName = in.getName();
    //         logger.info("problem features: " + features.toString());
    //         while (featit.hasNext()){
    //           precondStr = precondStr + featit.next() + "(" + inputName + ").\n";
    //         }
    //         OMString newOMStr = new OMString(precondStr);
    //         PreCondition newPrecond = new PreCondition(features.toString(),new OMString(precondStr));
    //         preConds.add(newPrecond);
    //         logger.info("new precond: " + newPrecond.toString());
    //       }}
    //     problem.setPreconds(preConds);
    return problem;
  }


  //   private String callService(ServiceEntry serviceEntry, ReasoningProblem problem)
  //   {
  //     try{
  //       ProvingProblem p = new ProvingProblem();
  //       TimeResource tr = new TimeResource();
  //       Logic l = new Logic();
  //       tr.setWallClockTime(60);
  //       p.setTimeResource(tr);
    
  //       String language = "tptp";
  //       Input probInput = problem.getInput("tptp");
  //       logger.info("probInput: " + probInput);
  //       logger.info("entry: " + serviceEntry.toString());
      
  //       OMString prob = (OMString) probInput.getContent();
    
  //       p.setLanguage(language);
  //       p.setFormalDescription(prob.getString());
  //       p.setNlDescription(problem.getName());
  //       p.setLogic(l);
    
  //       ACLMessage msg = new ACLMessage(ACLMessage.QUERY_IF);
  //       msg.addReceiver(serviceEntry.getAgent());
  //       msg.setLanguage(codec.getName());
  //       msg.setOntology(atpOntology.getName());
  //       getContentManager().fillContent(msg, p);
  //       send(msg);
  //       return "Query sent to service "+serviceEntry.getService().getName() + "...";
  //     } catch (CodecException e) {
  //       e.printStackTrace();
  //       return e.getMessage();
  //     }
  //     catch (OntologyException e) {
  //       e.printStackTrace();
  //       return e.getMessage();
  //     } catch (Exception e) {
  //       e.printStackTrace();
  //       return e.getMessage();
  //     }
  //   }
  
    
  public void onGuiEvent(GuiEvent ev) {
    String result;
    String queryOWLS = null;
    Query query = null;
    
    switch (ev.getType()) {
    case COMP_PLANS: 
      queryOWLS = (String)ev.getParameter(0);
      query = owlsParser.parseQuery(queryOWLS);
      logger.info("query: " + query.toString());
      result = computePlans(query, -1);
      logger.info("[Broker] result: "+ result);
      showResult(result);
      break;
    case PREV_PLAN: 
      previousGraph();
      break;
    case NEXT_PLAN: 
      nextGraph();
      break;
    case EXEC_PLAN: 
      showResult("Trying to execute current plan!");
      result = executeCurrentGraph();
      showResult(result);
      break;
    case COMP_POLICY: 
      showResult("Trying to compute optimal policy!");
      queryOWLS = (String)ev.getParameter(0);
      query = owlsParser.parseQuery(queryOWLS);
      result = computeOptimalPolicy(query);
      showResult(result);
      break;
    default:
      System.err.println("Received unexpexcted GuiEvent of type:"+ev.getType());
    }
  }
  
  public void showResult(String result)
  {
    if (gui != null) gui.setResult(result);
  }

  public void showServices(String serviceNames)
  {
    if (gui != null) gui.setServices(serviceNames);
  }
  
  public String executeCurrentGraph()
  {
    long cpuTime = 0;
    try{    
      ServicePlan currentPlan = (ServicePlan) executionPlans.get(executionPlanIndex);
      DirectedGraph currentExecutionGraph = (DirectedGraph) currentPlan.getGraph();
      if ((currentExecutionGraph != null) && (currentQuery != null))
        return executor.executeGraph(currentExecutionGraph, currentQuery, false);
      else 
        return owlJDom.createFailure(null, System.currentTimeMillis(), cpuTime, "No query or execution plan available!");
      //     } catch(java.net.ConnectException e){
      //       e.printStackTrace();
      //       return owlJDom.createFailure("Broker could not execute service:\n" + e);
    } catch(Exception e){
      e.printStackTrace();
      return owlJDom.createFailure(null, System.currentTimeMillis(), cpuTime, "Broker encountered a problem!\n" + e);
    }
  }
  
  public void nextGraph()
  {
    if ((!executionPlans.isEmpty()) && 
        (executionPlanIndex >= 0) && (executionPlanIndex < executionPlans.size() -1)){
      executionPlanIndex = executionPlanIndex + 1;
      if (gui != null) gui.showImage((BufferedImage) executionImages.get(executionPlanIndex));
    }
  }
  public void previousGraph()
  {
    if ((!executionPlans.isEmpty()) && (executionPlanIndex > 0)){
      executionPlanIndex = executionPlanIndex - 1;
      if (gui != null) gui.showImage((BufferedImage) executionImages.get(executionPlanIndex));
    }
  }
  
  
  private BufferedImage graph2Image(DirectedGraph graph, int nr)
  {
    File dotFile = null;
    File jpgFile = null;
    try{
      String dot = servicePlanner.graph2Dot(graph);
      logger.info(dot);
      writeFile(msHome + "/etc/data/graph/graph" + nr + ".dot", dot);
      
      dotFile = File.createTempFile("broker", ".dot");
      jpgFile = File.createTempFile("broker", ".jpg");
      jpgFile.deleteOnExit();
      
      writeFile(dotFile, dot);
      String command = dotExecutable + " -v -Tjpg "+dotFile.getAbsolutePath() + 
        " -o" + jpgFile.getAbsolutePath();
      logger.info("[Broker] starting dot...\n" + command);
      String[] commands = {"sh", "-c", command};
      this.exec(commands, new String[0], false, true);
      if (jpgFile.exists()) {
        BufferedImage image = ImageIO.read(jpgFile);
        dotFile.delete();
        //         FileOutputStream jpgOutFile = null;
        //         try {
        //           jpgOutFile = new FileOutputStream(msHome + "/etc/data/graph/graph" + nr + ".jpg");
        //           ImageIO.write(image, "JPG", jpgOutFile);
        //           jpgOutFile.close();
        //         } catch (IOException ioe) {
        //         } finally {
        //           if (jpgOutFile != null)
        //             try{ jpgOutFile.close(); } catch (Exception e) {}
        //         }
        
        return image;
      }
    } catch(Exception e){
      logger.debug("[Broker] Exception " + e);
    } finally{
      try{ 
        if (dotFile != null) dotFile.delete();
        //        if (jpgFile != null) jpgFile.delete();
      } catch(Exception e){ logger.debug("[Otter] Exception " + e); }
    }
    return null;
  }
  
  public void shutdown()
  {
    //    serviceRegistry.shutdownLocal();
    return;
  }
  
  public static void main(String argv[]) throws Exception {
    System.out.println(System.getProperty("java.endorsed.dirs"));
    String msHome = System.getProperty("mathserv.home");
    boolean gui = ((argv.length > 0) && (argv[0].equals("gui")));
    Broker broker = new Broker(gui);
    OWLSParser c = new OWLSParser();
    
    //     if (!gui){
    //       System.out.println("query");
    //       System.out.println("broker answer: " + answer);
//     }
    
//    if (!gui){
      // //    Query query = c.parseQueryFile(msHome + "/examples/queries/owls/result-query1.owl");
      // //    String answer = broker.computeOptimalPolicy(query);
//       String q;
//       broker.computePlans(q);
      

    String queryXML = ""; 
    String result = "";
    queryXML = broker.readFile(msHome + "/examples/queries/owls/1.1/result-query1.owl");
    // queryXML = broker.readFile(msHome + "/examples/queries/owls/1.1/nd-query1.owl");
    // queryXML = broker.readFile(msHome + "/examples/queries/owls/1.1/result-query1.owl");
    // queryXML = broker.readFile(msHome + "/examples/queries/owls/1.1/anyResult.owl");
//     Query query = c.parseQuery(queryXML);
//     broker.computePlans(query);
//     result = broker.computeOptimalPolicy(query);
    //       //      System.out.println("XML: " + queryXML);
    //       result = broker.brokerService(queryXML, 20);
    result = broker.findAtomicServices(queryXML);
    System.out.println(result);
      
//    }
    
      //     String res = broker.executeCurrentGraph();
    
  }
  
} // end Broker.java

      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

