// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/** For testing:

runt org.mathweb.broker.BrokerClient lambda.ags.uni-sb.de 8080 proveProblemOpt ${MATHSERV_HOME}/thirdParty/TPTPWorld/TPTP/Problems/PUZ/PUZ001+1.p 10

runt org.mathweb.broker.BrokerClient cluster.ags.uni-sb.de 8080 proveProblemOpt ${MATHSERV_HOME}/thirdParty/TPTPWorld/TPTP/Problems/PUZ/PUZ001+1.p 10

runt org.mathweb.broker.BrokerClient lambda.ags.uni-sb.de 8080 proveOmdocProblem /home/jzimmer/m.omdoc 10

runt org.mathweb.broker.BrokerClient cluster.ags.uni-sb.de 8080 proveOmdocProblem /home/jzimmer/serge.omdoc 10

runt org.mathweb.broker.BrokerClient localhost 8080 proveProblemOpt /home/jzimmer/import/MathServe/thirdParty/TPTPWorld/TPTP/Problems/PUZ/PUZ001+1.p 10

*/
package org.mathweb.broker;

import java.io.*;
import java.util.*;
import javax.xml.rpc.ParameterMode;

import sun.misc.Signal;
import sun.misc.SignalHandler;

import org.jdom.Element;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
import org.apache.axis.configuration.FileProvider;

import org.mathweb.onto.OWLJDom;

/**
   @author Juergen Zimmer, Saarland University, Germany
   @version $Date: 2006/08/24 13:41:18 $ $Revision: 1.13 $
*/
public class BrokerClient {
  
  static private int delay = 0;
  private static OWLJDom owlJDom = OWLJDom.getInstance();    
  protected static String msHome = System.getProperty("mathserv.home");
  
  static class MyHandler implements SignalHandler {
    
    public void handle(Signal sig) {
      return;
    }
  }

  
  /** 
   * Reads the contens of a file in a java.lang.String.
   * @param fileName the name of the file to read.
   * @return The contents of the file.
   */
  public String readFile(String fileName){
    FileInputStream isFile  = null;
    try {
      isFile = new FileInputStream(fileName);
      int nChar;
      StringBuffer sbRead = new StringBuffer();
      nChar = isFile.read();
      while (nChar > -1) {
        sbRead.append((char) nChar);
        nChar = isFile.read();
      }
      isFile.close();
      return sbRead.toString();
    } catch (IOException ioe) {
      return null;
    } finally {
      if (isFile != null)
        try{ isFile.close(); } catch (Exception e) {}
    }
  }
  

  
  /** 
   * Writes a string into a file
   * @param fileName the name of the file to write into.
   * @param content the string to be written.
   * @return True if the file as written successfully, false otherwise.
   */
  protected boolean writeFile(String fileName, String content){
    FileOutputStream outFile = null;
    try {
      outFile = new FileOutputStream(fileName) ;
      outFile.write(content.getBytes());
      outFile.close();
      return true;
    } catch (IOException ioe) {
      return false;
    } finally {
      if (outFile != null)
        try{ outFile.close(); } catch (Exception e) {}
    }
  }

  public void waitMillis(long millis)
  {
    long t1 = System.currentTimeMillis();
    while ((System.currentTimeMillis() - t1) < millis);
  }

  public String callMS(String endpoint, String problemFileName, String method, int time){
      String[] parts = problemFileName.split("/");
      String probName = "unknown";
      if(parts.length > 1)
        probName = parts[parts.length - 1].trim();
              
      String data =  probName + " \t 0 \t 0 \t Unknown \n";
      try {
        if (! ((method.equals("proveProblemOpt")) || (method.equals("proveProblemSplit")) || (method.equals("proveOmdocProblem")) ||
               (method.equals("proveProblemSplitJoin")) || (method.equals("proveProblemChoice"))))
          throw new Exception("The method " + method + " is not valid!");
        String problem = readFile(problemFileName);
        
        FileProvider config = new FileProvider(msHome + "/conf/client-config.wsdd");
        Service  service = new Service(config);
        Call     call    = (Call) service.createCall();
        
        call.setTargetEndpointAddress(new java.net.URL(endpoint));
        call.setOperationName( method );
        call.addParameter("in0", XMLType.SOAP_STRING, ParameterMode.IN);
        call.addParameter("in1", XMLType.SOAP_INT, ParameterMode.IN);
        call.setReturnType(XMLType.SOAP_STRING);
        call.setTimeout(new Integer(600*1000));
        
        String result = (String) call.invoke( new Object [] {problem, new Integer(time - delay) });
        
        System.out.println(result);
        Element resultRoot = owlJDom.getRootElement(result);
        if (! resultRoot.getChildren("Failure", OWLJDom.MW_NS).isEmpty())
          return result;
        else {
          Double wcDouble = (Double) OWLJDom.TIME_RESOURCE_WC.numberValueOf(resultRoot);
          Double cpuDouble = (Double) OWLJDom.TIME_RESOURCE_CPU.numberValueOf(resultRoot);
          String status = OWLJDom.STATUS.valueOf(resultRoot);
          String rdfID = OWLJDom.POTENTIAL_ID.valueOf(resultRoot);
          if ((status == null) || (status.equals(""))) status = "Unknown";
          else {
            String[] split = status.split("#");
            if (split.length > 1) status = split[1];
          }
          data =  probName + " \t" + cpuDouble.intValue() + " \t" + wcDouble.intValue() + 
            " \t" + rdfID + "  \t" +  status + "\n";
          
          if ((status.equals("Unsatisfiable")) || (status.equals("Theorem")))
            try{
              Element proof = (Element) OWLJDom.PROOF_FORMAL_PROOFS.selectSingleNode(resultRoot);
              if (proof != null){
                String tstpFileName = msHome + "/etc/data/tstp/"+ rdfID + ".tstp";
                writeFile(tstpFileName, proof.getTextTrim());
              }  
            } catch(Exception e){
            }
          else if ((status.equals("CounterSatisfiable")) || (status.equals("Satisfiable")))
            try{
              Element proof = (Element) OWLJDom.PROOF_FORMAL_PROOFS.selectSingleNode(resultRoot);
              if (proof != null){
                String tstpFileName = msHome + "/etc/data/tstp/"+ rdfID + ".tstp";
                writeFile(tstpFileName, proof.getTextTrim());
              }  
            } catch(Exception e){
            }
        }
      } catch(Exception e){
        e.printStackTrace();
          System.out.println("FAILURE");
          //        System.exit(1);
      }
      System.out.println(data);
      return data;
  }

  public static void main(String[] args) {
    if (args.length < 5) {
      System.out.println("Usage: BrokerClient <hostname> <port> <method> <TSTP input file> <time limit> [&proxy host &proxy port]");
      System.exit(1);
    } else {
      if (args.length > 6){
        String proxyHost = args[5];
        String proxyPort = args[6];
        if ((!proxyHost.equals("")) && (!proxyPort.equals(""))){
          System.out.println("using proxy: " + proxyHost + ":" + proxyPort);
          System.setProperty("http.proxyHost", proxyHost); 
          System.setProperty("http.proxyPort", proxyPort); 
        }}
      BrokerClient c = new BrokerClient();
      Signal.handle(new Signal("INT") , new MyHandler());
      Signal.handle(new Signal("TERM") , new MyHandler());
      
      String endpoint = "http://" + args[0] + ":" + args[1] + "/axis/services/Broker";
      String method = args[2];
      Integer timeInt = new Integer(args[4]);
      int time = timeInt.intValue();
      
      String problemFileName = args[3];
      c.callMS(endpoint, problemFileName, method, time);
      System.exit(0);
    }
  }
} // end BrokerClient.java

/** 
 java -Dhttp.proxyHost=SOAPscopeMachine -Dhttp.proxyPort=5049 myPackage.MyApp
*/

/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

