// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/*  For testing in /bin/sh
 runt org.mathweb.eval.TPTPEval atpChoice CNF_NKS_EPR 20 > out 2>&1
 runt org.mathweb.eval.TPTPEval atpChoice RandomCollection 20 > out 2>&1
 runt org.mathweb.eval.TPTPEval atpOpt2 RandomCollection 60 > out 2>&1
 runt org.mathweb.eval.TPTPEval atpChoice Test 20 > out 2>&1
 runt org.mathweb.eval.TPTPEval atpSplit RandomCollection 20 > out 2>&1
*/

package org.mathweb.broker;

import java.io.FileOutputStream;
import java.util.*;
import java.net.URI;

import salvo.jesus.graph.*;
import salvo.jesus.graph.visual.*;
import salvo.jesus.graph.visual.layout.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

import org.mindswap.owls.process.Process;
import org.mindswap.owls.process.AtomicProcess;
import org.mindswap.owls.service.Service;
import org.mindswap.query.ValueMap;
// new for 1.1.0
import org.mindswap.owl.OWLDataValue;
import org.mindswap.owl.OWLValue;

import dk.capgemini.tc.JProf;

import org.mathweb.core.BasicService;
import org.mathweb.onto.OntoReasoner;
import org.mathweb.onto.PellReasoner;
import org.mathweb.onto.OWLJDom;
import org.mathweb.serv.*;
import org.mathweb.graph.*;
import org.mathweb.broker.ServiceEntry;

import org.mathweb.reasoning.logic.*;
import org.mathweb.owls.OWLSParser;

public class GologExecutor extends BasicService 
{
  //  private boolean execAllChoices = true;
  private boolean execAllChoices = false;
   
  private static Logger logger = Logger.getLogger("GologExecutor");
  private static FileAppender appender = newAppender(msHome + "/logs/GologExecutor.log");
  
  private static Calendar rightNow = Calendar.getInstance();
//   private static String cpuTimeFileName = msHome + "/etc/data/cpu/cpuTime" + rightNow.get(Calendar.YEAR) + "-" +
//     rightNow.get(Calendar.MONTH) + "-" + rightNow.get(Calendar.DAY_OF_MONTH) + "-" +
//     rightNow.get(Calendar.HOUR_OF_DAY) + ":" + rightNow.get(Calendar.MINUTE) + ".dat";
  
  private OWLJDom owlJDom = OWLJDom.getInstance();

  private OntoReasoner reasoner;
  private ServiceRegistry registry;

  //  private Hashtable owlIndis;
  private Hashtable bindings;
  private Hashtable queryVarBindings;
  private Hashtable variableBindings;
  private Hashtable paramBindings;
  private Hashtable jdomBindings;

  private String finalResult;
    
  private long globalStartTime;
  private long globalEndTime;
  private long cpuStart;
  private long distributedCpuTime;
  private long distributedWCTime;
  private Random random;

  private boolean threaded = false; 
  private String[] reportCategories = null; 
  private boolean recommendAtomic = false; 
  private String accumulatedEffects = "";
  
  public static XPath xapplVars = newXPath("./var/@name");
  public static XPath xatomVars = newXPath("./atom/@name");
  private static boolean spy = false;
  //  private static boolean spy = true;
  private int counter = 0;
  
  /* ============================================================ */
  /*       ExecutionThread class for parallel execution           */
  /* ============================================================ */
  public class ExecutionThread extends Thread {
    private GologExecutor executor;
    private Element rootElement;
    private int timeResource = 0;
    private boolean failure;
        
//     public ExecutionThread()
//     {
//       super();
//     }
    
    public ExecutionThread(OntoReasoner reasoner, 
                           ServiceRegistry registry, 
                           Hashtable bindings, 
                           Hashtable jdomBindings, 
                           Hashtable variableBindings, 
                           Hashtable paramBindings, 
                           int counter,
                           Element rootElement, 
                           int givenTime)  {
      this.executor= new GologExecutor(reasoner, registry, bindings, jdomBindings, variableBindings, paramBindings, counter, givenTime);
      this.executor.setThreaded(true);
      this.rootElement = rootElement;
      this.timeResource = givenTime;
      this.failure = false;
    }
    
    final public void run(){
      try{ 
        this.executor.executeRec(this.rootElement, this.timeResource);
      } catch(Exception e){
        //        System.out.println("exception " + e);
        //e.printStackTrace();
        this.failure = true;
      }
    }
  
    public GologExecutor getExecutor()
    {
      return this.executor;
    }

    public boolean hasFailed()
    {
      return this.failure;
    }
    
  }
  
  /* ============================================================ */
  /*       Main Golog executor class                              */
  /* ============================================================ */
  public GologExecutor()
  {
    logger.setLevel(Level.ALL);
    //    logger.setLevel(Level.OFF);
    Enumeration enu = logger.getAllAppenders();
    if (!enu.hasMoreElements()) logger.addAppender(appender);
    
  }
  
  
  public GologExecutor(OntoReasoner reasoner, 
                       ServiceRegistry registry, 
                       Hashtable bindings, Hashtable jdomBindings, Hashtable variableBindings, Hashtable paramBindings, 
                       int counter, int timeLimit)
  {
    this.globalStartTime = System.currentTimeMillis();
    this.globalEndTime = (long) this.globalStartTime + timeLimit*1000;
    this.reasoner = reasoner;
    this.registry = registry;
    this.bindings = bindings;
    this.jdomBindings = jdomBindings;
    this.variableBindings = variableBindings;
    this.paramBindings = paramBindings;
    this.counter = counter;
    this.random = new Random(this.globalStartTime);
    
    this.distributedCpuTime = 0;
    this.distributedWCTime = 0;
//     FileAppender appender = null;
//     try {
//       appender = new FileAppender(new SimpleLayout(),msHome + "/logs/GologExecutor" + counter + ".log",false);
//     } catch(Exception e) {}
//     logger.addAppender(appender);
  }
  public void setThreaded(boolean value){
    this.threaded = value;
  }
  
  public String holds(ArrayList list)
  {
    String[] resultTerm = new String[list.size()];
    //    System.out.println("[java] holds : " + list);
    
    for(int k=0; k< list.size(); k++){
      resultTerm[k] = list.get(k).toString();
    }
//     if (resultTerm[0].equals("status"))
//       return "false";
//     else 
      return "true";
  }
  
  public Double getDistributedCpuTime()
  {
    return new Double((double) this.distributedCpuTime);
  }
  
  public Double getDistributedWCTime()
  {
    return new Double((double) this.distributedWCTime);
  }
  
  /** -----------------------------------------------------------------------------------
   * @param program a Golog procedure 
   * @param givenQuery the query the Golog procedure has been computed for. 
   * @param servReg a ServiceRegistry containing at least all services used in the Golog procedure.
   * @param timeLimit the timeLimit for executing the procedure.
   * @param checkPreconds if true, the preconditions of OWL-S services are tested before invoking the service. 
   * @return The final result of the procedure. 
   */
  public synchronized String executeProgram(String program, Query givenQuery, ServiceRegistry servReg, int timeLimit, boolean checkPreconds, String[] reportCategories, boolean recommendAtomic) 
  {
    
    logger.info("query " + givenQuery);
    this.globalStartTime = System.currentTimeMillis();
    this.globalEndTime = (long) globalStartTime + timeLimit*1000;
    this.cpuStart = JProf.getCurrentThreadCpuMillis();

    this.distributedCpuTime = 0;
    this.distributedWCTime = 0;
    
    this.registry = servReg;
    /*  just to count the filenames for the resulting individuals when they are written in files */
    this.counter = 1;
    //    this.owlIndis = new Hashtable();

    /* binds 'Prolog' variables of the Golog program to OWL RDF files of the individuals  */
    this.bindings = null;
    this.bindings = new Hashtable();
    this.queryVarBindings = new Hashtable();

    /* maps 'Prolog' variables to JDom Elements to avoid superfluous parsing.  */
    this.jdomBindings = new Hashtable();

    /* maps 'Prolog' variables to rdf:IDs of OWL individuals */
    this.variableBindings = new Hashtable();

    /* maps service I/O parameters to 'Prolog' variables  */
    this.paramBindings = new Hashtable();

    this.reasoner =  PellReasoner.getNewReasoner();

    this.random = new Random(this.globalStartTime);
    this.threaded = false;
    this.reportCategories = reportCategories;
    this.recommendAtomic = recommendAtomic;
    this.accumulatedEffects = "";
    
    Element progRoot = null;
    try {
      progRoot = owlJDom.getRootElement(program);
      Element head = progRoot.getChild("head");
      Element body = progRoot.getChild("body");
      Element headApp = head.getChild("appl");
      //    List queryVars = headApp.getChildren("var");
      List queryVarNames = xapplVars.selectNodes(headApp);
      System.out.println("prog vars: " + queryVarNames);
      
      ReasoningProblem queryProblem = givenQuery.getProblem();
      String queryUri = givenQuery.getURI();
      List queryInputs = queryProblem.getInputs();
      List queryOutputs = queryProblem.getOutputs();
      
      System.out.println("prog vars: " + queryVarNames);
      
      System.out.println("uri: " + queryUri);
      
      for(int i=0; i < queryInputs.size(); i++){
        Input in = (Input) queryInputs.get(i);
        String iName = in.getName();
        String owlIndi = in.getContents();
        Attribute varA = (Attribute) queryVarNames.get(i);
        String var = varA.getValue();
        logger.info("binding " + var + " to " + owlIndi);
        this.bindings.put(var, owlIndi);
        this.queryVarBindings.put(var, iName);
        
        URI rdfID = null;
        try{ 
          Element indiRoot = owlJDom.getRootElement(owlIndi);
          String xmlBase = indiRoot.getAttributeValue("base", Namespace.XML_NAMESPACE);
          List children = indiRoot.getChildren();
          for(int k=0; k<children.size(); k++){
            Element c = (Element) children.get(k);
            if (!c.getName().equals("TimeResource")){
              String potentialID = owlJDom.valueOf(owlJDom.RDF_ID, c);
              if (potentialID != null) {
                //                System.out.println("creating: " + xmlBase + potentialID);
                rdfID = URI.create(xmlBase + potentialID);
                k = children.size();
              }} }
          //          logger.info("rdfID: " + rdfID);
          if (rdfID != null){
            this.variableBindings.put(var, rdfID);
            this.paramBindings.put(queryUri + iName, var);
            reasoner.tellIndividual(owlIndi);
          }
        } catch(JDOMException e){
          e.printStackTrace();
          throw new Exception("Some of the inputs of query " + queryUri + " were not well-formed: \n"+e.getMessage());        
        }
      }
      //      logger.info("[executeProgram]["+counter+"] bindings: " + this.bindings);
      //       logger.info("["+counter+"] body: " + body);
      List bodies = body.getChildren();
      int givenTime = (new java.lang.Long((globalEndTime - System.currentTimeMillis())/1000)).intValue() - 1;

      /** call recursive variant to actually execute program  */
      logger.info("calling executeRec: " + (Element) bodies.get(0));
      executeRec((Element) bodies.get(0), givenTime);
      
      Element rdf = new Element("RDF", OWLJDom.RDF_NS);
      rdf.addNamespaceDeclaration(OWLJDom.MW_NS);
      rdf.setAttribute("base", OWLJDom.MW_NS.getURI(), Namespace.XML_NAMESPACE);
      
      //      System.out.println("jdomBindings: " + this.jdomBindings);

      /* initialize the list of output values given by the Query
         by giving them the name of the Start node + the type (signature) of the input.
      */
      for(int i=0; i < queryOutputs.size(); i++){
        Output out = (Output) queryOutputs.get(i);
        String oName = out.getName();
        Attribute varA = (Attribute) queryVarNames.get(queryInputs.size()+i);
        String var = varA.getValue();
        Element elem = (Element) this.jdomBindings.get(var);
        //        System.out.println("jdomBind: " + var + "-->" + elem);
        if (elem != null) 
          rdf.addContent(elem);
        else
          throw new Exception("Query not answered successfully");
      }
      
      //      System.out.println("cpu so far: " + (JProf.getCurrentThreadCpuMillis() - this.cpuStart));
      long cpuSpent = distributedCpuTime;
      //      long cpuSpent = (JProf.getCurrentThreadCpuMillis() - this.cpuStart) + distributedCpuTime;
      long wcSpent = System.currentTimeMillis() - this.globalStartTime;
      
      Element timeres = owlJDom.createTimeIndiElement(cpuSpent, wcSpent);
      rdf.addContent(timeres);
      
      String document = owlJDom.newXMLDocument(rdf);

      /* store the CPU time used by the GologExecutor in a file */
      //       FileOutputStream outFile = null;
      //       try {
      //         outFile = new FileOutputStream(cpuTimeFileName, true);
  
      //         String content = (JProf.getCurrentThreadCpuMillis() - this.cpuStart) - this.distributedCpuTime + "\n";

      //         outFile.write(content.getBytes());
      //         outFile.flush();
      //         outFile.close();
      //       } catch (Exception ioe) {
      //       } finally {
      //         if (outFile != null)
      //           try{ outFile.close(); } catch (Exception e) {}
      //       }

      /** finally return the RDF document containing the results */
      return document;      
    } catch(Exception ex){
      System.out.println("exception " + ex);
      if (ex instanceof RecommendException) {
        RecommendException rex = (RecommendException) ex;
        return recommendService(rex.getNode());
      }
      ex.printStackTrace();
      return owlJDom.createFailure("Golog Executor", this.globalStartTime, this.cpuStart, ex.getMessage());    
    }
  }

  private String recommendService(Element node)  {
    String name = node.getName();
    if (name.equals("appl")){
      String serviceUri = node.getAttributeValue("func");
      String xmlBase = "http://www.mathweb.org/Broker#";
      
      ReasoningService service = this.registry.get(serviceUri);
      String wsdl = service.getWsdlXML();
      
      try {
        Element rdf = new Element("RDF",  OWLJDom.RDF_NS);
        rdf.setAttribute("base", xmlBase, Namespace.XML_NAMESPACE);
        rdf.addNamespaceDeclaration(OWLJDom.MW_NS);
        Element recom = new Element("Recommendation",OWLJDom.MW_NS);
        
        Element messageE = new Element("message",OWLJDom.MW_NS);
        messageE.addContent("The MathServe Broker recommends using service " + serviceUri);
        
        Element justE = new Element("justification",OWLJDom.MW_NS);
        justE.addContent(accumulatedEffects);
      
        Element wsdlE = new Element("wsdl",OWLJDom.MW_NS);
        wsdlE.addContent(wsdl);
        recom.addContent(messageE);
        recom.addContent(justE);
        recom.addContent(wsdlE);
        
        rdf.addContent(recom);
        return owlJDom.newXMLDocument(rdf);
      } catch(Exception ex){
        owlJDom.createFailure("Golog Executor exception: " + ex.getMessage());    
      }
    }
    return owlJDom.createFailure("Can only recommend atomic services so far:\n");    
    //       if (name.equals("sequence")){
    
    //       }
    //       else 
    
  }
  
  
  /** ========================================================================
   * @param node a JDom element which is the root of the current Golog construct
   * @param givenTime the time allocated to sub-program with the node given as root.
   */
  public void executeRec(Element node, int givenTime) throws Exception {
    //    logger.info("executeRec called with " + node);
    //    System.out.println("given time: " + givenTime);
    
    String name = node.getName();
    
    /**        STOP      */
    if (name.equals("stop")){ return; }

    /**        SEQUENCE      */
    else if (name.equals("sequence")){
      //       if (recommendSequence)
      //         throw new RecommendException(node);
      
      List sequents = node.getChildren();
      int time = givenTime;
      
      for(int i=0; i<sequents.size(); i++){
        /* time allocated:  The total remaining time in seconds divided by the number of services still to be invoked */
        //        time = (new java.lang.Long(((globalEndTime - System.currentTimeMillis())/1000)).intValue());
        /* This doesn't work well. In the CASC scenario we have ATP(X,Y,Z) followed by senseCond(ATP(X,Y,Z))  so the 
                ATP would only get half of the time it should get...
                So we simply give all the remaining time to each of them. We should exclude senseCond statements and test statements (\phi)? 
                from the count.
        */
        //time = (new java.lang.Long(((globalEndTime - System.currentTimeMillis())/1000)/(sequents.size()-i)).intValue());
        time = (new java.lang.Long(((globalEndTime - System.currentTimeMillis())/1000)).intValue());
        
//         if ((this.reportCategories != null) && (!recommendAtomic) {
        executeRec((Element) sequents.get(i), time);
        //        if (time < givenTime) 
        //          executeRec((Element) sequents.get(i), time);
        //        else 
        //          executeRec((Element) sequents.get(i), givenTime);
      }
    }
    
    /**        SENSE-EFFECT     */
    else if (name.equals("senseEffect")){
      return;
    }

    /**        IF-THEN-ELSE      */
    else if (name.equals("if-then-else")){
      List children = node.getChildren();
      Element cond = (Element) children.get(0);
      Element thenPart = (Element) children.get(1);
      Element elsePart = (Element) children.get(2);
      boolean condVal = evaluateCondition(cond);
      if (condVal)
        executeRec(thenPart, givenTime);
      else 
        executeRec(elsePart, givenTime);
    }

    /**          BIND         */
    else if (name.equals("bind")){
      String varName = node.getAttributeValue("name");
      String toVar = node.getAttributeValue("to");
      System.out.println("Explicitely binding variable " + varName + " to " + toVar + "!");
      if (!this.bindings.containsKey(varName)){
        if (this.bindings.containsKey(toVar)){
          this.bindings.put(varName, this.bindings.get(toVar));
          this.jdomBindings.put(varName, this.jdomBindings.get(toVar));
          this.variableBindings.put(varName, this.variableBindings.get(toVar));
        }
      } else {
        System.out.println("Variable " + varName + " is already bound. Stopping execution!");
        return;
      }
    }

    /**          CHOICE         */
    else if (name.equals("choice")){
      List choices = node.getChildren();
      int cn = this.random.nextInt(choices.size());
      Element choice = (Element) choices.get(cn);
      executeRec(choice, givenTime);
    }

    /**          SPLIT         */
    else if (name.equals("split")){
      List splitElements = node.getChildren();
      int numSplits = splitElements.size();
      ExecutionThread[] threads = new ExecutionThread[numSplits];
      //      int time = givenTime / choices.size();
      
      /** first create all the threads  */
      for(int c =0 ; c< numSplits ; c++){
        Element splitElem = (Element) splitElements.get(c);
        threads[c] = new ExecutionThread(reasoner, registry, this.bindings, this.jdomBindings, this.variableBindings, this.paramBindings, counter,  splitElem, givenTime);
        counter += 1;
      }
      
      /**  then start them all  */
      for(int c =0 ; c< numSplits; c++){ 
        threads[c].start();
      }
      
      /**  wait until one thread is dead. TODO: This assumes that a successful
           thread will terminate first.  This is not true in general!!! We have
           to find a general notion of when a service execution is
           successful...  The OWL-S 1.1 technical report says that SPLIT
           synchronises as soon as all branches are scheduled, i.e. it doesn't
           wait for anything. This is bad because then Prolog variables might not be bound
      */
      boolean stop = false;
      boolean timeCounted = false;
      long startTime = System.currentTimeMillis();
      
      do {
        for(int c =0 ; c< numSplits ; c++){ 
          if (!threads[c].isAlive()){
            boolean failed = threads[c].hasFailed();
            if (!failed) {
              stop = true; 
              if (! timeCounted){
                /* store the time resources used up by this service. */
                GologExecutor exec = threads[c].getExecutor();
                Double cpuTime = (Double) exec.getDistributedCpuTime();
                Double wcTime = (Double) exec.getDistributedWCTime();
                if ((wcTime != null) && (!wcTime.isNaN())) {
                  timeCounted = true;
                  distributedWCTime += wcTime.longValue();
                }
                if ((cpuTime != null) && (!cpuTime.isNaN())){
                  timeCounted = true;
                  distributedCpuTime = distributedCpuTime + cpuTime.longValue();
                }
              }
              c = numSplits;  /* end the iteration */
            }
          }
          if ((System.currentTimeMillis() - startTime) > givenTime * 1000)
            stop = true;
        }
      } while (!stop);
      
      for(int c =0 ; c< numSplits ; c++){ 
        threads[c].destroy();
      }         
    }
    
    
    /**          SPLIT & JOIN         */
    /**  sams as SPLIT but waiting for all branches to finish */
    else if (name.equals("split_join")){
      List splitElements = node.getChildren();
      int numSplits = splitElements.size();
      ExecutionThread[] threads = new ExecutionThread[numSplits];
      //      int time = givenTime / choices.size();
      
      /** first create all the threads  */
      for(int c =0 ; c< numSplits ; c++){
        Element splitElem = (Element) splitElements.get(c);
        threads[c] = new ExecutionThread(reasoner, registry, this.bindings, this.jdomBindings, this.variableBindings, this.paramBindings, counter, splitElem, givenTime);
        counter += 1;
      }
      
      /**  then start them all  */
      for(int c =0 ; c< numSplits; c++){ 
        threads[c].start();
      }
      
      /**  wait until one thread is dead. TODO: This assumes that a successful
           thread will terminate first.  This is not true in general!!! We have
           to find a general notion of when a service execution is
           successful...  The OWL-S 1.1 technical report says that SPLIT
           synchronises as soon as all branches are scheduled, i.e. it doesn't
           wait for anything. This is bad because then Prolog variables might not be bound
      */
      boolean stop = true;
      do {
        stop = true;
        for(int c =0 ; c< numSplits ; c++){ 
          if (threads[c].isAlive()){
            stop = false; 
            c = numSplits;
          }
        }
      } while (!stop);
      /* store the time resources used up by this service. */
      for(int c =0 ; c< numSplits ; c++){ 
        GologExecutor exec = threads[c].getExecutor();
        Double cpuTime = (Double) exec.getDistributedCpuTime();
        Double wcTime = (Double) exec.getDistributedWCTime();
        if ((wcTime != null) && (!wcTime.isNaN())) {
          distributedWCTime += wcTime.longValue();
        }
        if ((cpuTime != null) && (!cpuTime.isNaN())){
          distributedCpuTime = distributedCpuTime + cpuTime.longValue();
        }
      }
      //       List splitElements = node.getChildren();
      //       //      int time = givenTime / choices.size();
      //       for(int c =0 ; c< splitElements.size(); c++){
      //         Element splitElem = (Element) splitElements.get(c);
      //         executeRec(choice, givenTime);
      //       }
    }
    /** ============================================ */
    /**      SERVICE INVOCATION         */
    /** ============================================ */
    else if (name.equals("appl")){
      String serviceUri = node.getAttributeValue("func");
      ReasoningService service = this.registry.get(serviceUri);
      
      /* service is not known */
      if (service == null)
        throw new Exception("The service " + serviceUri + " is unknown!");

      /* if the service is of a category that should not be executed (but reported) throw a special exception */
      if (this.reportCategories != null) {
        TaxonomyEntry tax = service.getTaxonomyEntry();
        String code = tax.code;
        for(int i=0; i< reportCategories.length; i++){
          if (code.equals(reportCategories[i])){
            throw new RecommendException(node);
          }
        }
      }
      
      /*  not enough time for service invocation */
      if (givenTime < 2) 
        throw new Exception("There is not enough time left to call service " + serviceUri + "!");
      
      List applVars = xapplVars.selectNodes(node);
      //      System.out.println("appl vars: " + applVars);

      Service owlsService = service.getOwlsService();
      AtomicProcess process = (AtomicProcess) owlsService.getProcess();
      
      List inputs = service.getProblem().getInputs();
      List outputs = service.getProblem().getOutputs();
      int numInputs = inputs.size();
      
      logger.debug("[java] executing " + serviceUri);
      Hashtable invocationTable = new Hashtable();
      
      boolean bigProblem = false;
      /* bind all the input parameters of the OWL-S profile */
      for(int i= 0 ; i< numInputs; i++){
        Input in = (Input) inputs.get(i);
        String iName = in.getName();
        String iType = in.getType();
        String varName = (String) ((Attribute) applVars.get(i)).getValue();
        String value = (String) this.bindings.get(varName);
        //        System.out.println("[java] input " + in);
        if (value == null) {
          if (iType.equals("TimeResource")) {
            //             int givenTime = (new java.lang.Long((globalEndTime - System.currentTimeMillis())/1000)).intValue();
            if (givenTime > 1){
              /* TODO: This is a hack, find good heuristics for time later! */
              if ((bigProblem) && (givenTime > 16))
                value = owlJDom.createTimeIndi((givenTime - 15), (givenTime - 15));
              else 
                value = owlJDom.createTimeIndi(givenTime, givenTime);
            } else {
              throw new Exception("There is not enough time left to call service " + serviceUri + "!");
            }
          } else {
            throw new Exception("Input " + iName + " of service " + serviceUri + " is not of type TimeResource and is not bound.");
          }
        } else {
          if (value.length() > 2000000) bigProblem = true;
        }
        //        logger.debug("[java] inputArg " + varName + " " + value);
        invocationTable.put(iName, value);
        this.paramBindings.put(serviceUri+ iName, varName);
      }
      
      /*  invoke the service with the input parameter mapping */ 
      ValueMap resultMap = null;
      resultMap = service.invokeOWLS(invocationTable);
      //      logger.debug("[executor]["+counter+"]  result map:\n" + resultMap);
      
      if (resultMap == null)
        throw new Exception("Invocation of service " + serviceUri + " failed.");
      
      /* bind all the result variables to the actual results. */
      boolean timeCounted = false;
      
      for(int k=0; k<outputs.size(); k++){
        /* bind the result only if the Golog program reserves a variable for it */
        if (numInputs + k < applVars.size()) {
          Output out = (Output) outputs.get(k);
          String oName = out.getName();
          String oType = out.getType();
          
          String outVar = (String) ((Attribute) applVars.get(numInputs + k)).getValue();
          //        logger.debug("[java] output " + out.toString());
          
          // new for OWL-S API 1.1.0 beta        
          OWLDataValue owlV = resultMap.getDataValue(process.getOutputs().getParameter(oName));		
          String value = (String) owlV.getValue();
          if (this.threaded) {
            if ((value.indexOf("GaveUp") > -1) 
                || (value.indexOf("ResourceOut") > -1)
                || (value.indexOf("Unknown") > -1)
                || (value.indexOf("Failure") > -1))
              throw new Exception("special failure for parallel execution");
          }
          //         String value = (String) resultMap.getValue(process.getOutputs().getParameter(oName));		
          //        logger.debug("[executor]["+counter+"]  result:\n" + value);
          
          URI resID = null;
          String potentialName= null;
          try{ 
            Element resultRoot = owlJDom.getRootElement(value);
            String xmlBase = resultRoot.getAttributeValue("base", Namespace.XML_NAMESPACE);
            if (owlJDom.hasChild(resultRoot, "Failure",  OWLJDom.MW_NS)){
              String message = OWLJDom.FAILURE_MESSAGE.valueOf(resultRoot);
              throw new Exception("Invocation of service " + serviceUri + " failed:\n" + message);
            } else {
              List children = resultRoot.getChildren();
              //            System.out.println("children: " + children);
              Element rightElement = null;
              for(int i=0; i<children.size(); i++){
                Element child = (Element) children.get(i);
                if (!child.getName().equals("TimeResource")){
                  String potentialID = owlJDom.valueOf(owlJDom.RDF_ID, child);
                  logger.debug("rdf:ID= " + potentialID);
                  if ((potentialID != null) && (!potentialID.trim().equals(""))) {
                    potentialName = potentialID;
                    //                  System.out.println("creating: " + xmlBase + potentialID);
                    resID = URI.create(xmlBase + potentialID);
                    rightElement=child;
                    i = children.size();
                  } else {
                    potentialID = owlJDom.valueOf(owlJDom.RDF_ABOUT, child);
                    logger.debug("rdf:about= " + potentialID);
                    if (potentialID != null) {
                      potentialName = potentialID;
                      //                    System.out.println("creating: " + potentialID);
                      resID = URI.create(potentialID);
                      rightElement=child;
                      i = children.size();
                    } 
                  }
                }               
              }
              
            
              /** try to save the result for debugging or later reference */
              //             if (potentialName != null) {
              //               try {
              //                 String owlFileName = msHome + "/etc/data/broker/" + potentialName+ "-" +oName+ this.counter + ".owl";
              //                 writeFile(owlFileName, value);
              //               }catch(Exception e){}
              //             }

              /* remember the JDom Element the Golog variable is bound to */
              logger.info("binding jdom: " + outVar + " --> " + rightElement);

              if (resID != null) {
                logger.debug("rdfID: " + resID);
                if (!this.bindings.containsKey(outVar)) {
                  this.bindings.put(outVar, value);
                  this.jdomBindings.put(outVar, rightElement.clone());
                  this.variableBindings.put(outVar, resID);
                  this.paramBindings.put(serviceUri+ oName, outVar);
                  reasoner.tellIndividual(value);
                }
                else {
                  System.out.println("Variable " + outVar + " is already bound. Stopping execution!");
                  return;
                }
              } else {
                logger.debug("Could not determine the rdf:ID of the result " + value);
              }
            
              //            logger.info("[service exec]["+counter+"] bindings: " + this.bindings);
            
            
              /* note: the result of services with multiple outputs is split with XSL style sheets. This can lead to a situation 
                 where there is no time resource info given. */
              if (! timeCounted){
                /* store the time resources used up by this service. */
                Double wcTime = (Double) OWLJDom.TIME_RESOURCE_WC.numberValueOf(resultRoot);
                Double cpuTime = (Double) OWLJDom.TIME_RESOURCE_CPU.numberValueOf(resultRoot);
                if ((wcTime != null) && (!wcTime.isNaN())) {
                  timeCounted = true;
                  distributedWCTime += wcTime.longValue();
                }
                if ((cpuTime != null) && (!cpuTime.isNaN())){
                  timeCounted = true;
                  distributedCpuTime = distributedCpuTime + cpuTime.longValue();
                }
              }
            
            
              //            System.out.println("resource used: " + distributedCpuTime + " " + distributedWCTime);
              //            System.out.println("cpu so far: " + (JProf.getCurrentThreadCpuMillis() - this.cpuStart));
            
            }
          } catch(JDOMException e){
            e.printStackTrace();
            logger.debug("[exec] exception " + e);
            throw new Exception("Some of the outputs of service " + serviceUri + " were not well-formed: \n"+e.getMessage());        
          }
        } // end if()
      } // end for()
      
      processEffects(service);
    } else {
      logger.debug("can not handle GOLOG expression " + name + " yet.");
      return;
    }
  }
  
  /** Adds the effects of a service to the OWL-DL knowledge base. 
   * @param service a reasoning service whose effects to add.
   */
  private void processEffects(ReasoningService service) {
    List effects = service.getProblem().getEffects(); 
    String serviceUri = service.getURI();
    
    for(int k=0; k<effects.size(); k++){
      Effect effect = (Effect) effects.get(k);
      CachedHashCodeObject condition = effect.getCondition();
      processEffect(condition,  serviceUri);
    }
  }

  /** Adds an effect literal to the OWL-DL knowledge base. 
   * @param condition a logic Literal
   * @param serviceUri the base URI of the service the literal belongs to. 
   */
  private void processEffect(CachedHashCodeObject condition, String serviceUri){
    
    if (condition instanceof Literal) {
      Literal lit = (Literal) condition;
      String property = lit.getPredicate().getName();
      Argument[] args = { lit.getArgument(0), lit.getArgument(1)};
      
      String[] varName = {args[0].getName(), args[1].getName() };
            
      URI[] argOwlId = new URI[2];
      String variable1 = "", variable2 = "";
      
      try{
        if (args[0] instanceof Variable){
          logger.info("var: " + args[0]);
          logger.info("params: " + paramBindings);
          variable1 = (String) this.paramBindings.get(serviceUri + varName[0]);
          logger.info("binding: " + variable1);
          argOwlId[0] = (URI) this.variableBindings.get(variable1);
        } else { argOwlId[0] = URI.create(OWLJDom.MW_NS.toString() + varName[0]);  }
        if (args[1] instanceof Variable){
          logger.info("var2: " + args[1]);
          variable2 = (String) this.paramBindings.get(serviceUri + varName[1]);
          logger.info("binding: " + variable2);
          argOwlId[1] = (URI) this.variableBindings.get(variable2);
        } else { argOwlId[1] = URI.create(OWLJDom.MW_NS.toString() + varName[1]);  }
        
        logger.debug("asserting: " + property + "("+argOwlId[0] + ", " + argOwlId[1]+")");
        reasoner.assertLiteral(property, argOwlId[0], argOwlId[1]);

        if (queryVarBindings.containsKey(variable1))
          variable1 = (String) queryVarBindings.get(variable1);
        accumulatedEffects = accumulatedEffects + property + "(" + variable1 + ", " + argOwlId[1] + ")\n";


      } catch(Exception e){
        e.printStackTrace();
      }
    } else if (condition instanceof Conjunct){
      Conjunct conj = (Conjunct) condition;
      List lits = conj.getLiterals(); 
      for(int i=0; i<lits.size(); i++){
        processEffect((Literal) lits.get(i), serviceUri);
      }
    } else if (condition instanceof Implication){
      Implication imp = (Implication) condition;
      //      System.out.println("can't handle implications right now!");
    }
  }
    
  /** Determines the truth value of a condition in a Golog program.
   * @param cond An <appl> JDom element
   * @return true, iff the condition expressed by the JDom element is entailed by the OWL-DL knowledge base.
   */
  private boolean evaluateCondition(Element cond) throws Exception {
    try{ 
      if (cond.getName().equals("appl")){
        String property = cond.getAttributeValue("func");
        List applVars = xapplVars.selectNodes(cond);
        List applAtoms = xatomVars.selectNodes(cond);
        //        System.out.println("cond vars: " + applVars);
        //        System.out.println("cond atoms: " + applAtoms);
        if (applVars.size() == 1) {
          String var0 = (String) ((Attribute) applVars.get(0)).getValue();
          //          System.out.println("indi: " + this.variableBindings.get(var0));
          if (var0 == null){
            logger.debug("Could not evaluate condition " + property + "(" + applVars + " " + applAtoms + ".");
            logger.debug("  variable " + var0 + " not bound.");
            return false; 
          }
          
          URI indiURI = (URI) this.variableBindings.get(var0);
          logger.debug("value of: " + property + " of indi " + indiURI + "...");
          String value = reasoner.valueOf(property, indiURI);
          if (value == null) {
            System.out.println("Could not evaluate condition " + property + "(" + applVars + " " + applAtoms + ".");
            return false;
          }
          
          if (applAtoms.size() == 1){
            String atom0 = (String) ((Attribute) applAtoms.get(0)).getValue();
            boolean result = (atom0.toLowerCase().equals(value.toLowerCase()) || (atom0.toLowerCase().equals((OWLJDom.MW_NS_STR+value).toLowerCase())));
            logger.debug("condition " + property + "(" + var0 + " " + atom0 + ") is " + result);
            return result;
          } else {
            throw new Exception("Could not evaluate condition " + property + "(" + applVars + " " + applAtoms + ").");
          }}
        else {
          throw new Exception("Could not evaluate condition " + property + "(" + applVars + " " + applAtoms + ".");
        }
      }
    } catch(JDOMException e){
      e.printStackTrace();
      throw new Exception("Could not evaluate condition " + cond + ": \n"+e.getMessage());
    }
    return false;
  }
  
   
  
  
  private boolean checkPreconditions(ReasoningService serv, Hashtable invocationTable)
  /* Does it make sense to check preconditions? I guess the checking needs
   * more time than the service realising it itself and failing.
   */  
  {
    long startTime = System.currentTimeMillis();
    List preconds = serv.getProblem().getPreconds(); //getDynamicPreconds();
    boolean fulfilled = true;
    
    for(int i=0; i<preconds.size(); i++){
      //      System.out.println("pre: " + preconds.get(i).getClass().getName());
      PreCondition pre = (PreCondition) preconds.get(i);
      CachedHashCodeObject condition = pre.getCondition();
      if (condition instanceof Literal) {
        if (! checkLiteral((Literal) condition))
          fulfilled = false;
      }
      else if (condition instanceof Conjunct){
        Conjunct conj = (Conjunct) condition;
        List lits = conj.getLiterals();
        for(int k=0; k<lits.size(); k++){
          if (!checkLiteral((Literal) lits.get(k)))
            fulfilled = false;
        }
      }
    }
    logger.debug("Checking preconds took me " + (System.currentTimeMillis() - startTime) + " msecs");
    return fulfilled; 
  }
  
  private boolean checkLiteral(Literal lit)
  {
    Predicate p = lit.getPredicate();
    Argument[] args = { lit.getArgument(0), lit.getArgument(1)};
    String pName = p.getName();
    if (!pName.equals("type")){
      return false;
    }
    return true;
  }
  
  public static XPath newXPath(String x)
  {
    try{
      XPath xpath =  XPath.newInstance(x);
      return xpath;
    } catch(Exception e){
      e.printStackTrace();
      return null;
    }
  }

  public static void main(String argv[]) throws Exception {
    String msHome = System.getProperty("mathserv.home");
     OWLSParser owlsParser = new OWLSParser();
     ServiceRegistry serviceRegistry = new ServiceRegistry();
     serviceRegistry.loadDefaultOwlsServices(owlsParser);
     GologExecutor ex = new GologExecutor();
     Query  query;
     String program;
     //     query = owlsParser.parseQueryFile(msHome + "/examples/queries/owls/1.1/nd-query1.owl");
     //program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/xml");
     //program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/xml3");
     program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/atpChoice");
     //program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/compositeOpt1");
     //program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/atpSplit");
     // program = ex.readFile(msHome + "/thirdParty/golog/dtgolog/zenonOpt");
     query = owlsParser.parseQueryFile(msHome + "/examples/queries/owls/1.1/result-query1.owl");
     query = owlsParser.parseQueryFile(msHome + "/examples/queries/owls/1.1/result-query2.owl");
     String result = ex.executeProgram(program, query, serviceRegistry, 10, false, null, false);
     System.out.println("result: " + result);
  }
    

}


/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */


//   public String createResult(String probID, long startTime, long cpuStart, List results) {
//     try{
//       Element rdf = new Element("RDF", OWLJDom.RDF_NS);
//       rdf.addNamespaceDeclaration(OWLJDom.MW_NS);
//       rdf.setAttribute("base", OWLJDom.MW_NS.getURI(), Namespace.XML_NAMESPACE);
      
//       Element time = new Element("time",  OWLJDom.MW_NS);
//       Element timeres = OWLJDom.createTimeResource(startTime, cpuStart);
//       for(int i=0; i<results.size(); i++){
//         String res = (String) results.get(i);
//         Element resultRoot = owlJDom.getRootElement(res);
//         List cs = resultRoot.getChildren();
//         for(int k=0; k<cs.size(); k++){
//           Element child = (Element) ((Element) cs.get(k)).clone();
//           rdf.addContent(child);
//         }
//       }
      
//       rdf.addContent(time); 
//       return owlJDom.newXMLDocument(rdf);
//     }
//     catch(Exception e){
//       return owlJDom.createFailure(probID, startTime, cpuStart, e.getMessage());
//     }
//   }
