// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/** sample calls:

for merge method:
run org.mathweb.eval.Merge /home/jzimmer/casc/unsolved-solved-by-vampire-7-spcs-remote-proveProblemOpt-600sec-5-9-2005.dat /home/jzimmer/import/mathserve/etc/eval/Casc-Opt-4.8.2005.dat

*/

package org.mathweb.eval;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Hashtable;
import javax.xml.rpc.ParameterMode;

import sun.misc.Signal;
import sun.misc.SignalHandler;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

import org.jdom.*;

import org.mathweb.onto.OWLJDom;
import org.mathweb.serv.*;
import org.mathweb.broker.*;
import org.mathweb.util.*;
import org.mathweb.owls.*;

/**
   @author Juergen Zimmer, Saarland University, Germany
   @version $Date: 2006/08/25 09:51:19 $ $Revision: 1.7 $
*/
public class Merge {
  
  static private int delay = 1;

  protected static String msHome = System.getProperty("mathserv.home");
  
  public String TPTP_HOME = msHome + "/thirdParty/TPTPWorld";
  
  private static OWLJDom owlJDom = OWLJDom.getInstance();    
  
  private boolean busy = false;

  //  private OWLSParser owlsParser = new OWLSParser();
  

  
  /** 
   * Reads the contens of a file in a java.lang.String.
   * @param fileName the name of the file to read.
   * @return The contents of the file.
   */
  public String readFile(String fileName){
    FileInputStream isFile  = null;
    try {
      isFile = new FileInputStream(fileName);
      int nChar;
      StringBuffer sbRead = new StringBuffer();
      nChar = isFile.read();
      while (nChar > -1) {
        sbRead.append((char) nChar);
        nChar = isFile.read();
      }
      isFile.close();
      return sbRead.toString();
    } catch (IOException ioe) {
      return null;
    } finally {
      if (isFile != null)
        try{ isFile.close(); } catch (Exception e) {}
    }
  }
  

  
  /** 
   * Writes a string into a file
   * @param fileName the name of the file to write into.
   * @param content the string to be written.
   * @return True if the file as written successfully, false otherwise.
   */
  protected boolean writeFile(String fileName, String content){
    FileOutputStream outFile = null;
    try {
      outFile = new FileOutputStream(fileName) ;
      outFile.write(content.getBytes());
      outFile.close();
      return true;
    } catch (IOException ioe) {
      return false;
    } finally {
      if (outFile != null)
        try{ outFile.close(); } catch (Exception e) {}
    }
  }
  
  
  /** 
   * Merges new and old results produced by the MathServe client callListMS.
   * @param newResultsFile the name of the file containing the new results.
   * @param oldResultsFile the name of the file containing the old results.
   * The result is written on the standard output and can be redirected in a file.
   */
  public void merge(String newResultsFile, String oldResultsFile)
  {

    try{
      String[] newResults = readFile(newResultsFile).split("\n");
      String[] oldResults = readFile(oldResultsFile).split("\n");
      Hashtable newRes = new Hashtable();

      for(int i=0; i< newResults.length; i++){
        String line = newResults[i];
        line.replaceAll("\t\t", "\t");
        String[] parts = line.split("\t");
        if (parts.length > 4){
          String problem = parts[0].trim();
          String cpuTimeStr = parts[1].trim();
          String wcTimeStr = parts[2].trim();
          String answerURI = parts[4].trim();
          String[] answerSplit = answerURI.split("#");
          String answer = null;
          if (answerSplit.length > 1) 
            answer = answerSplit[1].trim();
          else 
            answer = answerURI;
          //          new.put(problem, answer);
          newRes.put(problem, line);
        }}
      
      double problems = 0.0;
      double solved = 0.0;
      double cpuTime = 0.0;
      double wcTime = 0.0;
      double wcTimeSolved = 0.0;
            
      for(int i=0; i< oldResults.length; i++){
        String line = oldResults[i];
        line.replaceAll("\t\t", "\t");
        String[] parts = line.split("\t");
        if (parts.length > 4){
          String problem = parts[0].trim();
          String cpuTimeStr = parts[1].trim();
          String wcTimeStr = parts[2].trim();
          String answerURI = parts[4].trim();
          if (newRes.containsKey(problem))
            System.out.println(newRes.get(problem));
          else  
            System.out.println(line);
          
//             String[] answerSplit = answerURI.split("#");
//             String answer = null;
//           if (answerSplit.length > 1) 
//             answer = answerSplit[1].trim();
//           else 
//             answer = answerURI;
//           System.out.println("problem " + problem);
//           System.out.println("old " + answer);
//           System.out.println("old " + old.get(problem));
          
        } else { 
          System.out.println("FAIL " + line);
        } 
      }
    }catch(Exception e) {
      e.printStackTrace();
    }
  }

  public void unsolved(String newResultsFile)
  {

    try{
      String[] newResults = readFile(newResultsFile).split("\n");
      Hashtable newRes = new Hashtable();
      
      for(int i=0; i< newResults.length; i++){
        String line = newResults[i];
        line.replaceAll("\t\t", "\t");
        String[] parts = line.split("\t");
        if (parts.length > 4){
          String cpuTimeStr = parts[1].trim();
          String wcTimeStr = parts[2].trim();
          String problem = parts[0].trim();
          String answerURI = parts[4].trim();
          String[] answerSplit = answerURI.split("#");
          String answer = null;
          if (answerSplit.length > 1) 
            answer = answerSplit[1].trim();
          else 
            answer = answerURI;
          if ((answer.equals("Unknown")) || (answer.equals("GaveUp")) || (answer.equals("ResourceOut")))
            System.out.println(line);
        }
      }
    }catch(Exception e) {
      e.printStackTrace();
      
    }
  }

  public void checkE(String GHResultsFile, String msUnsolvedFile, String probsFile)
  {
    Hashtable eRes = new Hashtable();
    try{
      String[] eResults = readFile(GHResultsFile).split("\n");

      for(int i=0; i< eResults.length; i++){
        String line = null;
        boolean found = false;
        boolean end = false;

        /* First look for the beginning, i.e. the first line in the GHResults file that 
           has more than two characters that are not blanks */
        do {
          if (i >= eResults.length )
            end = true;
          else {
            line = eResults[i];
            i++;
            if ((line.length() > 2) && (!line.substring(0, 2).equals("  ")))
              found = true;
            if (i > eResults.length) 
              end = true;
          }
        } while (! (found || end));
        
        if (found) {
          //          System.out.println("prob line: " + line);

          /* If you found the first line, split it in parts. The problem name is the first part */
          String[] parts = line.split(" ");
          String problem = parts[0];
          System.out.print(problem + "\t");
          found = false;
          end = false;

          /* Look for the follow-up line which contains the name of the prover after three blanks */
          do {
            if (i >= eResults.length )
               end = true; 
            else {
              line = eResults[i];
              //            System.out.println("E line: " + line);
              i++;
              /* Empty lines mark the start of the block for the next problem. */
              if (line.equals(""))
                end = true;
              /* Look for the ATP name. */
              else if ((line.length() > 19)){
                //System.out.println(">"+line.substring(0,19)+"<");
                //if  (line.substring(0,20).equals("   E            0.99"))
                 if  (line.substring(0,20).equals("   EP           0.99"))
                //if (line.substring(0,19).equals("   Vampire      7.0"))
                //if (line.substring(0,19).equals("   Vampire      8.1"))
                  found = true;
              }
            }            
          } while (! (found || end));
          
          if (!found) 
            System.out.println("");

          /* If you found an entry for the ATP, then put it in the eRes hashtable  */
          if (found) {
            //System.out.println("cutting line " +line);
            if (line.length() > 35) {
              String result = line.substring(29,36).trim();
              if (result.equals("UNS-Ref") || result.equals("UNS-Ass"))
                result = "Unsatisfiable";
              else if (result.equals("SAT-Ass"))
                result = "Satisfiable";
              else if (result.equals("THM-Ass") || result.equals("THM-Ref"))
                result = "Theorem";

              eRes.put(problem + ".p", result);
              System.out.println(result);
            } 
            //              System.out.println("false line" + line);
          }
        }
      }
    }catch(Exception e) {
      e.printStackTrace();
    }
  }
  
    //    System.out.println("***************");
//     String[] msUnsolved = readFile(msUnsolvedFile).split("\n");
//     String[] originalProbs = readFile(probsFile).split("\n");
    
//     /* Now go through the list of unsolved problems   */
//     for(int i=0; i< msUnsolved.length; i++){
//       String line = msUnsolved[i];
//       line.replaceAll("\t\t", "\t");
//       String[] parts = line.split("\t");
//       if (parts.length > 4){
//         String cpuTimeStr = parts[1].trim();
//         String wcTimeStr = parts[2].trim();
//         String problem = parts[0].trim();
//         String resID = parts[3].trim();
//         String answer = parts[4].trim();
//         String system = "unknown";
     
//         /* Check which prover was called by MathServe   */
//         if (resID.indexOf("Ep") > -1) system = "Ep";
//         if (resID.indexOf("Spass") > -1) system = "Spass";
//         if (resID.indexOf("Vampire") > -1) system = "Vampire";
//         if (resID.indexOf("Otter") > -1) system = "Otter";
        
//         /* Get the result of the ATP that participated in the CASC   */
//         String eResult = (String) eRes.get(problem);
//         //          System.out.println(problem + "\t" + eResult);
//         if (eResult != null)
//           /* Could the ATP solve the problem?   */
//           if (! ((eResult.equals("Timeout")) || eResult.equals("GaveUp"))){
//             boolean found = false;
//             String output = "";
//             String newProb = problem.replaceAll(".p", "");
            
//             /* Look for the original problem filename corresponding to this
//              * problem name  */
//             for(int l=0; l < originalProbs.length; l++){
//               //                //System.out.println("checking: " + originalProbs[l]);
//               if (originalProbs[l].indexOf(newProb) > -1){
//                 output = originalProbs[l];
//                 l = originalProbs.length;
//                 found = true;
//               }
//             }
// //             if (!found){
// //               System.out.println("Problem " + problem+ " not in original problem set!");
// //             }else 
//             //              output = problem;
//               //           
//             if (found) 
//               if (system.equals("Ep"))
//                 //              if (system.equals("Ep"))
//                 System.out.println(output + "\t" + system + "\t" + cpuTimeStr + "\t" + wcTimeStr + "\t" + answer);
//           } 
//       }
//     }
//  }            
  
  
  /*  the name of a golog program  and of a TSTP SPC file and time in seconds */
  public static void main(String[] args) {
    Merge c = new Merge();
    //    c.merge(args[0], args[1]);
    //c.unsolved(args[0]); 
    
    
    /*   run org.mathweb.eval.Merge /home/jzimmer/casc/GHResults /home/jzimmer/casc/unsolved /home/jzimmer/casc/ForJZ-orig */
    /*   run org.mathweb.eval.Merge /home/jzimmer/GHResults /home/jzimmer/casc/2006/ForJZ /home/jzimmer/casc/ForJZ-orig */
    c.checkE(args[0], args[1], args[2]);

  }

} // end Merge.java

      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

