// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

/** sample calls:

runt org.mathweb.eval.TPTPEval2 lambda.ags.uni-sb.de 8080 proveProblemChoice /home/jzimmer/test/casc /home/jzimmer/ForJZ 10
runt org.mathweb.eval.TPTPEval2 pan.ags.uni-sb.de 8080 proveProblemChoice /home/jzimmer/test/casc /home/jzimmer/ForJZ 10

 /home/jzimmer/nb/MathServe-CASC/bin/callListMS lambda.ags.uni-sb.de 8080 proveProblemOpt /home/jzimmer/nb/casc-test/Active /home/jzimmer/nb/casc-test/ForJZ 600

 /home/jzimmer/nb/MathServe-CASC/bin/callListMS lambda.ags.uni-sb.de 8080 proveProblemOpt /home/jzimmer/nb/casc-test/Active /home/jzimmer/nb/casc-test/spcs/CNF_NKS_EPR 600
*/

package org.mathweb.eval;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.FileOutputStream;
import java.io.BufferedReader;
import java.io.IOException;

import java.net.ConnectException;

import java.util.Calendar;
import javax.xml.rpc.ParameterMode;

import sun.misc.Signal;
import sun.misc.SignalHandler;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

import org.jdom.*;

import org.mathweb.onto.OWLJDom;
import org.mathweb.util.Restart;

import org.apache.log4j.xml.DOMConfigurator;
/**
   @author Juergen Zimmer, Saarland University, Germany
   @version $Date: 2006/08/24 13:41:18 $ $Revision: 1.5 $
*/
public class TPTPEval2 {
  
  static private int delay = 1;

  protected static String msHome = System.getProperty("mathserv.home");
  
  public String TPTP_HOME = msHome + "/thirdParty/TPTPWorld";
  
  private static OWLJDom owlJDom = OWLJDom.getInstance();    
  
  private boolean busy = false;
  private boolean shutdown = false;

  //  private OWLSParser owlsParser = new OWLSParser();
  
  static class MyHandler implements SignalHandler {
    
    TPTPEval2 myObj;
    
    public MyHandler(TPTPEval2 evalObj){
      super();
      myObj = evalObj;
    }
    
    public void handle(Signal sig) {
      myObj.shutdown();
    }
  }
  
  /** 
   * Reads the contens of a file in a java.lang.String.
   * @param fileName the name of the file to read.
   * @return The contents of the file.
   */
  public String readFile(String fileName){
    FileInputStream isFile  = null;
    try {
      isFile = new FileInputStream(fileName);
      int nChar;
      StringBuffer sbRead = new StringBuffer();
      nChar = isFile.read();
      while (nChar > -1) {
        sbRead.append((char) nChar);
        nChar = isFile.read();
      }
      isFile.close();
      return sbRead.toString();
    } catch (IOException ioe) {
      return null;
    } finally {
      if (isFile != null)
        try{ isFile.close(); } catch (Exception e) {}
    }
  }
  

  
  /** 
   * Writes a string into a file
   * @param fileName the name of the file to write into.
   * @param content the string to be written.
   * @return True if the file as written successfully, false otherwise.
   */
  protected boolean writeFile(String fileName, String content){
    FileOutputStream outFile = null;
    try {
      outFile = new FileOutputStream(fileName) ;
      outFile.write(content.getBytes());
      outFile.close();
      return true;
    } catch (IOException ioe) {
      return false;
    } finally {
      if (outFile != null)
        try{ outFile.close(); } catch (Exception e) {}
    }
  }
  
  public void evalRemote(String endpoint, String listFileName, String prefix, String method, int time)
  {
    //    String method = "proveProblemOpt";
    //    String endpoint = "http://lambda.ags.uni-sb.de:8080/axis/services/Broker";
    Calendar rightNow = Calendar.getInstance();
    String listFile = null;
    String[] listParts = listFileName.split("/");
    if (listParts.length > 1) 
      listFile = listParts[listParts.length - 1];
    else 
      listFile = listFileName;
    
    String outputFileName = msHome + "/etc/eval/" + listFile + "-remote-" + method + "-"+ time + "sec-" +
      rightNow.get(Calendar.DAY_OF_MONTH) + "-" + (rightNow.get(Calendar.MONTH) + 1) + "-" + rightNow.get(Calendar.YEAR) + ".dat";
    FileOutputStream outputFile = null; 
    
    String spcFileName = listFileName;

    try{
      if (! ((method.equals("proveProblemOpt")) || (method.equals("proveProblemSplit")) ||
             (method.equals("proveProblemSplitJoin")) || (method.equals("proveProblemChoice"))))
        throw new Exception("The method " + method + " is not valid!");
    
      outputFile = new FileOutputStream(outputFileName) ;
      String[] spcLines = readFile(spcFileName).split("\n");
      int problems = spcLines.length;
      String[] probNames = new String[problems];
      
      Integer timeInt = new Integer(time - delay);
      Service service = new Service();
      Call call;
      call = (Call) service.createCall();
      
      call.setTargetEndpointAddress(new java.net.URL(endpoint));
      call.setOperationName( method );
      call.addParameter("in0", XMLType.XSD_STRING, ParameterMode.IN);
      call.addParameter("in1", XMLType.XSD_INT, ParameterMode.IN);
      call.setReturnType(XMLType.XSD_STRING);
      call.setTimeout(new Integer((time + 120)*1000));

      int restartCounter = 0;
      boolean failure = false;
      String failureMessage = "";
      
      for(int i = 0; i< problems; i++){
        if (this.shutdown) i = problems;
        else {
          String line = spcLines[i].trim();
          String[] parts = line.split("/");
          String prob = null;
          if (parts.length > 1) 
            prob = parts[parts.length - 1].trim();
          else 
            prob = line.trim();
          
          probNames[i]= prob;
          String fileName = prefix + "/" + line;
          //          System.out.println("prob: " + prob + "   group: " + group);
          
          String content =  probNames[i] + " \t 0 \t 0 \t Unknown \n";
          try{ 
            String problem = readFile(fileName);
            if (problem == null){
              System.out.println("Could not read problem file " + fileName);
              System.exit(-1);
            }
            
            /* ======================================= */
            /* Actually call MathServe via SOAP client */
            /* ======================================= */
            String result = (String) call.invoke( new Object [] {problem, timeInt });
            
            //            System.out.println(result);
            
            Element resultRoot = owlJDom.getRootElement(result);
            try {   
              failureMessage = OWLJDom.FAILURE_MESSAGE.valueOf(resultRoot);
              //              System.out.println(failureMessage);
              if (!failureMessage.trim().equals(""))
                failure = true;
              else 
                failure = false;
            } catch (JDOMException ex){
              failure = false;
            }
            if (failure) {
              System.out.println("The invocation of some service by the MathServe broker failed:\n " + failureMessage + "\n");
              if (failureMessage.indexOf("There is not enough time left") > -1) {
                Double wcDouble = (Double) OWLJDom.TIME_RESOURCE_WC.numberValueOf(resultRoot);
                Double cpuDouble = (Double) OWLJDom.TIME_RESOURCE_CPU.numberValueOf(resultRoot);
                content =  probNames[i] + " \t" + cpuDouble.intValue() + " \t" + wcDouble.intValue() + " \t" + "Broker" + "    \t" +  "Timeout" + "\n";
                //                System.out.println(content);
                outputFile.write(content.getBytes());
                outputFile.flush();
              } else                 
                throw new Exception(failureMessage);
            } else {
              Double wcDouble = (Double) OWLJDom.TIME_RESOURCE_WC.numberValueOf(resultRoot);
              Double cpuDouble = (Double) OWLJDom.TIME_RESOURCE_CPU.numberValueOf(resultRoot);
              String status = OWLJDom.STATUS.valueOf(resultRoot);
              String rdfID = OWLJDom.POTENTIAL_ID.valueOf(resultRoot);
              String system = OWLJDom.SYSTEM.valueOf(resultRoot);
              
              if ((system == null) || (system.trim().equals("")))
                throw new Exception("Could not determine name of the ATP system responsible for result.");
              
              if ((status == null) || (status.equals(""))) status = "Unknown";
              else {
                String[] split = status.split("#");
                if (split.length > 1) status = split[1];
              }
              
              content =  probNames[i] + " \t" + cpuDouble.intValue() + " \t" + wcDouble.intValue() + " \t" + system + "    \t" +  status + "\n";
              restartCounter = 0;
              
              //              System.out.println(content);
              outputFile.write(content.getBytes());
              outputFile.flush();
            }
          } catch(org.apache.axis.AxisFault ex){
            System.out.println("Server is unreachable! \n" + ex.getMessage() + "\n");
            if (this.shutdown) {
              System.exit(-1);
            }
            //ex.printStackTrace();
            System.out.print("Restarting server via XMLRPC interface...");
            
            boolean restarted = Restart.restart();
            if (!restarted){
              System.out.println("failed!");
              System.out.println("Could not restart server! Please restart manually! Exiting now...");
              System.exit(-1);
            } 
            System.out.println("done!");
            
            if (restartCounter == 0) {
              /* send the same problem again if the server has only crashed once with it */
              restartCounter++;
              if (i>=0) i--;
            } else {
              restartCounter = 0;
              content =  probNames[i] + " \t" + "0" + " \t" + "0" + " \t" + "Unknown" + "    \t" + "GaveUp" + "\n";
              //              System.out.println(content);
              outputFile.write(content.getBytes());
              outputFile.flush();
            }
          } catch(Exception e){
            System.out.println(e.getClass().getName());
            if (this.shutdown) System.exit(-1);  
            System.out.println("Some error occured! Restarting server via XMLRPC interface!"); 
          
            boolean restarted = Restart.restart();
            if (!restarted){
              System.out.println("Could not restart server! Exiting...");
              System.exit(-1);
            }
            if (restartCounter == 0) {
              restartCounter++;
              if (i>=0) i--;
            } else {
              restartCounter = 0; 
              content =  probNames[i] + " \t" + "0" + " \t" + "0" + " \t" + "Unknown" + "    \t" + "GaveUp" + "\n";
              //              System.out.println(content);
              outputFile.write(content.getBytes());
              outputFile.flush();
            }
          }
        }
      }
    }catch(Exception e) {
      e.printStackTrace();
    }finally{
      try{
        outputFile.close();
      }catch(Exception e) {
        e.printStackTrace();
      }
    }
  }


  public synchronized void shutdown(){
    this.shutdown = true;
  }
    
  /*  the name of a golog program  and of a TSTP SPC file and time in seconds */
  public static void main(String[] args) {
    DOMConfigurator.configure(msHome + File.separator + "conf" + File.separator + "log4j.xml");
    TPTPEval2 c = new TPTPEval2();
    Signal.handle(new Signal("INT") , new MyHandler(c));
    Signal.handle(new Signal("TERM") , new MyHandler(c));

    try{
      if (args.length < 6) {
        System.out.println("Usage: TPTPEval2 <hostname> <port> <method> <dir prefix for TSTP files> <TSTP problem list file> <time limit> [&proxy host &proxy port]");
        System.exit(1);
      } else {
        if (args.length > 7){
          String proxyHost = args[6];
          String proxyPort = args[7];
          if ((!proxyHost.equals("")) && (!proxyPort.equals(""))){
            System.out.println("using proxy: " + proxyHost + ":" + proxyPort);
            System.setProperty("http.proxyHost", proxyHost); 
            System.setProperty("http.proxyPort", proxyPort); 
          }}
        String endpoint = "http://" + args[0] + ":" + args[1] + "/axis/services/Broker";
        String method = args[2];
        String prefix = args[3];
        String probListFileName = args[4];
        Integer timeInt = new Integer(args[5]);
        int time = timeInt.intValue();
        
        c.evalRemote(endpoint, probListFileName, prefix, method, time);
      }
    } catch(Exception e){
      e.printStackTrace();
      System.out.println("FAILURE");
        System.out.println("Usage: TPTPEval2 <hostname> <port> <method> <dir prefix for TSTP files> <TSTP problem list file> <time limit> [&proxy host &proxy port]");
      System.exit(1);
    }
  }

} // end TPTPEval2.java

      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */


            //             System.out.println("Some error occured! Please restart the server!\n Press ENTER after restart!");
            //             BufferedReader keyboard = new BufferedReader(new InputStreamReader(System.in),1);
            //             try {
//               String input = keyboard.readLine();
//             } catch (java.io.IOException exc) { exc.printStackTrace(); } 

//             if ((status.equals("Unsatisfiable")) || (status.equals("Theorem")))
//               try{
//                 Element proof = (Element) OWLJDom.PROOF_FORMAL_PROOFS.selectSingleNode(resultRoot);
//                 if (proof != null){
//                   String tstpFileName = msHome + "/etc/data/tstp/"+ rdfID + ".tstp";
//                   writeFile(tstpFileName, proof.getTextTrim());
//                 }  
//               } catch(Exception e){
//                 e.printStackTrace();
//                 System.exit(1);
//               }
//             else if ((status.equals("CounterSatisfiable")) || (status.equals("Satisfiable")))
//               try{
//                 Element proof = (Element) OWLJDom.PROOF_FORMAL_PROOFS.selectSingleNode(resultRoot);
//                 if (proof != null){
//                   String tstpFileName = msHome + "/etc/data/tstp/"+ rdfID + ".tstp";
//                   writeFile(tstpFileName, proof.getTextTrim());
//                 }  
//               } catch(Exception e){
//                 e.printStackTrace();
//                 System.exit(1);
//               }   
