// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISLAIMER OF ALL WARRANTIES.
*****************************************************************/
package org.mathweb.gen;

import java.io.*;
import java.util.*;
import java.net.URI;

import javax.xml.transform.Transformer;
import javax.xml.transform.ErrorListener;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender;
import org.apache.log4j.SimpleLayout; 
 
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

//import mathsaid.MATHsAiDweb;

import org.mathweb.core.BasicService;

//import OMDocJDom;
//import BasicService;

/**
@author Juergen Zimmer, Saarland University, Germany
@version $Date: 2006/08/24 13:41:18 $ $Revision: 1.7 $
*/
public class MathsaidService extends BasicService {

  /* The default timeout for MathsaidService systems */
  private int defaultTime = 60;
  protected OMDocJDom omdocJDom;
  protected String name;

  /* The MathServ ontology class that is accepted by the service as an input */
  public String acceptedConcept = "OmdocTheory";
  
  /* The MathServ ontology class of the OWL individual that is returned by the service */
  public String resultConcept = "OmdocTheory";
  
  private static Logger logger = Logger.getLogger("MathsaidService");
  private static String logFileName = "/home/serge/Mathsaid.log";
  private static FileAppender appender = newAppender(logFileName);

  private static String xsltFileName = "/home/serge/systems/MathServe/etc/xsl/omdoc/omdoc2mathsaid.xsl";

  private static int maxInputLength = 100000;
  private static int maxOutputLength = 80000;
  
  public MathsaidService()
  {
    super();
    omdocJDom = OMDocJDom.getInstance();
    
    logger.setLevel(Level.ALL);
    //logger.setLevel(Level.OFF);
    Enumeration enu = logger.getAllAppenders();
    if (!enu.hasMoreElements()) logger.addAppender(appender);
  }

  
  /** 
   * @param omdocAxioms a proving problem in RDF, i.e. an individual of the OWL class ProvingProblem as an XML string.
   * @param timeRes the time resource given for the problem, i.e. an individual of the OWL class TimeResource as an XML string.
   * @return an individual of the OWL class specified in the private string <i>resultConcept</i> of the class. All proof objects are returned in the XTSTP language, i.e. the XML version of TSTP.
   */
  public String generateTheorems(String omdocAxioms, String timeRes) {
    
    /*  initialise local variables */
    long startTime = System.currentTimeMillis();
    long cpuStart = System.currentTimeMillis();;
    String theoryID = null;
    boolean valid = false; 
    int time = 0;
    File omdocFile = null;
    
    try{
      
      
      /*  If the file is too big, use the modified SAX parser: */
//       if ((omdocAxioms != null) && (omdocAxioms.length() > 200000)){
//         //      System.out.println("monster problem " + theoryID);
//         //        return omdocJDom.createFailure(theoryID, startTime, cpuStart, "Problem (of size " + omdocAxioms.length() + " bytes) is too big to be handled by MathServe at the moment.");
//         Document axiomsDoc = omdocJDom.parseLargeDocument(omdocAxioms);
//         StringWriter writer = new StringWriter();
// //         new XMLOutputter().output(axiomsDoc,writer);
// //         /* The new XML document with huge problem descriptions written in a file */
// //         omdocAxioms = writer.toString(); 
//         theoryRoot = axiomsDoc.getRootElement();
//       } else {
      Element theoryRoot = omdocJDom.getRootElement(omdocAxioms);
      Element timeRoot = omdocJDom.getRootElement(timeRes);
      //       }
      
      String xmlBase = theoryRoot.getAttributeValue("base", Namespace.XML_NAMESPACE);
      
      logger.info("... parsing, took me " + (System.currentTimeMillis() - startTime) + " msecs");
      //        List cs = theoryRoot.getChildren();
      //        for(int i=0; i<cs.size();i++) logger.debug(cs.get(i));
      
      /*  look for potential rdf:IDs of the individual we are looking for. */
      List children = theoryRoot.getChildren();
      Element rightChild = null;
      for(int i=0; i<children.size(); i++){
        Element c = (Element) children.get(i);
        String potentialID = omdocJDom.valueOf(omdocJDom.RDF_ID, c);
        if (potentialID != null) {
          theoryID = xmlBase + potentialID;
          rightChild = c;
          break;
        }
      }
      if (theoryID == null) 
        return omdocJDom.createFailure(xmlBase, startTime, cpuStart, "Problem has to be of class " + acceptedConcept + "!");
      logger.info("[MathsaidService] axioms ID: " + theoryID);
      
      
      URI probURI = URI.create(theoryID);
      
      Element formal = (Element) OMDocJDom.OMDOC_THEORY_FORMAL_DESCRIPTIONS.selectSingleNode(theoryRoot);
      //       Attribute resource = formal.getAttribute("resource");
      //       if (resource != null){
      //         try{ 
      //           URI probUri = URI.create(resource.getValue());
      //           omdocFile = new File(probUri);
      //         } catch(Exception ex){
      //         }
      //       } else {
      String omdocXML = formal.getText();
      //       omdocFile = File.createTempFile("axioms", ".omdoc");
      //       omdocJDom.writeFile(omdocFile, omdocXML);
      //       }
      //      System.setProperty("javax.xml.transform.TransformerFactory", "org.apache.xalan.processor.TransformerFactoryImpl");
      logger.debug("omdocTheory: " + omdocXML);
      TransformerFactory tFactory = TransformerFactory.newInstance();
      Transformer transformer = tFactory.newTransformer(new StreamSource(xsltFileName));
      StringWriter  writer = new StringWriter();
      transformer.transform(new StreamSource(new StringReader(omdocXML)), 
                            new StreamResult(writer));
      String mathsaidString = new String(writer.toString().getBytes(), "UTF-8");
      //      String mathsaidString = new String(writer.toString().getBytes(), "ISO-8859-1");
      logger.info("[MathsaidService] input: \n" + mathsaidString);
      
      time = 0;
      Double num = (Double) OMDocJDom.TIME_RESOURCE_WC.numberValueOf(timeRoot);
      if (!num.isNaN()) 
        time = num.intValue();
      //      logger.info("[MathsaidService] time: " + time);
      
      System.setProperty("user.dir", msHome + sep +  "thirdParty" + sep + "mathsaid");
//       MATHsAiDweb mathsaid = new MATHsAiDweb();
      writeFile("/home/serge/fail.dat", mathsaidString);
      String result = "foo"; //mathsaid.generateTheorems(mathsaidString);
      //      return createResult(theoryID, result, startTime, cpuStart);
      return result;
      
    } catch(Exception ex){
      ex.printStackTrace();
      return omdocJDom.createFailure(theoryID, startTime,  cpuStart,"Error: "+ex.getMessage());
    }
  }
  
  
  /** 
   * @param theoryID the ID of the original OmdocTheory.
   * @param output the omdoc output by MathsAid.
   */
  public String createResult(String theoryID, String output, long startTime, long cpuStart) {
    try{
      /*  create the XML root element rdf:RDF */
      Element rdf = new Element("RDF", OMDocJDom.RDF_NS);
      rdf.addNamespaceDeclaration(OMDocJDom.MW_NS);
      rdf.setAttribute("base", OMDocJDom.MW_NS.getURI(), Namespace.XML_NAMESPACE);

      /*  create the OmdocTheory element */
      Element theory = new Element(resultConcept,OMDocJDom.MW_NS);
      //      String resultID = OMDocJDom.MW_NS_STR+name+ "_" +System.currentTimeMillis();
      Element formal = new Element("formalDescription",  OMDocJDom.MW_NS);
      formal.addContent(output);
      theory.addContent(formal);

      rdf.addContent(theory); 
      return omdocJDom.newXMLDocument(rdf);
    }
    catch(Exception e){
      e.printStackTrace();
      return omdocJDom.createFailure(theoryID, startTime, cpuStart, e.getMessage());
    }
  }
  
  public static FileAppender newAppender(String logFileName){
    FileAppender appender = null;
    try {
      appender = new FileAppender(new SimpleLayout(), logFileName, false);
    } catch(Exception e) {}
    return appender;
  }

  public static void main(String [] args) {
    System.setProperty("file.encoding", "ISO-8859-1");
    
    MathsaidService service = new MathsaidService();
    OMDocJDom omdocJDom = OMDocJDom.getInstance();

    int time = 30;
    String timeInd = omdocJDom.createTimeIndi(time, time);
     
    String omdoc; 
    omdoc = omdocJDom.readFile("/home/serge/systems/MathServe/examples/omdoc/mathsaid-test.omdoc");

    String omdocTheory = omdocJDom.createTheory(omdoc);

    //System.out.println("RDF file:\n" + omdocTheory);

    String result = service.generateTheorems(omdocTheory, timeInd);
    //    omdocJDom.writeFile("/home/jzimmer/mathsaid/SET005+0.dat", result);
    
    System.out.println(result);

  }

//   public void shutdown()
//   {
// //         for(Enumeration enum = instances.keys(); enum.hasMoreElements();){
// //             String key = (String) enum.nextElement();
// //             killInstance(key);
// //         }
//     }
    
} // end MathsaidService

      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */

