// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

package org.mathweb.jsp;

import java.io.*;  
import java.net.InetAddress;
import javax.xml.rpc.ParameterMode;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;

import org.jdom.*;

import org.mathweb.onto.OWLJDom;


public class ClientBean implements Serializable 
{
  private String format;
  private String problem;
  private String timeout;
  private String problemSource;
  private String uploadProblem;
  
  private static OWLJDom owlJDom = OWLJDom.getInstance();    

  public ClientBean(){  
    format = "foo";
    problem = "bar";
    timeout = "111";
    problemSource = "foo";
    uploadProblem = "bar";
  }
  
  public void setFormat(String s)
  {
    format = s;
  }

  public void setProblem(String p)
  {
    problem = p;
  }

  public void setTimeout(String t)
  {
    timeout = t;
  }

  public void setProblemSource(String s)
  {
    problemSource = s;
  }

  public void setUploadProblem(String s)
  {
    uploadProblem = s;
  }

  public String getFormat()
  {
    return format;
  }

  public String getProblem()
  {
    return problem;
  }

  public String getTimeout()
  {
    return timeout;
  }

  public String getProblemSource()
  {
    return problemSource;
  }

  public String getUploadProblem()
  {
    return uploadProblem;
  }

  public String call(){
    Integer timeInt = new Integer(this.timeout);
    int time = timeInt.intValue();
    
    Service service = new Service();
    Call call;
    
    StringBuffer answer = new StringBuffer();
    boolean failure = false;
    String failureMessage = "";
    
    int wcTime = 0;
    int cpuTime = 0;
      
    String system = "Unknown";
    String status = "Unknown";
    String output = "";

    try{ 
      call = (Call) service.createCall();
      
      String endpoint = "http://" + java.net.InetAddress.getLocalHost().getCanonicalHostName() + ":8080/axis/services/Broker";
      String method = null;
      if (format.equals("tptp"))
	method = "proveProblemOpt";
      else if (format.equals("omdoc"))
	method = "proveOmdocProblem";
      
      call.setTargetEndpointAddress(new java.net.URL(endpoint));
      call.setOperationName( method );
      call.addParameter("in0", XMLType.XSD_STRING, ParameterMode.IN);
      call.addParameter("in1", XMLType.XSD_INT, ParameterMode.IN);
      call.setReturnType(XMLType.XSD_STRING);
      call.setTimeout(new Integer((time + 120)*1000));
      
      if (problemSource.equals("upload"))
        problem = uploadProblem;
      
      if ((problem == null) || (problem.equals(""))){
	return "No problem or empty problem submitted!";
      }
      
      String result = (String) call.invoke( new Object [] {problem, timeInt });
      
      //            System.out.println(result);
      
      Element resultRoot = owlJDom.getRootElement(result);
      try {   
	failureMessage = OWLJDom.FAILURE_MESSAGE.valueOf(resultRoot);
	//              System.out.println(failureMessage);
	failure = !failureMessage.trim().equals("");
      } catch (Exception ex){
	failure = false;
      }
      
      Double wcDouble = (Double) OWLJDom.TIME_RESOURCE_WC.numberValueOf(resultRoot);
      Double cpuDouble = (Double) OWLJDom.TIME_RESOURCE_CPU.numberValueOf(resultRoot);
      wcTime = wcDouble.intValue();
      cpuTime = cpuDouble.intValue();
      
      if (failure) {
        status = "Failure";
	output = "The invocation of some service by the MathServe broker failed:<br><p> " + failureMessage + "</p></br>";
	
	if (failureMessage.indexOf("There is not enough time left") > -1) {
	  status = "Timeout";
	  //                System.out.println(content);
	} else {
	  status = "Failure";
          output = "MathServe encountered a problem: <br><p>" + failureMessage + "</p>";
        }
      } else {
        status = OWLJDom.STATUS.valueOf(resultRoot);
        system = OWLJDom.SYSTEM.valueOf(resultRoot);
        output = OWLJDom.OUTPUT.valueOf(resultRoot);
        
	if ((system == null) || (system.trim().equals("")))
	  system = "Unknown";
        //throw new Exception("Could not determine name of the ATP system responsible for result.");
	
	if ((status == null) || (status.equals(""))) status = "Unknown";
	else {
	  String[] split = status.split("#");
	  if (split.length > 1) status = split[1];
	}
      }
    } catch(Exception e){
      status = "Failure"; 
      output = e.toString();
    }
    answer.append("<table style=\"text-align: left;\" border=\"1\" cellpadding=\"2\" cellspacing=\"2\">\n");
    answer.append("<tbody>\n"); 
    answer.append("<tr><td> Problem Status</td><td><b>" + status + "</b></td></tr> <br>");
    answer.append("<tr><td> System Used</td><td><b>" + system + "</b></td></tr> <br>");
    answer.append("<tr><td> WC Time </td><td><b>" + wcTime + "</b>msec</td></tr> <br>");
    answer.append("<tr><td> CPU Time</td><td><b>" + cpuTime + "</b>msec</td></tr> <br>");
    answer.append("</tbody>\n"); 
    answer.append("</table><br><br>");      
    answer.append("<font size=+1>ATP system output:</font><br><br>"+ output.replaceAll("\\n","<br>"));
    return answer.toString();
  }
}



    
