// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/

package org.mathweb.mg;

import java.io.*;
import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout;

import org.mathweb.atp.ATP;
import org.mathweb.util.TSTPCreator;
import org.mathweb.core.BasicService;
import org.mathweb.onto.OWLJDom;


/**
@author Juergen Zimmer Universitaet des Saarlandes
@version $Date: 2006/08/24 13:01:08 $ $Revision: 1.2 $
*/
public class Sem extends ModelChecker {
  
  public String SEM_HOME = getSystemProperty("sem.home", "SEM_HOME");
  public String TPTP_HOME = getSystemProperty("tptp.home.dir", "TPTP_HOME_DIR");
  public String[] sem_envs = new String[] {"SEM_HOME="+SEM_HOME};
  public String semExecutable = findSystemExecutable(SEM_HOME, "sem", "Sem.exe");
  
  private static Logger logger = Logger.getLogger("Sem");
  private static String logFileName = getSystemProperty("sem.log.file", "SEM_LOG_FILE");
  private static FileAppender appender = newAppender(logFileName);
    
  private static String sep = File.separator;
    
  private static final int defaultNice = 15;
  private static String modelFound = "MODEL FOUND";
  private static String noModelFound = "MODEL NOT FOUND";

  
  public Sem() 
  {
    super();
    name = "Sem_1_7";
    
    /* The MathServ ontology class that is accepted by the service as an input */
    acceptedConcept = "TstpProblem";
    /* The MathServ ontology class of the OWL individual that is returned by the service */
    resultConcept = "ModGenResult";

    logger.setLevel(Level.ALL);
    //    logger.setLevel(Level.OFF);
    if (!logger.getAllAppenders().hasMoreElements()) logger.addAppender(appender);
  }
  
  public String[] callMC(File problemFile, String language, String format, int time, String options) throws Exception
  {
    File inputFile = null;
    File outputFile = null;
    try{
      outputFile = File.createTempFile("sem", ".out");
      
      String semOptions = " ";
      //      System.out.println(inputFile + " " + outputFile);
      
      if (language.equals(OWLJDom.MW_NS_STR+"TSTP")) {
        inputFile = tstp2X.tstp2X(problemFile, 
                                  "sem:all:1",
                                  "add_equality");
        System.out.println("new prob " + readFile(inputFile));
        String command = semExecutable + " " +  inputFile.getAbsolutePath() 
          +  " " + " > " + outputFile.getAbsolutePath() + " 2>&1";
        logger.debug("[SEM] command: " + command);
        String[] commands = {"sh", "-c", command};
        logger.debug("[SEM] starting SEM ...");
        this.start(command, sem_envs, defaultNice, time);
        //      this.exec(commands, sem_envs, false, true);
        logger.debug("reading result..." + outputFile.getAbsolutePath());
        String output = this.readFile(outputFile);
        logger.debug("[SEM] output..." + output);
        
        /*  split the output in lines and examine them to extract the status
            NOTE: tptp2X negates the conjecture when it creates prover9 input */
        String[] split = output.split("\n");
        String status = "Unknown";
        
        for(int i=0; i<split.length; i++){
          if (split[i].equals("MODEL FOUND"))
            {
              status = "Satisfiable";
              i = split.length;
            }
        }
        return new String[] {status, output};
      } 
      else{
        throw new Exception("Input format "+language+" not known or not supported.");
      }
    } catch(Exception e){
      logger.debug("[Sem] Exception " + e);
      throw e;
    } finally{
      try{ 
        //        if (outputFile != null) outputFile.delete();
      } catch(Exception e){ logger.debug("[Sem] Exception " + e); }
    }
  }
  
  
  public static void main(String argv[]) throws Exception {
      Sem e = new Sem();
      OWLJDom owlJDom = OWLJDom.getInstance();
      TSTPCreator creator = new TSTPCreator();
      String timeInd = owlJDom.createTimeIndi(50,50);
      //      timeInd = e.readFile("/home/jzimmer/import/mathserve/examples/owl/fault-time.owl");
      
      String ind = null;
      //      ind = creator.tstp2OWLProblem(msHome + "/thirdParty/TPTPWorld/TPTP/Problems/SET/SET062+3.p");

      //ind = creator.tstp2OWLProblem("/home/jzimmer/example.tstp");
      ind = creator.tstp2OWLProblem("/home/jzimmer/import/MathServe/thirdParty/TPTPWorld/TPTP/Problems/ALG/ALG008-1.p");
      
      System.out.println(ind);
      
      try {
        System.out.println(timeInd);
        String result = null;
//         for (int i=0; i< 700; i++) 
//         {
//             System.out.println(" call " + i);            
//             e = new Sem();
        result = e.findModel(ind, timeInd);
        //result = e.proveProblemXml(ind, timeInd);
        //System.out.println(result);
        
        System.out.println(result);
      }
      catch(Exception ex){
        ex.printStackTrace();
        System.out.println(ex);
      }
  }
  
  
} // end Sem.java


/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */
