 // -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/
package org.mathweb.onto;

import java.io.*;
import java.util.*;
import java.net.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
import org.xml.sax.*;
import org.xml.sax.helpers.XMLFilterImpl;
import org.xml.sax.helpers.AttributesImpl;

import dk.capgemini.tc.JProf;

import org.mathweb.core.BasicService;
import org.mathweb.atp.ATP;

public class OWLJDom extends BasicService
{
  //******************** Namespaces ******************************
  public static final String XSD_NS_STR = "http://www.w3.org/2001/XMLSchema#";
  public static final String RDF_NS_STR = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
  public static final String RDFS_NS_STR = "http://www.w3.org/2001/XMLSchema#";
  public static final String MW_NS_STR = "http://www.mathweb.org/owl/mathweb.owl#";
  public static final String STAT_NS_STR = "http://www.mathweb.org/owl/status.owl#";
  public static final String PERF_NS_STR = "http://www.mathweb.org/atp/performance#";
  public static final String SWRLX_NS_STR = "http://www.daml.org/services/owl-s/1.1/generic/swrlx.owl#";
  
  public static final Namespace XSD_NS = Namespace.getNamespace("xsd", XSD_NS_STR );
  public static final Namespace RDF_NS = Namespace.getNamespace("rdf", RDF_NS_STR );
  public static final Namespace RDFS_NS = Namespace.getNamespace("rdfs", RDFS_NS_STR );

  public static final Namespace MW_NS = Namespace.getNamespace("mw", MW_NS_STR);
  public static final Namespace STAT_NS = Namespace.getNamespace("stat", STAT_NS_STR);
  public static final Namespace PERF_NS = Namespace.getNamespace("perf", PERF_NS_STR);
  public static final Namespace SWRLX_NS = Namespace.getNamespace("swrlx", SWRLX_NS_STR);

 //***********  XPaths  ****************************************
  public static final XPath RDF_ID = newXPath("./@rdf:ID");
  public static final XPath RDF_ABOUT = newXPath("./@rdf:about");
  public static final XPath XML_BASE = newXPath("./@xml:base");
  public static final XPath FAILURE_MESSAGE = newXPath("./mw:Failure/mw:message");
  public static final XPath TSTP_FOF_PROBLEM = newXPath("./mw:TstpFOFProblem");
  public static final XPath TSTP_CNF_PROBLEM = newXPath("./mw:TstpCNFProblem"); 
  public static final XPath POST_PROBLEM = newXPath("./mw:PostProvingProblem");
  public static final XPath OMDOC_PROBLEM = newXPath("./mw:OmdocProvingProblem");
  public static final XPath PROVING_PROBLEM_TIME_RESOURCE = newXPath("./mw:time/mw:TimeResource/mw:wallClockTime");
  public static final XPath TIME_RESOURCE_WC = newXPath("./mw:TimeResource/mw:wallClockTime");
  public static final XPath TIME_RESOURCE_CPU = newXPath("./mw:TimeResource/mw:cpuTime");

  public static final XPath PROVING_PROBLEM_FORMAL_DESCRIPTION = newXPath("./mw:formalDescription");
  public static final XPath PROVING_PROBLEM_FORMAL_DESCRIPTIONS = newXPath(".//mw:formalDescription");
  public static final XPath PROVING_PROBLEM_LOGIC = newXPath("./mw:inLogic");
  public static final XPath PROVING_PROBLEM_LANGUAGE =  newXPath("/mw:language");
  public static final XPath FO_ATP_RESULT = newXPath("./mw:FoAtpResult");
  public static final XPath FO_ATP_RESULT_PROOF = newXPath("./mw:proof");
  public static final XPath PROOF_FORMAL_PROOF = newXPath("./mw:formalProof");
  public static final XPath PROOF_FORMAL_PROOFS = newXPath(".//mw:formalProof");
  public static final XPath PROOF_PROOF_OF = newXPath("./mw:proofOf/@rdf:resource");
  public static final XPath BRFP_REFUTATION = newXPath("./mw:TstpCnfBrFPRefutation");
  public static final XPath SYSTEM = newXPath(".//mw:system");
  public static final XPath STATUS = newXPath(".//mw:status/@rdf:resource");
  public static final XPath OUTPUT = newXPath(".//mw:output");
  public static final XPath POTENTIAL_ID = newXPath(".//@rdf:ID");

  public static final EntityRef MW_ENTITY = new EntityRef("mw", MW_NS.getURI());
  public static final EntityRef RDF_ENTITY = new EntityRef("rdf", RDF_NS.getURI());

  private HashMap xPaths;
  
  private SAXBuilder builder;
  private XMLOutputter outputter;

  private static OWLJDom theInstance = (new OWLJDom()).initialize();
  
  public static OWLJDom getInstance() {
    return theInstance;
  }
  
  public static XPath newXPath(String x)
  {
    try{
      XPath xpath =  XPath.newInstance(x);
      xpath.addNamespace(RDF_NS);
      xpath.addNamespace(RDFS_NS);
      xpath.addNamespace(OWLJDom.MW_NS);
      xpath.addNamespace(OWLJDom.STAT_NS);
      xpath.addNamespace(OWLJDom.PERF_NS);
      return xpath;
    } catch(Exception e){
      return null;
    }
  }
  
  public void OWLJDom()
  { return;
  }
  
  public OWLJDom initialize(){
    builder = new SAXBuilder();
    builder.setExpandEntities(false);
    Format format = Format.getPrettyFormat();
    format.setLineSeparator("\n");
    outputter = new XMLOutputter(format);
    //outputter = new XMLOutputter(" ", false);
    return this;
  }
  
  public synchronized Element getRootElement(String xml) throws JDOMException, IOException
  {
    StringReader reader = new StringReader(xml);
    Document doc = builder.build(reader);
    return doc.getRootElement();
  }

  public Document parseLargeDocument(String xml) throws JDOMException, IOException
  {
    StringReader reader = new StringReader(xml);
    builder = new SAXBuilder();
    builder.setXMLFilter(new XMLFilterImpl() {
        boolean isReadingIntoFile = false;
        File file = null;
        Writer out = null;

        public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
          if("formalDescription".equals(localName)) {
            isReadingIntoFile = true;
            try {
              file  = File.createTempFile("formalDescription","");
              System.out.println("externalised in " + file.getAbsolutePath());
              out = new OutputStreamWriter(new FileOutputStream(file),"utf-8");
              out.write(ATP.tstpFixString);
              AttributesImpl attsNew = new AttributesImpl(atts);
              for(int i= 0; i< attsNew.getLength(); i++)
                if (attsNew.getLocalName(i).equals("datatype"))
                  attsNew.removeAttribute(i);
              attsNew.addAttribute("http://www.w3.org/1999/02/22-rdf-syntax-ns#","resource","rdf:resource",null,file.toURI().toString());
              atts = attsNew;
            } catch(IOException ex) {ex.printStackTrace();}
          }
          super.startElement(uri,localName,qName,atts);
        }
        
        public void endElement(String uri, String localName, String qName) throws SAXException {
          if(isReadingIntoFile && "formalDescription".equals(localName)) {
            try {
              out.flush();
              out.close();
              isReadingIntoFile = false;
              out = null; file = null;
            } catch(IOException ex) {ex.printStackTrace(); }
          }
          super.endElement(uri, localName, qName);    //To change body of overridden methods use File | Settings | File Templates.
            }
        
        public void characters(char ch[], int start, int length) throws SAXException {
          if(isReadingIntoFile) {
            try {
              out.write(ch,start,length);
            } catch(IOException ex) {ex.printStackTrace();}
          }
                else
                  super.characters(ch, start, length);    //To change body of overridden methods use File | Settings | File Templates.
        }
        
        public void ignorableWhitespace(char ch[], int start, int length) throws SAXException {
          if(isReadingIntoFile) {
            try {
              out.write(ch,start,length);
            } catch(IOException ex) {ex.printStackTrace();}
          }
          else
            super.ignorableWhitespace(ch, start, length);    //To change body of overridden methods use File | Settings | File Templates.
        }
      });
    
    Document doc = builder.build(reader);
    return doc;
  }
  
  public String newXMLDocument(Element root) throws JDOMException, IOException
  {
    StringWriter writer = new StringWriter();
    Document doc = new Document(root);
    //    doc.setDocType(new DocType("RDF", RDF_NS.getURI()));
    outputter.output(doc, writer);
    writer.flush();
    writer.close();
    return writer.toString();
  }
  
  public String valueOf(String xpathID, Element root)
  {
//     System.out.println("[OWLJDom] xpath + root " + xpathID + " " + root);
//     System.out.println("[OWLJDom] paths " + xPaths);
    Object o = xPaths.get(xpathID);
//     System.out.println("[OWLJDom] " + o);
    
    try {
      return ((XPath) o).valueOf(root);
    } catch(Exception e){
      e.printStackTrace();
    }
    return null;
  }
  
  public String valueOf(XPath xpath, Element root)
  {
    try {
      return xpath.valueOf(root);
    } catch(Exception e){
      e.printStackTrace();
    }
    return "";
  } 
  public Object getSingleNode(String xpathID, Element root)
  {
//     System.out.println("[OWLJDom] xpath + root " + xpathID + " " + root);
//     System.out.println("[OWLJDom] paths " + xPaths);
    Object o = xPaths.get(xpathID);
//     System.out.println("[OWLJDom] " + o);
    
    try {
      return ((XPath) o).selectSingleNode(root);
    } catch(Exception e){
      e.printStackTrace();
    }
    return null;
  }

  public Object getSingleNode(XPath xpath, Element root)
  {
    try {
      return xpath.selectSingleNode(root);
    } catch(Exception e){
      e.printStackTrace();
    }
    return null;
  }

  public boolean hasChild(Element node, String name, Namespace ns) 
  {
    //    System.out.println("node:" + node);
    List cs = node.getChildren(name, ns);
    //    System.out.println("list:" + cs);
    
    return (!cs.isEmpty());
  }

  public String createTimeIndi(long cpuTimeVal,  long wcTimeVal){
    int newCpu = (new Long(cpuTimeVal)).intValue();
    int newWc = (new Long(wcTimeVal)).intValue();
    return createTimeIndi(newCpu, newWc);
  }

  public String createTimeIndi(int cpuTimeVal,  int wcTimeVal){
    try{
      Element timeRes = OWLJDom.createTimeResource(cpuTimeVal, wcTimeVal);
      Element rdf = new Element("RDF", OWLJDom.RDF_NS);
      rdf.addNamespaceDeclaration(OWLJDom.MW_NS);
      rdf.setAttribute("base", OWLJDom.MW_NS.getURI(), Namespace.XML_NAMESPACE);
      rdf.addContent(timeRes);
      return newXMLDocument(rdf);
    } catch(Exception e) {
      e.printStackTrace();
      return "Some problem occured while creating an OWL Failure: " + e.getMessage();
    }
  }
  
  public Element createTimeIndiElement(long cpuTimeVal,  long wcTimeVal){
    int newCpu = (new Long(cpuTimeVal)).intValue();
    int newWc = (new Long(wcTimeVal)).intValue();
    return OWLJDom.createTimeResource(newCpu, newWc);
  }
  public static Element createTimeResource(int cpuTimeVal,  int wcTimeVal){
    Integer intCpuTime = new Integer(cpuTimeVal);
    Integer intWCTime = new Integer(wcTimeVal);
    //    Element time = new Element("time",OWLJDom.MW_NS);
    Element timeres = new  Element("TimeResource",OWLJDom.MW_NS);
    Element wcTime = new  Element("wallClockTime",OWLJDom.MW_NS);
    Element cpuTime = new  Element("cpuTime",OWLJDom.MW_NS);
    
    wcTime.addContent(intWCTime.toString());
    wcTime.setAttribute("datatype", OWLJDom.RDFS_NS_STR+"integer", OWLJDom.RDF_NS);
    
    cpuTime.addContent(intCpuTime.toString());
    cpuTime.setAttribute("datatype", OWLJDom.RDFS_NS_STR+"integer", OWLJDom.RDF_NS);
    
    timeres.addContent(cpuTime);
    timeres.addContent(wcTime);
    return timeres;
  }
  
  public static Element createTimeResource(long startTime, long cpuStart){
    Integer intWCTime = new Integer((new Long((System.currentTimeMillis()-startTime))).intValue());
    Integer intCpuTime = new Integer((new Long((JProf.getCurrentThreadCpuMillis() - cpuStart))).intValue());
    return createTimeResource(intCpuTime.intValue(), intWCTime.intValue());
  }
  
  public String createFailure(String probID, long startTime, long cpuStart, String str){
    return createFailure(startTime, cpuStart, str + "\n For problem " + probID + ".\n");
  }

  public String createFailure(long startTime, long cpuStart, String str){
    try{
      Element rdf = new Element("RDF",  OWLJDom.RDF_NS);
      Element failure = new Element("Failure",OWLJDom.MW_NS);
      Element message = new Element("message",OWLJDom.MW_NS);
      Element time = createTimeResource(startTime, cpuStart);
      //      System.out.println("string:\n" + str);
      message.addContent(str);
      failure.addContent(message);
      rdf.addContent(failure);
      rdf.addContent(time);
      return newXMLDocument(rdf);
    } catch(Exception e) {
      e.printStackTrace();
      return "Some problem occured while creating an OWL Failure: " + e.getMessage();
    }
  }
  
  public String createFailure(String str){
    try{
      Element rdf = new Element("RDF",  OWLJDom.RDF_NS);
      Element failure = new Element("Failure",OWLJDom.MW_NS);
      Element message = new Element("message",OWLJDom.MW_NS);
      //      System.out.println("string:\n" + str);
      message.addContent(str);
      failure.addContent(message);
      rdf.addContent(failure);
      return newXMLDocument(rdf);
    } catch(Exception e) {
      e.printStackTrace();
      return "Some problem occured while creating an OWL Failure: " + e.getMessage();
    }
  }


  public String createConfirmation(String str){
    try{
      Element rdf = new Element("RDF",  OWLJDom.RDF_NS);
      Element conf = new Element("Confirmation",OWLJDom.MW_NS);
      Element message = new Element("message",OWLJDom.MW_NS);
      conf.addContent(str);
      conf.addContent(message);
      rdf.addContent(conf);
      return newXMLDocument(rdf);
      
    } catch(Exception e) {
      e.printStackTrace();
      return "Some problem occured while creating an OWL Confirmation: " + e.getMessage();
    }
  }
  
  public static void main(String argv[]) throws Exception {
    String msHome = System.getProperty("mathserv.home");
    OWLJDom dom = OWLJDom.getInstance();
    String xml = dom.readFile(msHome + "/etc/services/owls/HarveyService.owl");
    StringReader reader = new StringReader(xml);
    SAXBuilder builder;
    builder = new SAXBuilder();
    builder.setExpandEntities(false);
    Document doc = builder.build(reader);
    Element root = doc.getRootElement();
    System.out.println("root: " + root);
    
  }

}
      
/* Settings for Emacs.
 * Local variables:
 * c-basic-offset: 2
 * tab-width: 2
 * indent-tabs-mode: nil
 * End:
 */
