// -*- indent-tab-smode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/


package org.mathweb.trans;

import java.io.*;
import java.util.Vector;
import java.net.MalformedURLException;

import org.apache.xmlrpc.*;
import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;

import org.mathweb.core.BasicService;
import org.mathweb.core.CommandThread;

import org.mathweb.onto.OWLJDom;
import org.mathweb.trans.OmegaThreadXmlRpcHandler;


/**
@author Juergen Zimmer University of Edinburgh
@version $Date: 2006/10/27 10:11:02 $ $Revision: 1.1 $
*/
public class OmegaThread extends Thread {

  /** Lisp executables and environments */
  private static BasicService core = new BasicService();
  private String ACL_HOME = core.getSystemProperty("acl.home", "ACL_HOME");
  private String aclExecutable = core.findSystemExecutable(ACL_HOME, "lisp", "alisp");
  
  private String omegaDump = core.getSystemProperty("omega.dump", "OMEGA_DUMP");
  private String omegaExecutable = core.getSystemProperty("omega.exec", "OMEGA_EXEC");
  private String omegaXtermExecutable = core.getSystemProperty("omega.exec.xterm", "OMEGA_EXEC_XTERM");
  private String omegaEmacs = core.getSystemProperty("omega.emacs.exec", "OMEGA_EMACS_EXEC");
  private String omegaIlisp = core.getSystemProperty("omega.ilisp", "OMEGA_ILISP");
  private String DISPLAY = core.getSystemProperty("omega.display", "OMEGA_DISPLAY");
  private String emacsGeometry = core.getSystemProperty("omega.emacs.geometry", "OMEGA_EMACS_GEOMETRY");
  private String[] acl_envs = new String[] {"ACL_HOME="+ACL_HOME, "DISPLAY="+DISPLAY};


  private static Logger logger = Logger.getLogger("OmegaThread");
  private static String logFileName = core.getSystemProperty("omega.log.file", "OMEGA_LOG_FILE");

  private static String lispLogFileName = core.getSystemProperty("acl.log.file", "ACL_LOG_FILE");

  /** For communication with Lisp process */
  private static String sep = File.separator;
  private int serverPort = 0;
  private int defaultPort = 12345;
  private CommandThread lispThread;
  
  private WebServer xmlRpcServer;
  private String lispServerURL = "";
  private org.apache.xmlrpc.XmlRpcClient lispServerConnection;
  
  private static final int defaultNice = 15;
  private Throwable exception;
  private boolean started = false;
  private boolean ready = false;
  private int problemCounter = 1;
  private static int logfileCounter = 1;
  private static int lisplogfileCounter = 1;
  private String user = "foo";
  
  public OmegaThread(String usr)
  {
    core.mkTempDir("omega");
    //    owlJDom = OWLJDom.getInstance();
    DISPLAY = DISPLAY.replace('"', '\'');
    logger.setLevel(Level.ALL);
    //    logger.setLevel(Level.OFF);
    FileAppender appender = BasicService.newAppender(logFileName + usr + (logfileCounter++) + ".log");
    logger.removeAllAppenders();
    logger.addAppender(appender);

    xmlRpcServer = null;
    lispServerConnection = null;
    started = false;
    ready = false;
    user = usr;
    //    startOmega();
  }
  
  public void run()
  {
    startOmega();
  }
  
  
  public String callHoSetDefsElim(String postProb) throws Exception {
    String noDefsProb = null;
    File probFile = null;
    try{
      System.out.println("POST prob: "+ postProb);
      String tempDirPath = core.tempDir.getAbsolutePath();
      String probFileName = tempDirPath + sep + "problem" + problemCounter + ".post";
      System.out.println("prob file: "+ probFileName);
      problemCounter ++;
      probFile = new File(probFileName);
      core.writeFile(probFile, postProb);
      Vector params = new Vector();
      params.addElement(probFileName);
      
      Object result = callLispCommand("read-problem", params);
      System.out.println("result of read-problem " + result);
      
      params = new Vector();
      result = callLispCommand("expand-set-ext", params);
      System.out.println("result: " + result + result.getClass().getName());
      noDefsProb = (String) result;
      return noDefsProb;
    } catch (XmlRpcException e) {
      logger.debug("Omega error " + e.code + ":" + e.getMessage());
      throw new Exception("Omega error (code " + e.code + "): " + e.getMessage());
    } finally{
      try{ 
        //        if (probFile != null) probFile.delete();
      } catch(Exception e){ logger.debug("[Ho2Fo] Exception " + e); }
    }
  }
  
  
  private void startXmlRpcServer() throws ClassNotFoundException {
    //    XmlRpc.setDriver(Class.forName(".SAXDriver"));
    boolean success = false;
    serverPort = defaultPort;
    do{
      try{
        xmlRpcServer = new WebServer(serverPort);
        xmlRpcServer.start();
        logger.debug("Started Java XMLRPC server on port "+serverPort);
        xmlRpcServer.addHandler ( "$default", new OmegaThreadXmlRpcHandler(this));
        success = true;
      }catch(Exception e){
        serverPort++;
      }
      if ((serverPort - defaultPort) > 40) {
        logger.debug("More than 40 ports over "+ defaultPort + " are bound. Giving up!");
        serverPort = 0;
        break;
      }
    } while (!success);
  }
  
  public OmegaThread startOmegaEmacs(){
    if (!started){
      try {
        startXmlRpcServer();
        String serverURL = "http://localhost:" + serverPort;
        String ilisp = core.readFile(omegaIlisp);
        String tempDirPath = core.tempDir.getAbsolutePath();
        String inputFileName = tempDirPath + sep + "omega.el";
        String emacsStartup = ilisp + "\n" +
          "(setq loui-program " + 
          "\"" + omegaExecutable + " " + aclExecutable + " " + omegaDump + " " + serverURL + "\") ";
        core.writeFile(inputFileName, emacsStartup);
        
        if (serverPort != 0) {
          String command2 = omegaEmacs + " " +
            "-bg white -fg black -debug-init -display " + DISPLAY + " -name \"Omega Service (Emacs)\" " +
            "-geometry " + emacsGeometry + " -font 8x13" +
            " -l " + inputFileName + " -f loui";
          logger.debug("omega command: " + command2);
          lispThread = core.runThreaded(command2, acl_envs);
          logger.debug("after execution");
          started= true;
          return this;
        }
      }
      catch(ClassNotFoundException e) {
        System.err.println("Something went wrong when starting Omega:" +e);
        logger.fatal("Something went wrong when starting Omega:" +e);
      }
    }
    return this;
  }

  public OmegaThread startOmegaXterm(){
    if (!started){
      try {
        startXmlRpcServer();
        String serverURL = "http://localhost:" + serverPort;
        String tempDirPath = core.tempDir.getAbsolutePath();
        
        if (serverPort != 0) {
          String command2 =  omegaXtermExecutable + " " + aclExecutable + " " + omegaDump + " " + serverURL + " localhost:0.0 ";
          logger.debug("omega command: " + command2);
          lispThread = core.runThreaded(command2, acl_envs);
          started = true;
          return this;
        }
      }
      catch(ClassNotFoundException e) {
        System.err.println("Something went wrong when starting Omega:" +e);
        logger.fatal("Something went wrong when starting Omega:" +e);
      }
    }
    return this;
  }

  public OmegaThread startOmega() {
      if (!started){
        try {
          startXmlRpcServer();
          String serverURL = "http://localhost:" + serverPort;
          String tempDirPath = core.tempDir.getAbsolutePath();
          String outputFileName = tempDirPath + sep + "omega" + ".out";
          
          if (serverPort != 0) {
            String command2 = aclExecutable + " -I " + omegaDump + 
              " -qq -e \"(progn (ags::rpc~start-server '(\\\"" + serverURL + 
              "\\\" \\\"initLispServer\\\" \\\""+ user + "\\\")) " +
              "(setf excl::*restart-app-function* (lambda () (omega::omega~top))))\" > " + lispLogFileName + user + (lisplogfileCounter++) + ".log 2>&1";
            logger.debug("omega command: " + command2);
            lispThread = core.runThreaded(command2, acl_envs);
          }
          started = true;
          return this;
        }
        catch(Exception e) {
          System.err.println("Something went wrong when starting Omega:" +e);
          logger.fatal("Something went wrong when starting Omega:" +e);
        }
      }
      return this;
  }
 
  public boolean isStarted() { return this.started; }
  public boolean isReady() { return this.ready; }
  public void setReady() { this.ready = true; }
  
  public void setLispServerConnection(String url)
  {
    try{
      lispServerURL = url;
      lispServerConnection = new org.apache.xmlrpc.XmlRpcClient(url);
    } catch (MalformedURLException e){
      logger.debug("Trouble with RPC call " + e.getStackTrace());
      e.printStackTrace();
    }
  }
  
  public org.apache.xmlrpc.XmlRpcClient getLispServerConnection()
  {
    return lispServerConnection;
  }
  

  public Object callLispCommand(String command, Vector params) throws XmlRpcException
  {
    try{
      Object result = lispServerConnection.execute(command, params);
      //      logger.debug("Result: " + result + " " + result.getClass());
      return result;
    } catch (MalformedURLException e){
      logger.debug("Trouble with RPC call " + e.getStackTrace());
      e.printStackTrace();
    } catch(IOException e) {
      logger.debug("Trouble with RPC call " + e.getStackTrace());
      e.printStackTrace();
    }
    return null;
  }
  
  /**  Shuts down the underlying lisp process as cleanly as possible. 
   */
  public void shutdown()
  {
    try { 
      Vector params = new Vector();
      if (lispServerConnection != null) {
        logger.debug("shutting down lisp process!");
        callLispCommand("shutdown", params);
	lispServerConnection = null;
        logger.debug("waiting 3 secs for Lisp process to terminate...");
        synchronized(this){
          this.wait(3000);
        }
        logger.debug("...done.");
        xmlRpcServer.removeHandler("$default");
        xmlRpcServer.shutdown();
        lispThread.shutdown();
        logger.debug("Lisp process terminated!");
      }
    } catch (XmlRpcException e) {
      logger.debug("Trouble with RPC call " + e);
      e.printStackTrace();
    } catch (InterruptedException e) {
      logger.debug("Trouble with RPC call " + e);
      e.printStackTrace();
    } finally {
      core.removeTempDir();
    }
  }
   
  public  void test() {
    while(! this.ready) {
      try {
	//      synchronized(this.ready){
	System.out.println("not ready");
	synchronized(this){
          this.wait(1000);
        }
      } catch(Exception ex){
	ex.printStackTrace();
      }
    }
    try{
      String postProb = core.readFile(BasicService.msHome + "/examples/post/agathas-murderer.post");
      String[] cnf;
      for (int i=0; i< 20; i++){
	//	cnf = callOmegaClausifier(postProb);
	//      logger.debug("[Omega] generated cnf: " + cnf);
      }
      
      
//       nd = ndForFOF(fofProb, tstpProof);
//       logger.debug("[Omega] generated ND proof: " + nd);
      
      //       Vector params = new Vector();
      //       params.addElement(probFile);
      //       callLispCommand("read-problem", params);
      //       params = new Vector();
      //       Object result = callLispCommand("clausify", params);
      //       String prob = (String) result;
      //        String delta = (String) result.get(1);
      //OtterService o = new OtterService();
      //String n = o.callProver(prob, "TSTP", 300, "");
      //      logger.debug("[Omega] result " + prob); 
//       logger.debug("[Omega] final result2: " + nd);
    }
    catch(Exception ex){
      ex.printStackTrace();
    }
  } //end test()
  
  
  public static void main(String argv[]) throws Exception {
    OmegaThread e = new OmegaThread("test");
    //e.startOmegaEmacs();
    //    e.startOmegaXterm();
    e.startOmega();
    e.test();
    e.shutdown();
    System.exit(0);
  }
  
} // end Omega.java

      
