// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/
package org.mathweb.util;

import java.io.*;
import java.util.*;
import java.net.URI;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.Namespace;
import org.jdom.JDOMException;
import org.jdom.xpath.XPath;
 
import org.mathweb.core.BasicService;
import org.mathweb.onto.OWLJDom;

/**
@author Juergen Zimmer, Saarland University, Germany
@version $Date: 2006/10/16 10:07:30 $ $Revision: 1.6 $
*/
public class CompareATPs extends BasicService {
  private Quoter quoter = new Quoter();
  private static String xmlBase = "http://www.tptp.org/Problems#";
  private static OWLJDom owlJDom = OWLJDom.getInstance();
  private static String sep = File.separator;
  
  private static int time = 600;
  
  private static String[] provers = new String[]  {"dctp", "e", "paradox", "spass", "vampire", "waldmeister" } ;
  
  private static String[] spcs = new String[] {"FOF_CSA_EPR",
                                               "FOF_CSA_RFO",
                                               "FOF_SAT_EPR",
                                               "FOF_SAT_RFO",
                                               "FOF_NKC_EPR",
                                               "FOF_NKC_RFO_EQU",
                                               "FOF_NKC_RFO_NEQ",
                                               "FOF_NKS_NUN_EPR",
                                               "FOF_NKS_NUN_RFO_NEQ",
                                               "FOF_NKS_NUN_RFO_EQU",
                                               "CNF_NKS_EPR",
                                               "CNF_NKS_RFO_NEQ_NHN", 
                                               "CNF_SAT_EPR",
                                               "CNF_SAT_RFO_NEQ", 
                                               "CNF_SAT_RFO_EQU_NUE", 
                                               "CNF_SAT_RFO_PEQ_UEQ",
                                               "CNF_NKS_RFO_NEQ_HRN", 
                                               "CNF_NKS_RFO_SEQ_HRN", 
                                               "CNF_NKS_RFO_SEQ_NHN", 
                                               "CNF_NKS_RFO_PEQ_NUE", 
                                               "CNF_NKS_RFO_PEQ_UEQ"  };

  private static String[] texSpcs = new String[] {"FOF\\_CSA\\_EPR",
                                               "FOF\\_CSA\\_RFO",
                                               "FOF\\_SAT\\_EPR",
                                               "FOF\\_SAT\\_RFO",
                                               "FOF\\_NKC\\_EPR",
                                               "FOF\\_NKC\\_RFO\\_EQU",
                                               "FOF\\_NKC\\_RFO\\_NEQ",
                                               "FOF\\_NKS\\_NUN\\_EPR",
                                               "FOF\\_NKS\\_NUN\\_RFO\\_NEQ",
                                               "FOF\\_NKS\\_NUN\\_RFO\\_EQU",
                                               "CNF\\_NKS\\_EPR",
                                               "CNF\\_NKS\\_RFO\\_NEQ\\_NHN", 
                                               "CNF\\_SAT\\_EPR",
                                               "CNF\\_SAT\\_RFO\\_NEQ", 
                                               "CNF\\_SAT\\_RFO\\_EQU\\_NUE", 
                                               "CNF\\_SAT\\_RFO\\_PEQ\\_UEQ",
                                               "CNF\\_NKS\\_RFO\\_NEQ\\_HRN", 
                                               "CNF\\_NKS\\_RFO\\_SEQ\\_HRN", 
                                               "CNF\\_NKS\\_RFO\\_SEQ\\_NHN", 
                                               "CNF\\_NKS\\_RFO\\_PEQ\\_NUE", 
                                               "CNF\\_NKS\\_RFO\\_PEQ\\_UEQ"  };
  
  private static int[] texNums = new int[] {321, 46,48,17, 504,1231,39,116,1,0,480,543,221,276,224,94,462,391,1816,417,765 };
  
//     480 CNF_NKS_EPR
//     462 CNF_NKS_RFO_NEQ_HRN
//     543 CNF_NKS_RFO_NEQ_NHN
//     417 CNF_NKS_RFO_PEQ_NUE
//     765 CNF_NKS_RFO_PEQ_UEQ
//     391 CNF_NKS_RFO_SEQ_HRN
//     1816 CNF_NKS_RFO_SEQ_NHN
//     221 CNF_SAT_EPR
//     221 CNF_SAT_EPR~
//     224 CNF_SAT_RFO_EQU_NUE
//     276 CNF_SAT_RFO_NEQ
//      94 CNF_SAT_RFO_PEQ_UEQ
//     321 FOF_CSA_EPR
//      46 FOF_CSA_RFO
//      47 FOF_CSA_RFO~
//     504 FOF_NKC_EPR
//    1231 FOF_NKC_RFO_EQU
//      39 FOF_NKC_RFO_NEQ
//     116 FOF_NKS_NUN_EPR
//       1 FOF_NKS_NUN_RFO_EQU
//       0 FOF_NKS_NUN_RFO_NEQ
//      48 FOF_SAT_EPR
//      17 FOF_SAT_RFO
 

  private static String[] cascSpcs = new String[] {"CNF_NKS_RFO_PEQ_NUE", "CNF_NKS_EPR", "CNF_NKS_RFO_NEQ_NHN",
                                                     "CNF_NKS_RFO_PEQ_UEQ", "CNF_NKS_RFO_SEQ_NHN", "FOF_NKC_RFO_NEQ", "FOF_NKC_RFO_EQU"};
//   private static String[] cascSpcs = new String[] {"CNF_NKS_EPR", "CNF_NKS_RFO_PEQ_NUE", "CNF_NKS_RFO_SEQ_NHN",  "FOF_NKC_RFO_NEQ",
//                                                      "CNF_NKS_RFO_NEQ_NHN",  "CNF_NKS_RFO_PEQ_UEQ",  "FOF_NKC_RFO_EQU"};
  
  

  public String compare() { 
    
    Hashtable[] performs = new Hashtable[provers.length];
    Hashtable[] costs = new Hashtable[provers.length];
    
    try{
      for (int s = 0; s< provers.length; s++){
        
        //      String data = readFile(msHome + sep + "thirdParty" + sep + "TPTPWorld" + sep + "performance" + sep + "new" +
        String data = readFile(msHome + sep + "thirdParty" + sep + "TPTPWorld" + sep + "performance" + sep + "2006" +
                               sep + provers[s] + "-" + time);
        String[] lines = data.split("\n");
        System.out.println("data " + data);
        performs[s] = new Hashtable();
        costs[s] = new Hashtable();
        
        if (lines.length > 0) {
          String servName = lines[0];
          
          for(int i=1 ; i< lines.length; i++){
            System.out.println("line " + lines[i]);
            String[] parts = lines[i].split(" ");
            if (parts.length > 2){
              String className = parts[0];
              Float probability = new Float(parts[1]);
              Float costFloat = new Float(parts[2]);
              Integer cost = new Integer(costFloat.intValue());
              
              Float newProb = new Float(Math.round(probability.floatValue() * 100));
              
              performs[s].put(className, newProb);
              costs[s].put(className, cost);
              
            } else {
              System.out.println("invalid entry " + lines[i]);
              System.exit(1);
            }
          }
        }
      }
      
      StringBuffer result = new StringBuffer();
      result.append("<html><head><meta content=\"text/html; charset=ISO-8859-1\" http-equiv=\"content-type\"><title></title></head><body>");
      
      result.append("This table has been generated automatically from TPTP performance data. Table rows in red indicate SPCs populated by CASC-20 problems. Bold figures indicate the best prover in an SPC.");
      
      result.append("<font size=-1><table style=\"width: 100%; text-align: left;\" border=\"1\" cellpadding=\"2\"cellspacing=\"2\"><tbody>");
      
      result.append("<tr>");
      result.append("<td>");
      result.append("SPC");
      result.append("</td>");
      for (int p=0; p<provers.length; p++){
        result.append("<td>");
        result.append(provers[p]);
        result.append("</td>");
      }
      result.append("<td>");
      result.append("Best");
      result.append("</td>");
      
      result.append("</tr>");
      
      for(int s = 0; s< spcs.length; s++){
        boolean cascSPC = false; 
        for(int k=0; k< cascSpcs.length; k++){
          if ((!cascSPC) && (spcs[s].equals(cascSpcs[k]))){
            cascSPC = true;
          }
        }
        

        result.append("<tr>");
        result.append("<td>");
        if (cascSPC)
          result.append("<font color=\"red\">");
        result.append(spcs[s]);
        if (cascSPC)
          result.append("</font>");
        result.append("</td>");
        float max = (float) 0.0;
        String maxProver = "";
        int maxCost = 0;
        String explain = "";
        
        for (int p=0; p<provers.length; p++){
          Float pr = (Float) performs[p].get(spcs[s]);
          if (pr.floatValue() > max) {
            max = pr.floatValue();
            maxProver = provers[p];
            maxCost = ((Integer) costs[p].get(spcs[s])).intValue();
          } else
            if (pr.floatValue() == max) {
              int co = ((Integer) costs[p].get(spcs[s])).intValue();
              if (co < maxCost){
                max = pr.floatValue();
                maxProver = provers[p];
                maxCost = co;
                explain = " (cost)";
              }
            }
        }
        for (int p=0; p<provers.length; p++){
          result.append("<td>");
          if (provers[p].equals(maxProver))
            result.append("<b>");
          if (cascSPC)
            result.append("<font color=\"red\">");
          
          result.append(performs[p].get(spcs[s]));
          if (cascSPC)
            result.append("</font>");
          if (provers[p].equals(maxProver))
            result.append("</b>");
          result.append("</td>");
        }
        result.append("<td>");
        if (cascSPC)
          result.append("<font color=\"red\">");
        result.append(maxProver);
        result.append(explain);
        if (cascSPC)
          result.append("</font>");
        result.append("</td>");
        
        result.append("</tr>");
      }
      result.append("</tbody></table></font><br></body></html>");
      
      return result.toString();
      
    } catch(Exception e) {
      e.printStackTrace();
      return null;
    }
  }

  public String compareTex() { 
    
    Hashtable[] performs = new Hashtable[provers.length];
    Hashtable[] costs = new Hashtable[provers.length];
    try{
      for (int s = 0; s< provers.length; s++){
        
        //      String data = readFile(msHome + sep + "thirdParty" + sep + "TPTPWorld" + sep + "performance" + sep + "new" +
        String data = readFile(msHome + sep + "thirdParty" + sep + "TPTPWorld" + sep + "performance" + sep + "2006" +
                               sep + provers[s] + "-" + time);
        String[] lines = data.split("\n");
        System.out.println("data " + data);
        performs[s] = new Hashtable();
        costs[s] = new Hashtable();
        
        if (lines.length > 0) {
          String servName = lines[0];
          
          for(int i=1 ; i< lines.length; i++){
            System.out.println("line " + lines[i]);
            String[] parts = lines[i].split(" ");
            if (parts.length > 2){
              String className = parts[0];
              Float probability = new Float(parts[1]);
              Float costFloat = new Float(parts[2]);
              Integer cost = new Integer(costFloat.intValue());
              
              Float newProb = new Float(Math.round(probability.floatValue() * 100));
              
              performs[s].put(className, newProb);
              costs[s].put(className, cost);
              System.out.println(provers[s] + " " + className + " " + newProb);

            } else {
              System.out.println("invalid entry " + lines[i]);
              System.exit(1);
            }
          }
        }
      }
      
      StringBuffer result = new StringBuffer();
      result.append("\\begin{tabular}[h]{|l|r|r|r|r|r|r|r|}\\hline\n");
      result.append("\\multicolumn{1}{|c|}{Specialists} & \\multicolumn{1}{c|}{No.} & \\multicolumn{1}{c|}{\\dctp} & \\multicolumn{1}{c|}{\\eprover}  & \\multicolumn{1}{c|}{\\paradox}  & \\multicolumn{1}{c|}{\\spass}   & \\multicolumn{1}{c|}{\\vampire}  & \\multicolumn{1}{c|}{\\waldmeister}\\\\\n");
      result.append("\\multicolumn{1}{|c|}{Problem Class} & Probs & &&&&&\\\\\\hline\\hline\n");
      
      for(int s = 0; s< spcs.length; s++){
        boolean cascSPC = false; 
        for(int k=0; k< cascSpcs.length; k++){
          if ((!cascSPC) && (spcs[s].equals(cascSpcs[k]))){
            cascSPC = true;
          }
        }     
        
        if (cascSPC)
          result.append("{\\bf ");
        result.append(texSpcs[s]);
        if (cascSPC)
          result.append("}");
        
        if ((texSpcs[s].length() < 14))
          result.append("\t");
        if ((texSpcs[s].length() > 13) && cascSPC)
          result.append("\t");
        else  
          result.append("\t\t");

        result.append(" & ");
        result.append(""+texNums[s]);
        result.append("\t & ");

        float max = (float) 0.0;
        String maxProver = "";
        int maxCost = 0;
        String explain = "";
        
        for (int p=0; p<provers.length; p++){
          Float pr = (Float) performs[p].get(spcs[s]);
          if (pr.floatValue() > max) {
            max = pr.floatValue();
            maxProver = provers[p];
            maxCost = ((Integer) costs[p].get(spcs[s])).intValue();
          } else
            if (pr.floatValue() == max) {
              int co = ((Integer) costs[p].get(spcs[s])).intValue();
              if (co < maxCost){
                max = pr.floatValue();
                maxProver = provers[p];
                maxCost = co;
                explain = " (cost)";
              }
            }
        }
        for (int p=0; p<provers.length; p++){
          if (provers[p].equals(maxProver) && (! texSpcs[s].equals("FOF\\_NKS\\_NUN\\_RFO\\_NEQ")))
            result.append("{\\bf ");
          //           if (cascSPC)
          //             result.append("<font color=\"red\">");
          
          if (texSpcs[s].equals("FOF\\_NKS\\_NUN\\_RFO\\_NEQ"))          
            result.append("0.0");
          else
            result.append(performs[p].get(spcs[s]));
//           if (cascSPC)
//             result.append("}");
          if (provers[p].equals(maxProver)  && (! texSpcs[s].equals("FOF\\_NKS\\_NUN\\_RFO\\_NEQ")))
            result.append("}");
          
          if (p<provers.length -1) 
            result.append("\t &");
        }
//         if (cascSPC)
//           result.append("<font color=\"red\">");
//         result.append(maxProver);
//         result.append(explain);
//         if (cascSPC)
//           result.append("</font>");
//         result.append("</td>");
        
        result.append("\\\\\\hline\n");
      }
      result.append("\\end{tabular}\n\n\n");
      result.append("%%% Local Variables: \n");
      result.append("%%% mode: latex\n");
      result.append("%%% TeX-master: \"main\"\n");
      result.append("%%% End:\n"); 

      return result.toString();
      
    } catch(Exception e) {
      e.printStackTrace();
      return null;
    }
  }
  
  public static void main(String[] args) {
    CompareATPs creat  = new CompareATPs();
    //    String result= creat.compare();
    //    creat.writeFile("/home/jzimmer/public_html/atp/performance/comparison.html", result);
    String result= creat.compareTex();
    creat.writeFile("/home/jzimmer/thesis/comparison.tex", result);
    System.out.println(result);
    
    System.exit(0);
  }
}

  
  
