// -*- indent-tabs-mode: nil -*-
/*****************************************************************
This file is part of the MathServe system

Authors: Juergen Zimmer

email: jzimmer@ags.uni-sb.de
   
For information about this program, write to:                          
MathWeb-Project                                                        
AG Siekmann/FB Informatik                                            
Universitaet des Saarlandes                                          
Gebaeude E 1.1                                                     
D-66041 Saarbruecken                                                 
Germany    

For information about the newest version of MathServe, see 
http://www.ags.uni-sb.de/~jzimmer/mathserve.html

See the file "LICENSE.txt" or
http://www.ags.uni-sb.de/~jzimmer/LICENSE.html
for information on usage and redistribution
of this file, and for a DISCLAIMER OF ALL WARRANTIES.
*****************************************************************/
package org.mathweb.util;

import java.io.*;
import java.util.*;

import org.apache.log4j.Level;
import org.apache.log4j.Logger; 
import org.apache.log4j.FileAppender; 
import org.apache.log4j.SimpleLayout; 

/**
@author Juergen Zimmer, Saarland University, Germany
@version $Date: 2006/08/24 13:40:36 $ $Revision: 1.16 $
*/
public class TPTPAnalyser {
  
  public String readFile(String fileName){
    FileInputStream isFile = null;
    try {
      isFile = new FileInputStream(fileName);
      int nChar;
      StringBuffer sbRead = new StringBuffer();
      nChar = isFile.read();
      while (nChar > -1) {
        sbRead.append((char) nChar);
        nChar = isFile.read();
      }
      isFile.close();
      return sbRead.toString();
    } catch (IOException ioe) {
      return null;
    } finally{
      if (isFile != null) 
        try {isFile.close();
        } catch (IOException ioe) {
        }
    } 
  }
    
  public boolean writeFile(String fileName, String content){
    try {
      FileOutputStream outFile = new FileOutputStream(fileName) ;
      outFile.write(content.getBytes());
      outFile.close();
      return true;
    } catch (IOException ioe) {
      return false;
    }
  }
  
  
  public void extractGologResults(String inputFileName)  { 
    String msHome = System.getProperty("mathserv.home");
    try{
      String input = readFile(msHome + "/etc/eval/"+inputFileName); 
      //String input = readFile(inputFileName); 
      StringBuffer buff = new StringBuffer();
      String[] lines = input.split("\n");
      boolean afterHeader = false;
      
      double problems = 0.0;
      double solved = 0.0;
      double cpuTime = 0.0;
      double wcTime = 0.0;
      double wcTimeSolved = 0.0;
      double unknown = 0.0;
      
      for(int i=0; i< lines.length; i++){
        String line = lines[i];
        // skip the second entry for that problem:
        line.replaceAll("\t\t", "\t");
        String[] parts = line.split("\t");
        if (parts.length > 4){
          problems = problems + 1.0;
          String cpuTimeStr = parts[1].trim();
          String wcTimeStr = parts[2].trim();
          String problem = parts[0].trim();
          String answerURI = parts[4].trim();
          String[] answerSplit = answerURI.split("#");
          String answer = null;
          if (answerSplit.length > 1) 
            answer = answerSplit[1].trim();
          else 
            answer = answerURI;
          System.out.println("problem " + problem);
          System.out.println("answer " + answer);
          
          if ((answer!=null) &&
              ((answer.equals("Unsatisfiable") 
                || answer.equals("CounterSatisfiable") 
                || answer.equals("Satisfiable") 
                || answer.equals("Theorem")))){
              // problem successfully solved 
              solved = solved + 1.0;
              try{
                  Float cpu = new Float((new Integer(cpuTimeStr)).floatValue());
                  cpuTime = cpuTime + cpu.doubleValue();
                  Float wc = new Float((new Integer(wcTimeStr)).floatValue());
                  wcTimeSolved = wcTimeSolved + wc.doubleValue();
              } catch(Exception e){}   
          } else {
              if (answer.equals(""))
                  System.out.println("failure status: " + answer);
              unknown = unknown + 1.0;
          }
          try{  
              Float wc = new Float((new Integer(wcTimeStr)).floatValue());
              wcTime = wcTime + wc.doubleValue();
          }   catch(Exception e){}
          //        else{
          //             Float cpu = new Float(Float.parseFloat(cpuTimeStr) * 1000);
          //             cpuTime = cpuTime + cpu.doubleValue();
          //           }
        } else 
            {
                System.out.println(line);
            }
        
      }
      System.out.println(problems + " " + solved + " " + unknown + " " + cpuTime + " " + wcTime);
      
      double s;
      if (problems == 0.0) s = 0;
      else s = solved / problems;
      
      /* average cpu time for successful runs */
      double c;
      if (solved == 0.0) c = 0;
      else c = cpuTime / solved;
      
      /* average wc time to answer query */
      double d;
      if (problems == 0.0) d = 0;
      else d = wcTime / problems;
      

      /* average wc time to SUCCESSFULLY answer query */
      double e;
      if (solved == 0.0) e = 0;
      else e = wcTimeSolved / solved;
      
      
      String output = (new Double(s)).floatValue() + " " + (new Double(c)).floatValue() + " " + (new Double(e)).floatValue() +" " + (new Double(d)).floatValue() + "\n";
      //     String output = (new Double(s)).floatValue() + " " + (new Double(c)).floatValue() + "\n";
      writeFile(msHome + "/etc/eval/results/"+inputFileName, output);
      
    } catch(Exception ex){
      ex.printStackTrace();
    }
  }

  
  public void extractResults(String inputFileName)  { 
    String msHome = System.getProperty("mathserv.home");
    try{
      //      String input = readFile(msHome + "/thirdParty/TPTPWorld/rawdata/casc/"+inputFileName); 
      String input = readFile(msHome + "/thirdParty/TPTPWorld/rawdata/2006/"+inputFileName); 
      //String input = readFile(msHome + "/thirdParty/TPTPWorld/rawdata/"+inputFileName); 
      StringBuffer buff = new StringBuffer();
      String[] lines = input.split("\n");
      boolean afterHeader = false;
      
      double problems = 0.0;
      double solved = 0.0;
      double cpuTime = 0.0;
      
      for(int i=0; i< lines.length; i++){
        String line = lines[i];
        // skip the second entry for that problem:
        i++;
        problems = problems + 1.0;
        String[] parts = line.split(" ");
        if (parts.length > 3){
            String problem = parts[0].trim();
            String answer = parts[1].trim();
            String cpuTimeStr = parts[2].trim();
          
          if (answer.equals("Unsatisfiable") 
              || answer.equals("CounterSatisfiable") 
              || answer.equals("Satisfiable") 
              || answer.equals("Theorem")){
            // problem successfully solved 
            solved = solved + 1.0;
            Float cpu = new Float(Float.parseFloat(cpuTimeStr) * 1000);
            cpuTime = cpuTime + cpu.doubleValue();
            //        else{
            //             Float cpu = new Float(Float.parseFloat(cpuTimeStr) * 1000);
            //             cpuTime = cpuTime + cpu.doubleValue();
            //           }
          }
        }
      }
      
      System.out.println(problems + " " + solved +  " " + cpuTime);
      String output;
      
      if (problems > 0.0){
        double s = solved / problems;
        if (s == 0.0) 
          s = 0.001;
        
        double c;
        if (solved == 0.0) c = 0;
        else c = cpuTime / solved;
      
        //String output = (new Double(problems)).intValue() + " " + (new Double(s)).floatValue() + " " + (new Double(c)).floatValue() + "\n";
        output = (new Double(s)).floatValue() + " " + (new Double(c)).floatValue() + "\n";
        //      writeFile(msHome + "/thirdParty/TPTPWorld/performance/casc/"+inputFileName, output);
      } else 
        output = "0.5 0.0 \n";

      writeFile(msHome + "/thirdParty/TPTPWorld/performance/2006/"+inputFileName, output);
      //writeFile(msHome + "/thirdParty/TPTPWorld/performance/new/"+inputFileName, output);
      
    } catch(Exception ex){
        ex.printStackTrace();
    }           
  }
    
  
  public static void main(String[] args) {
    TPTPAnalyser analyser  = new TPTPAnalyser();
    analyser.extractResults(args[0]);
    //analyser.extractGologResults(args[0]);
    System.exit(0);
  }
}

  
  
