//-----------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdarg.h>

#include "DataTypes.h"
#include "Utilities.h"
#include "FileUtilities.h"
#include "Tokenizer.h"
#include "Parsing.h"
#include "Signature.h"
#include "Examine.h"
#include "List.h"
#include "Tree.h"
#include "Compare.h"
#include "Modify.h"
#include "Interpret.h"
#include "PrintTSTP.h"
#include "SystemOnTPTP.h"

#include "GDV.h"
//-----------------------------------------------------------------------------
void GlobalInterruptHandler(int TheSignal) {

    extern int GlobalInterrupted;

    printf("GDV has been interrupted\n");
    GlobalInterrupted = 1;
}
//-----------------------------------------------------------------------------
int ProcessCommandLine(int argc,char * argv[],OptionsType * OptionValues) {

    int OptionChar;

    OptionValues->Quietness = 1;
    OptionValues->AutoMode = 0;
    OptionValues->ForceContinue = 0;
    OptionValues->VerifyLeaves = 0;
    OptionValues->NoExpensiveChecks = 0;
    OptionValues->NoIntroducedAxioms = 0;
    strcpy(OptionValues->ProblemFileName,"");
    OptionValues->CheckParentRelevance = 0;
    OptionValues->CheckRefutation = 0;
    strcpy(OptionValues->TheoremProver,DEFAULT_THEOREM_PROVER);
    strcpy(OptionValues->ModelFinder,DEFAULT_MODEL_FINDER);
    strcpy(OptionValues->Saturator,DEFAULT_SATURATOR);
    strcpy(OptionValues->UnsatisfiabilityChecker,
DEFAULT_UNSATISFIABILITY_CHECKER);
    strcpy(OptionValues->CounterTheoremProver,DEFAULT_COUNTERTHEOREM_PROVER);
    OptionValues->TimeLimit = DEFAULT_TIME_LIMIT;
    OptionValues->KeepFiles = 0;
    strcpy(OptionValues->KeepFilesDirectory,DEFAULT_KEEP_FILES_DIRECTORY);
    strcpy(OptionValues->DerivationFileName,"--");
    
    while ((OptionChar = getopt(argc,argv,"q:aflxni:k:vrp:u:m:s:t:h")) != -1) {
        switch (OptionChar) {
            case 'q':
                OptionValues->Quietness = atoi(optarg);
                break;
            case 'a':
                OptionValues->AutoMode = 1;
                break;
            case 'f':
                OptionValues->ForceContinue = 1;
                break;
            case 'l':
                OptionValues->VerifyLeaves = 1;
                break;
            case 'x':
                OptionValues->NoExpensiveChecks = 1;
                break;
            case 'n':
                OptionValues->NoIntroducedAxioms = 1;
                break;
            case 'i':
                strcpy(OptionValues->ProblemFileName,optarg);
                break;
            case 'k':
                OptionValues->KeepFiles = 1;
                strcpy(OptionValues->KeepFilesDirectory,optarg);
                break;
            case 'v':
                OptionValues->CheckParentRelevance = 1;
                break;
            case 'r':
                OptionValues->CheckRefutation = 1;
                break;
            case 'p':
                strcpy(OptionValues->TheoremProver,optarg);
                break;
            case 'u':
                strcpy(OptionValues->UnsatisfiabilityChecker,optarg);
                break;
            case 'm':
                strcpy(OptionValues->ModelFinder,optarg);
                break;
            case 's':
                strcpy(OptionValues->Saturator,optarg);
                break;
            case 't':
                OptionValues->TimeLimit = atoi(optarg);
                break;
            case 'h':
            case '?':
                printf("Usage: %s <options> <derivation file>\n",argv[0]);
                printf("<options> are ...\n");
                printf("-q<quietness>  - control amount of output (1)\n");
                printf("-a             - automode (no)\n");
                printf("-f             - force continue on failure (no)\n");
                printf("-l             - verify leaves (no)\n");
                printf("-x             - suppress expensive checks (no)\n");
                printf("-n             - don't expect introduced axioms (no)\n");
                printf("-i<input file> - specify problem file (none)\n");
                printf("-k<directory>  - keep discharge files in the directory (none)\n");
                printf("-v             - check relevance of logical consequences (no)\n");
                printf("-r             - derivation should be a refutation (no)\n");
                printf("-p             - FOF theorem prover System---Version (%s)\n",DEFAULT_THEOREM_PROVER);
                printf("-c             - FOF countertheorem prover System---Version (%s)\n",DEFAULT_COUNTERTHEOREM_PROVER);
                printf("-u             - unsatifiability checker System---Version (%s)\n",DEFAULT_UNSATISFIABILITY_CHECKER);
                printf("-m             - model finder System---Version (%s)\n",DEFAULT_MODEL_FINDER);
                printf("-s             - saturator System---Version (%s)\n",DEFAULT_SATURATOR);
                printf("-t<time limit> - CPU limit for discharge attempts (%ds)\n",DEFAULT_TIME_LIMIT);
                printf("-h             - print this help\n");
                printf("<derivation file> is ... (--)\n");
                printf("  Any normal file name\n");
                printf("  -- for stdin\n");
                exit(EXIT_FAILURE);
                break;
            default:
                printf("ERROR: Something wrong in getopt\n");
                exit(EXIT_FAILURE);
                break;
        }
    }

//----Check for problem file name
    if (argv[optind] != NULL) {
        strcpy(OptionValues->DerivationFileName,argv[optind]);
    }

//----Echo if in verbose mode
    if (OptionValues->Quietness == 0) {
        printf("Command line options:\n");
        printf("    Quietness %d\n",OptionValues->Quietness);
        printf("    ForceContinue %d\n",OptionValues->ForceContinue);
        printf("    VerifyLeaves %s\n",OptionValues->VerifyLeaves?
(!strcmp(OptionValues->ProblemFileName,"")?"from derivation":
OptionValues->ProblemFileName):"0");
        printf("    NoExpensiveChecks %d\n",OptionValues->NoExpensiveChecks);
        printf("    NoIntroducedAxioms %d\n",OptionValues->NoIntroducedAxioms);
        printf("    ForceContinue %d\n",OptionValues->ForceContinue);
        printf("    CheckParentRelevance %d\n",
OptionValues->CheckParentRelevance);
        printf("    CheckRefutation %d\n",OptionValues->CheckRefutation);
        printf("    TheoremProver %s\n",OptionValues->TheoremProver);
        printf("    ModelFinder %s\n",OptionValues->ModelFinder);
        printf("    Saturator %s\n",OptionValues->Saturator);
        printf("    UnsatisfiabilityChecker %s\n",
OptionValues->UnsatisfiabilityChecker);
        printf("    CounterTheoremProver %s\n",
OptionValues->CounterTheoremProver);
        printf("    TimeLimit %d\n",OptionValues->TimeLimit);
        printf("    KeepFiles %s\n",OptionValues->KeepFiles?
OptionValues->KeepFilesDirectory:"0");
        printf("    DerivationFileName %s\n",OptionValues->DerivationFileName);
    }

    return(1);
}
//-----------------------------------------------------------------------------
double GetTotalCPUTime(void) {

    struct rusage MyTime;
    struct rusage ChildTime;

    if (getrusage(RUSAGE_SELF,&MyTime) == 0 && getrusage(RUSAGE_CHILDREN,
&ChildTime) == 0) {
        return(
MyTime.ru_utime.tv_sec + MyTime.ru_utime.tv_usec/1000000.0 + 
ChildTime.ru_utime.tv_sec + ChildTime.ru_utime.tv_usec/1000000.0 +
MyTime.ru_stime.tv_sec + MyTime.ru_stime.tv_usec/1000000.0 + 
ChildTime.ru_stime.tv_sec + ChildTime.ru_stime.tv_usec/1000000.0
);
    } else {
        return(-1.0);
    }
}
//-----------------------------------------------------------------------------
char * MakePrintableList(StringParts Names,int NumberOfNames,String ListNames) {

    int NameNumber;

    strcpy(ListNames,"");
    for (NameNumber = 0;NameNumber < NumberOfNames;NameNumber++) {
        strcat(ListNames,Names[NameNumber]);
        strcat(ListNames," ");
    }

    return(ListNames);
}
//-----------------------------------------------------------------------------
int IsNewPredicate(char * Predicate) {

    static char * AllNewSymbols = NULL;
    static int LengthNewSymbols = 0;
    String PrincipleSymbol;

//----First time Malloc the memory
    if (LengthNewSymbols == 0) {
        AllNewSymbols = (char *)Malloc(sizeof(String));
        LengthNewSymbols = STRINGLENGTH;
    }

    if (NameInList(PrincipleSymbol,AllNewSymbols)) {
        return(0);
    } else {
//----Need local in String to ensure it can be extended
        strcpy(PrincipleSymbol,Predicate);
        strcat(PrincipleSymbol,"\n");
        ExtendString(&AllNewSymbols,PrincipleSymbol,&LengthNewSymbols);
        return(1);
    }
}
//-----------------------------------------------------------------------------
void EmptyAndDeleteDirectory(String Directory) {

    String UNIXCommand;

    sprintf(UNIXCommand,"rm -rf %s",Directory);
    system(UNIXCommand);
}
//-----------------------------------------------------------------------------
int CreateDirectory(String Directory,String DerivationFileName) {

    String DerivationFileBasename;

    if (!strcmp(DerivationFileName,"--")) {
        sprintf(DerivationFileBasename,"GDV%06d",getpid());
    } else {
        PathBasename(DerivationFileName,DerivationFileBasename,NULL);
    }

    strcat(Directory,"/");
    strcat(Directory,DerivationFileBasename);
    strcat(Directory,".gdv");

//----Delete any previous version
    EmptyAndDeleteDirectory(Directory);
//----And make the new one
    if (mkdir(Directory,0755) != 0) {
        perror("Creating working directory");
        return(0);
    } else {
        return(1);
    }
}
//-----------------------------------------------------------------------------
void AddVerifiedTag(ANNOTATEDFORMULA AnnotatedFormula,SIGNATURE Signature,
char * TagValue) {

    String VerifiedTag;

    sprintf(VerifiedTag,"verified(%s)",TagValue);
    AddUsefulInformationToAnnotatedFormula(AnnotatedFormula,Signature,
VerifiedTag);
}
//-----------------------------------------------------------------------------
int GDVCheckTheorem(OptionsType OptionValues,LISTNODE Axioms,
ANNOTATEDFORMULA Conjecture,char * FileBaseName,char * Extension,
int TestCounterTheorem) {

    String OutputFileName;

    strcpy(OutputFileName,FileBaseName);
    strcat(OutputFileName,".");
    strcat(OutputFileName,Extension);
    strcat(OutputFileName,".dis");

    return(SystemOnTPTP(Axioms,Conjecture,OptionValues.TheoremProver,"Theorem",
TestCounterTheorem,OptionValues.CounterTheoremProver,"CounterSatisfiable",
OptionValues.TimeLimit," ",OptionValues.KeepFiles,
OptionValues.KeepFilesDirectory,OutputFileName,OutputFileName));
}
//-----------------------------------------------------------------------------
int GDVCheckSatisfiable(OptionsType OptionValues,LISTNODE Formulae,
char * FileBaseName,char * Extension,int TestUnsatisfiability) {

    String OutputFileName;
    int CheckResult;
    FILE * Handle;

    if (ListOfAnnotatedFormulaTrueInInterpretation(Formulae,positive)) {
        if (OptionValues.KeepFiles) {
            strcpy(OutputFileName,FileBaseName);
            strcat(OutputFileName,".");
            strcat(OutputFileName,Extension);
            strcat(OutputFileName,"_positive.dis");
            SystemOnTPTPFileName(OptionValues.KeepFilesDirectory,
OutputFileName,NULL,OutputFileName);
            if ((Handle = OpenFileInMode(OutputFileName,"w")) != NULL) {
                fclose(Handle);
            }
        }
        return(1);
    } else if (ListOfAnnotatedFormulaTrueInInterpretation(Formulae,negative)) {
        if (OptionValues.KeepFiles) {
            strcpy(OutputFileName,FileBaseName);
            strcat(OutputFileName,".");
            strcat(OutputFileName,Extension);
            strcat(OutputFileName,"_negative.dis");
            SystemOnTPTPFileName(OptionValues.KeepFilesDirectory,
OutputFileName,NULL,OutputFileName);
            if ((Handle = OpenFileInMode(OutputFileName,"w")) != NULL) {
                fclose(Handle);
            }
        }
        return(1);
//----Try finite model finder
    } else {
        strcpy(OutputFileName,FileBaseName);
        strcat(OutputFileName,".");
        strcat(OutputFileName,Extension);
        strcat(OutputFileName,"_model.dis");
        if ((CheckResult = SystemOnTPTP(Formulae,NULL,
OptionValues.ModelFinder,"Satisfiable",TestUnsatisfiability,
OptionValues.UnsatisfiabilityChecker,"Unsatisfiable",OptionValues.TimeLimit,
" ",OptionValues.KeepFiles,OptionValues.KeepFilesDirectory,OutputFileName,
OutputFileName)) != 0) {
            return(CheckResult);
        } else {
            strcpy(OutputFileName,FileBaseName);
            strcat(OutputFileName,".");
            strcat(OutputFileName,Extension);
            strcat(OutputFileName,"_saturate.dis");
            return(SystemOnTPTP(Formulae,NULL,OptionValues.Saturator,
"Satisfiable",0,NULL,NULL,OptionValues.TimeLimit," ",OptionValues.KeepFiles,
OptionValues.KeepFilesDirectory,OutputFileName,OutputFileName));
        }
    }
}
//-----------------------------------------------------------------------------
int CorrectlyInferred(OptionsType OptionValues,ANNOTATEDFORMULA Target,
String FormulaName,LISTNODE ParentAnnotatedFormulae,char * ParentNames,
String SZSStatus,char * FileBaseName,char * Comment) {

    int Correct;
    ANNOTATEDFORMULA NewTarget;
    String NewFormulaName;
    String NewParentNames;
    int CheckResult;

    if (!strcmp(SZSStatus,"thm")) {
        if (OptionValues.CheckParentRelevance) {
            if ((CheckResult = GDVCheckSatisfiable(OptionValues,
ParentAnnotatedFormulae,FileBaseName,"parents.sat",1)) == 1) {
                Correct = 1;
                QPRINTF(OptionValues,2)(
"SUCCESS: %s has sat parents %s %s\n",FormulaName,ParentNames,
Comment != NULL?Comment:"");
//----Looked for unsat and didn't find it, so that's also OK
            } else if (CheckResult == 0) {
                Correct = 1;
                QPRINTF(OptionValues,2)(
"SUCCESS: %s does not have unsat parents %s %s\n",FormulaName,ParentNames,
Comment != NULL?Comment:"");
//----If single parent from the negated conjecture then can be unsatisfiable 
            } else if (OptionValues.CheckRefutation &&
ParentAnnotatedFormulae->Next == NULL && GetStatus(
ParentAnnotatedFormulae->AnnotatedFormula,NULL) == negated_conjecture) {
                Correct = 1;
                QPRINTF(OptionValues,1)(
"WARNING: %s does not have sat parents %s, tolerated because one is a negated_conjecture %s\n",FormulaName,ParentNames,Comment != NULL?Comment:"");
//----Parents of false formulae can be satisfiable
            } else if (OptionValues.CheckRefutation && 
FalseAnnotatedFormula(Target)) {
                Correct = 1;
                QPRINTF(OptionValues,1)(
"WARNING: %s does not have sat parents %s, tolerated because it's $false %s\n",
FormulaName,ParentNames,Comment != NULL?Comment:"");
            } else {
                Correct = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: %s does not have sat parents %s %s\n",FormulaName,ParentNames,
Comment != NULL?Comment:"");
            }
        } else {
            Correct = 1;
        }

        if (!GlobalInterrupted && (Correct || OptionValues.ForceContinue)) {
            if ((CheckResult = GDVCheckTheorem(OptionValues,
ParentAnnotatedFormulae,Target,FileBaseName,"thm",1)) == 1) {
                Correct = 1;
                QPRINTF(OptionValues,2)(
"SUCCESS: %s is a %s of %s %s\n", FormulaName,SZSStatus,ParentNames == NULL?
"itself":ParentNames,Comment != NULL?Comment:"");
            } else if (CheckResult == 0) {
                Correct = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: %s fails to be a %s of %s %s\n",FormulaName,SZSStatus,
ParentNames == NULL?"itself":ParentNames,Comment != NULL?Comment:"");
            } else {
                Correct = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: %s is not a %s of %s %s\n",FormulaName,SZSStatus,ParentNames == NULL?
"itself":ParentNames,Comment != NULL?Comment:"");
            }
        }
        return(Correct);

    } else if (!strcmp(SZSStatus,"cth")) {
        Negate(Target,0);
        Correct = CorrectlyInferred(OptionValues,Target,FormulaName,
ParentAnnotatedFormulae,ParentNames,"thm",FileBaseName,"(Negated cth)");
        Negate(Target,1);
        return(Correct);

    } else if (!strcmp(SZSStatus,"sab")) {
//----This is the weak reverse check
        NewTarget = ParentAnnotatedFormulae->AnnotatedFormula;
        ParentAnnotatedFormulae->AnnotatedFormula = Target;
        Correct = CorrectlyInferred(OptionValues,NewTarget,
GetName(NewTarget,NewFormulaName),ParentAnnotatedFormulae,
GetName(ParentAnnotatedFormulae->AnnotatedFormula,NewParentNames),"thm",
FileBaseName,"(Inverted sab)");
        ParentAnnotatedFormulae->AnnotatedFormula = NewTarget;
        if (Correct) {
            QPRINTF(OptionValues,2)(
"WARNING: Incomplete check of SZS status sab\n");
        }
        return(Correct);

    } else {
        QPRINTF(OptionValues,1)(
"WARNING: Cannot cope with SZS status %s\n",SZSStatus);
        return(0);
    }
}
//-----------------------------------------------------------------------------
ANNOTATEDFORMULA MakeSplitDefinitionFor(ANNOTATEDFORMULA SplitChild,
SIGNATURE Signature) {

    String FormulaName;
    ANNOTATEDFORMULA SplitDefinition;
    FORMULA Formula;
    String CreatorString;

//----Check it is a disjunction
    Formula = SplitChild->AnnotatedFormulaUnion.AnnotatedTSTPFormula.
FormulaWithVariables->Formula;
    if (Formula->Type != binary || 
Formula->FormulaUnion.BinaryFormula.Connective != disjunction || 
Formula->FormulaUnion.BinaryFormula.LHS->Type != atom ||
GetArity(Formula->FormulaUnion.BinaryFormula.LHS->FormulaUnion.Atom) != 0) {
        return(NULL);
    }

    strcpy(FormulaName,GetSymbol(Formula->FormulaUnion.BinaryFormula.LHS->
FormulaUnion.Atom));
    strcat(FormulaName,"_defn");
    SplitDefinition = DuplicateAnnotatedFormula(SplitChild,Signature);
    Formula = SplitDefinition->AnnotatedFormulaUnion.AnnotatedTSTPFormula.
FormulaWithVariables->Formula;
    SetSyntax(SplitDefinition,tptp_fof);
    SetName(SplitDefinition,FormulaName);
    SetStatus(SplitDefinition,definition,nonstatus);
    Formula->FormulaUnion.BinaryFormula.Connective = equivalence;
    UniversalQuantifyFormula(&(Formula->FormulaUnion.BinaryFormula.RHS),
SplitDefinition->AnnotatedFormulaUnion.AnnotatedTSTPFormula.
FormulaWithVariables->Variables);
    NegateFormula(&(Formula->FormulaUnion.BinaryFormula.RHS));
    EnsureShortForm(SplitDefinition);
    sprintf(CreatorString,"introduced(definition,[esplit_defn(%s)])",
GetName(SplitChild,FormulaName));
    SetSourceFromString(SplitDefinition,Signature,CreatorString);

    return(SplitDefinition);
}
//-----------------------------------------------------------------------------
int MakeESplitDefinitions(OptionsType OptionValues,LISTNODE Head,SIGNATURE
Signature,LISTNODE * SplitDefinitions) {

    LISTNODE Target;
    LISTNODE Sibling;
    LISTNODE SplitSiblings;
    int OKSoFar;
    String InferenceRule;
    String InferenceInfo;
    String ProcessedTag;
    String AllParentNames;
    String ParentName;
    String SiblingName;
    String AllSiblingParentNames;
    int SiblingNumber;
    ANNOTATEDFORMULA SplitDefinition;
    String SplitDefinitionName;
    String SplitSiblingsNames;

    *SplitDefinitions = NULL;
    OKSoFar = 1;
    Target = Head;
    while (OKSoFar && Target != NULL) {
        GetName(Target->AnnotatedFormula,SiblingName);
//----Check if derived by an esplit and not already processed
        GetInferenceRule(Target->AnnotatedFormula,InferenceRule);
        if (GetInferenceInfoTerm(Target->AnnotatedFormula,InferenceRule,
InferenceInfo) != NULL && ExtractTermArguments(InferenceInfo) &&
strstr(InferenceInfo,"esplit,") == InferenceInfo && GetUsefulInfoTerm(
Target->AnnotatedFormula,"psuedo_split_from",1,ProcessedTag) == NULL) {
//----Get the split parent
            GetNodeParentNames(Target->AnnotatedFormula,AllParentNames);
//----Crude check that there's just one parent
            if (strchr(AllParentNames,'\n') != strrchr(AllParentNames,'\n')) {
                QPRINTF(OptionValues,2)(
"FAILURE: %s has been esplit without defns from more than one parent\n",
SiblingName);
                OKSoFar = 0;
            } else {
                SplitSiblings = NULL;
                AddListNode(&SplitSiblings,SplitSiblings,Target->
AnnotatedFormula);

                strcpy(ParentName,AllParentNames);
                *strchr(ParentName,'\n') = '\0';
                sprintf(ProcessedTag,"psuedo_split_from(%s)",ParentName);
                AddUsefulInformationToAnnotatedFormula(Target->AnnotatedFormula,
Signature,ProcessedTag);

                strcpy(SplitSiblingsNames,"psuedo_split_to(");
                strcat(SplitSiblingsNames,SiblingName);

//----Go through and look for split siblings (with same parent)
                Sibling = Target->Next;
                while (OKSoFar && Sibling != NULL) {
                    if (GetInferenceInfoTerm(Sibling->AnnotatedFormula,
InferenceRule,InferenceInfo) != NULL && ExtractTermArguments(InferenceInfo) &&
strstr(InferenceInfo,"esplit,") == InferenceInfo && GetUsefulInfoTerm(
Sibling->AnnotatedFormula,"psuedo_split_from",1,ProcessedTag) == NULL) {
                        GetName(Sibling->AnnotatedFormula,SiblingName);
                        GetNodeParentNames(Sibling->AnnotatedFormula,
AllSiblingParentNames);
//----Another sibling
                        if (!strcmp(AllParentNames,AllSiblingParentNames)) {
                            AddListNode(&SplitSiblings,SplitSiblings,
Sibling->AnnotatedFormula);
                            AddUsefulInformationToAnnotatedFormula(
Sibling->AnnotatedFormula,Signature,ProcessedTag);
                            strcat(SplitSiblingsNames,",");
                            strcat(SplitSiblingsNames,SiblingName);
                        }
                    }
                    Sibling = Sibling->Next;
                }
                strcat(SplitSiblingsNames,")");
                AddUsefulInformationToAnnotatedFormula(
GetAnnotatedFormulaFromListByName(Head,ParentName),Signature,
SplitSiblingsNames);

//----Make the definitions for all except the first in the list (which was
//----the last found and I assume to be the link clause)
                Sibling = SplitSiblings->Next;
                SiblingNumber = 1;
                while (OKSoFar && Sibling != NULL) {
                    GetName(Sibling->AnnotatedFormula,SiblingName);
                    if ((SplitDefinition = MakeSplitDefinitionFor(
Sibling->AnnotatedFormula,Signature)) == NULL) {
                        QPRINTF(OptionValues,2)(
"FAILURE: %s cannot be converted to a split definition\n",SiblingName);
                    } else {
                        AddListNode(SplitDefinitions,*SplitDefinitions,
SplitDefinition);
                        sprintf(ProcessedTag,"esplit_defn(%s)",
GetName(SplitDefinition,SplitDefinitionName));
                        AddUsefulInformationToAnnotatedFormula(
Sibling->AnnotatedFormula,Signature,ProcessedTag);
                    }
                    Sibling = Sibling->Next;
                }
                FreeListOfAnnotatedFormulae(&SplitSiblings);
            }
        }
    Target = Target->Next;
    }

    if (!OKSoFar) {
        FreeListOfAnnotatedFormulae(SplitDefinitions);
    }

//DEBUG PrintListOfAnnotatedTSTPNodes(stdout,*SplitDefinitions,tptp,1);

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int GetSPASSSplitPosition(char * InferenceInfo,char * PutSplitPositionHere) {

    char * Start;

    strcpy(PutSplitPositionHere,InferenceInfo);
    if ((Start = strstr(PutSplitPositionHere,"position(")) != NULL &&
Start[strlen(Start)-1] == ']') {
        Start[strlen(Start)-1] = '\0';
        strcpy(PutSplitPositionHere,Start);
        return(ExtractTermArguments(PutSplitPositionHere));
    } else {
        return(0);
    }
}
//-----------------------------------------------------------------------------
int InsertSPASSSplitInfo(OptionsType OptionValues,LISTNODE Head,
SIGNATURE Signature,int * NumberOfExplicitSplits) {

    int OKSoFar;
    LISTNODE Target;
    LISTNODE Sibling;
    String InferenceRule;
    String InferenceInfo;
    String ProcessedTag;
    String AllParentNames;
    String ParentName;
    String SiblingName;
    String AllSiblingParentNames;
    String SplitPosition;
    String NextSplitPosition;
    String SplitSiblingsNames;

    OKSoFar = 1;
    *NumberOfExplicitSplits = 0;
    Target = Head;
    while (OKSoFar && Target != NULL) {
        GetName(Target->AnnotatedFormula,SiblingName);
//----Check if derived by an sr_split and not already processed
        GetInferenceRule(Target->AnnotatedFormula,InferenceRule);
        if (GetInferenceInfoTerm(Target->AnnotatedFormula,InferenceRule,
InferenceInfo) != NULL && ExtractTermArguments(InferenceInfo) &&
strstr(InferenceInfo,"sr_split,") == InferenceInfo && GetUsefulInfoTerm(
Target->AnnotatedFormula,"explicit_split_from",1,ProcessedTag) == NULL &&
//----Get the split position - must be a left
GetSPASSSplitPosition(InferenceInfo,SplitPosition) &&
SplitPosition[strlen(SplitPosition)-1] == 'l') {
//----Get the split parent
            GetNodeParentNames(Target->AnnotatedFormula,AllParentNames);
//----Crude check that there's just one parent
            if (strchr(AllParentNames,'\n') != strrchr(AllParentNames,'\n')) {
                QPRINTF(OptionValues,2)(
"FAILURE: %s has been sr_split from more than one parent\n",SiblingName);
                OKSoFar = 0;
            }
            if (OKSoFar) {
                (*NumberOfExplicitSplits)++;

                strcpy(ParentName,AllParentNames);
                *strchr(ParentName,'\n') = '\0';
                sprintf(ProcessedTag,"explicit_split_from(%s)",ParentName);
                AddUsefulInformationToAnnotatedFormula(Target->AnnotatedFormula,
Signature,ProcessedTag);
                strcpy(SplitSiblingsNames,"explicit_split_to(");
                strcat(SplitSiblingsNames,SiblingName);

//----Look for the right child next
                strcpy(NextSplitPosition,SplitPosition);
                NextSplitPosition[strlen(NextSplitPosition)-1] = 'r';
//----Go through and look for split siblings (with same parent)
                Sibling = Target->Next;
                while (OKSoFar && Sibling != NULL) {
                    if (GetInferenceInfoTerm(Sibling->AnnotatedFormula,
InferenceRule,InferenceInfo) != NULL && ExtractTermArguments(InferenceInfo) &&
strstr(InferenceInfo,"sr_split,") == InferenceInfo && GetUsefulInfoTerm(
Sibling->AnnotatedFormula,"psuedo_split_from",1,ProcessedTag) == NULL) {
                        GetName(Sibling->AnnotatedFormula,SiblingName);
                        GetNodeParentNames(Sibling->AnnotatedFormula,
AllSiblingParentNames);
//----The sibling or anti-kid
                        if (!strcmp(AllParentNames,AllSiblingParentNames) &&
GetSPASSSplitPosition(InferenceInfo,SplitPosition) &&
!strcmp(SplitPosition,NextSplitPosition)) {
                            AddUsefulInformationToAnnotatedFormula(
Sibling->AnnotatedFormula,Signature,ProcessedTag);
                            strcat(SplitSiblingsNames,",");
                            strcat(SplitSiblingsNames,SiblingName);
                            strcat(SplitSiblingsNames,")");
//----Stop the search and clear the required split position
                            Sibling = NULL;
                            strcpy(NextSplitPosition,"");
                        } else {
                            Sibling = Sibling->Next;
                        }
                    } else {
                        Sibling = Sibling->Next;
                    }
                }
//----By now the next required split position should be empty
                if (strcmp(NextSplitPosition,"")) {
                    QPRINTF(OptionValues,2)(
"FAILURE: %s has no explicit split right child\n",ParentName);
                    OKSoFar = 0;
                } else {
                    AddUsefulInformationToAnnotatedFormula(
GetAnnotatedFormulaFromListByName(Head,ParentName),Signature,
SplitSiblingsNames);
                }
            }
        }
        Target = Target->Next;
    }
    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int StructuralCompletion(OptionsType OptionValues,LISTNODE * Head,
SIGNATURE Signature) {

    int OKSoFar;
    LISTNODE SplitDefinitions;
    LISTNODE TargetDefinition;
    int NumberOfInstances;

    OKSoFar = 1;

//----Add definitions for E's psuedo splitting if not expected
//----Hopefully this will be unnecessary in the future 
    if (!GlobalInterrupted && OKSoFar && OptionValues.NoIntroducedAxioms) {
        if (MakeESplitDefinitions(OptionValues,*Head,Signature,
&SplitDefinitions)) {
//----Report only if there are some
            if (SplitDefinitions != NULL) {
                TargetDefinition = SplitDefinitions;
                while (TargetDefinition != NULL) {
                    AddListNode(Head,*Head,TargetDefinition->AnnotatedFormula);
                    TargetDefinition = TargetDefinition->Next;
                }
                QPRINTF(OptionValues,2)(
"SUCCESS: All esplits have had definitions added\n");
                FreeListOfAnnotatedFormulae(&SplitDefinitions);
            }
        } else {
            OKSoFar = 0;
        }
    }

    if (!GlobalInterrupted && OKSoFar) {
        if (InsertSPASSSplitInfo(OptionValues,*Head,Signature,
&NumberOfInstances)) {
//----Report only if there are some
            if (NumberOfInstances > 0) {
                QPRINTF(OptionValues,2)(
"SUCCESS: All sr_splits have both children in place\n");
            }
        } else {
            OKSoFar = 0;
        }
    }

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int MakeListOfParents(OptionsType OptionValues,LISTNODE Head,
String FormulaName,StringParts ParentNames,int NumberOfParents,
LISTNODE * ParentList) {

    int ParentNumber;
    ANNOTATEDFORMULA ParentAnnotatedFormula;

    *ParentList = NULL;
    for (ParentNumber = 0;ParentNumber < NumberOfParents;ParentNumber++) {
        if ((ParentAnnotatedFormula = GetAnnotatedFormulaFromListByName(
Head,ParentNames[ParentNumber])) == NULL) {
            QPRINTF(OptionValues,2)(
"FAILURE: %s has a non-existent parent %s\n",FormulaName,
ParentNames[ParentNumber]);
            FreeListOfAnnotatedFormulae(ParentList);
            return(0);
        } else {
            AddListNode(ParentList,NULL,ParentAnnotatedFormula);
            ParentList = &((*ParentList)->Next);
        }
    }

    return(1);
}
//-----------------------------------------------------------------------------
int AllParentsExist(OptionsType OptionValues,LISTNODE Head) {

    LISTNODE Target;
    int OKSoFar;
    String AllParentNames;
    StringParts ParentNames;
    int NumberOfParents;
    String FormulaName;
    LISTNODE ParentList;
    String InferenceRule;

    OKSoFar = 1;

    Target = Head;
    while (OKSoFar && Target != NULL) {
        if (DerivedAnnotatedFormula(Target->AnnotatedFormula)) {
            GetName(Target->AnnotatedFormula,FormulaName);
            if (GetInferenceRule(Target->AnnotatedFormula,InferenceRule) == 
NULL) {
                QPRINTF(OptionValues,2)(
"FAILURE: %s has no inference rule\n",FormulaName);
                OKSoFar = 0;
            } else {
                GetNodeParentNames(Target->AnnotatedFormula,AllParentNames);
                if ((NumberOfParents = Tokenize(AllParentNames,ParentNames,
"\n")) == 0) {
                    QPRINTF(OptionValues,1)(
"WARNING: %s is derived from no parents\n",FormulaName);
                }
                if (!strcmp(InferenceRule,"") && NumberOfParents != 1) {
                    QPRINTF(OptionValues,2)(
"FAILURE: %s is a copy of not one parent\n",FormulaName);
                    OKSoFar = 0;
                } else if (MakeListOfParents(OptionValues,Head,FormulaName,
ParentNames,NumberOfParents,&ParentList)) {
                    FreeListOfAnnotatedFormulae(&ParentList);
                } else {
                    OKSoFar = 0;
                }
            }
        }
        Target = Target->Next;
    }

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int CyclicTree(TREENODE TreeRoot) {

    int ParentIndex;
//DEBUG String FormulaName;

//DEBUG printf("check cycle at %s\n",GetName(TreeRoot->AnnotatedFormula,FormulaName));
//----If listnode already marked, we're done
    if (TreeRoot->Visited == 2) {
//DEBUG printf("already removed %s\n",FormulaName);
        return(0);
    }

//----If list node previous in chain, error
    if (TreeRoot->Visited == 1) {
//DEBUG printf("in a loop at %s\n",FormulaName);
        return(1);
    }

    TreeRoot->Visited = 1;
    for (ParentIndex = 0;ParentIndex < TreeRoot->NumberOfParents;
ParentIndex++) {
        if (CyclicTree(TreeRoot->Parents[ParentIndex])) {
//DEBUG printf("more of loop at %s\n",FormulaName);
            return(1);
        }
    }

//----Mark the subtree as clear
//DEBUG printf("subtree clear at %s\n",FormulaName);
    TreeRoot->Visited = 2;
    return(0);
}
//-----------------------------------------------------------------------------
int CyclicRootList(ROOTLIST RootListHead) {

    ResetRootListVisited(RootListHead);
    while (RootListHead != NULL) {
        if (CyclicTree(RootListHead->TheTree)) {
            return(1);
        }
        RootListHead = RootListHead->Next;
    }
    ResetRootListVisited(RootListHead);
    return(0);
}
//-----------------------------------------------------------------------------
ROOTLIST GetFalseNodes(ROOTLIST RootListHead,LISTNODE Head) {

    ROOTLIST FalseList;
    TREENODE FalseNode;

    FalseList = NULL;
    while (Head != NULL) {
        if (FalseAnnotatedFormula(Head->AnnotatedFormula)) {
            if ((FalseNode = AnnotatedFormulaInTrees(RootListHead,
Head->AnnotatedFormula)) == NULL) {
                CodingError("False node is not in any tree");
            } else {
                AddRootNode(&FalseList,FalseList,FalseNode);
            }
        }
        Head = Head->Next;
    }
    return(FalseList);
}
//-----------------------------------------------------------------------------
ROOTLIST GetFalseRootList(ROOTLIST RootListHead) {

    ROOTLIST FalseRootList;

    FalseRootList = NULL;
    while (RootListHead != NULL) {
        if (FalseAnnotatedFormula(RootListHead->TheTree->AnnotatedFormula)) {
            AddRootNode(&FalseRootList,FalseRootList,RootListHead->TheTree);
        }
        RootListHead = RootListHead->Next;
    }

    return(FalseRootList);
}
//-----------------------------------------------------------------------------
TREENODE GetFalseRootNode(ROOTLIST RootListHead) {

    while (RootListHead != NULL) {
        if (FalseAnnotatedFormula(RootListHead->TheTree->AnnotatedFormula)) {
            return(RootListHead->TheTree);
        }
        RootListHead = RootListHead->Next;
    }

    return(NULL);
}
//-----------------------------------------------------------------------------
TREENODE CheckFalseRootNode(OptionsType OptionValues,ROOTLIST RootListHead) {

    TREENODE FalseRoot;

    if ((FalseRoot = GetFalseRootNode(RootListHead)) == NULL) {
        QPRINTF(OptionValues,2)("FAILURE: Derivation has no false roots\n");
    }

    return(FalseRoot);
}
//-----------------------------------------------------------------------------
//----Must replace this by stuff from Tree.c
int IsAncestor(LISTNODE Head,ANNOTATEDFORMULA Ancestor,ANNOTATEDFORMULA 
Descendant,int THMNodesOnly) {

    String AncestorName;
    String AllParentNames;
    int NumberOfParents;
    StringParts ParentNames;
    int ParentNumber;
    ANNOTATEDFORMULA Parent;
    String DescendantName;
    String ParentStatus;

    GetName(Ancestor,AncestorName);
    GetName(Descendant,DescendantName);
    GetNodeParentNames(Descendant,AllParentNames);
    NumberOfParents = Tokenize(AllParentNames,ParentNames,"\n");
    for (ParentNumber=0;ParentNumber < NumberOfParents;ParentNumber++) {
        if (!strcmp(AncestorName,ParentNames[ParentNumber])) {
            return(1);
        } else {
            Parent = GetAnnotatedFormulaFromListByName(Head,
ParentNames[ParentNumber]);
            if ((!THMNodesOnly || 
(GetInferenceInfoTerm(Parent,"status",ParentStatus) != NULL && 
!strcmp(ParentStatus,"status(thm)"))) &&
IsAncestor(Head,Ancestor,Parent,THMNodesOnly)) {
                return(1);
            }
        }
    }
    return(0);
}
//-----------------------------------------------------------------------------
TREENODE DoAnnotatedFormulaInTreeTHM(TREENODE ATree,ANNOTATEDFORMULA
LookingForThis,int THMNodesOnly) {

    int ParentIndex;
    TREENODE AncestorNode;
    String ParentStatus;

    if (ATree != NULL) {
        if (!ATree->Visited) {
//----Must mark visited here because of the return in the middle, which
//----might leave it unmarked, and therefore the reset does not go up.
            ATree->Visited = 1;
//----Is this the one?
            if (ATree->AnnotatedFormula == LookingForThis) {
                return(ATree);
            } else {
//----Look in subtrees subject to THM veto
                if (!THMNodesOnly || (GetInferenceInfoTerm(ATree->
AnnotatedFormula,"status",ParentStatus) != NULL && !strcmp(ParentStatus,
"status(thm)"))) {
                    for (ParentIndex = 0; ParentIndex < ATree->NumberOfParents;
ParentIndex++) {
                        if ((AncestorNode = DoAnnotatedFormulaInTreeTHM(ATree->
Parents[ParentIndex],LookingForThis,THMNodesOnly)) != NULL) {
                            return(AncestorNode);
                        }
                    }
                }
            }
        } else {
            return(NULL);
        }
    }

    return(NULL);
}
//-----------------------------------------------------------------------------
TREENODE AnnotatedFormulaInTreeTHM(TREENODE ATree,ANNOTATEDFORMULA
LookingForThis,int THMNodesOnly) {

    TREENODE TreeNodeFound;

    ResetTreeVisited(ATree);
    TreeNodeFound = DoAnnotatedFormulaInTreeTHM(ATree,LookingForThis,
THMNodesOnly);
    ResetTreeVisited(ATree);
    return(TreeNodeFound);
}
//-----------------------------------------------------------------------------
TREENODE AnnotatedFormulaInTreesTHM(ROOTLIST RootListHead,ANNOTATEDFORMULA
LookingForThis,int THMNodesOnly) {

    TREENODE TreeNode;

    while (RootListHead != NULL) {
        if ((TreeNode = AnnotatedFormulaInTreeTHM(RootListHead->TheTree,
LookingForThis,THMNodesOnly)) != NULL) {
            return(TreeNode);
        } else {
            RootListHead = RootListHead->Next;
        }
    }
    return(NULL);
}
//-----------------------------------------------------------------------------
int UsesFormulae(OptionsType OptionValues,LISTNODE Head,ROOTLIST RootListHead) {

    LISTNODE Target;
    StatusType Status;
    String FormulaName;
    int FoundAConjecture;
    int OKSoFar;

    FoundAConjecture = 0;
    OKSoFar = 1;
    Target = Head;
    while (Target != NULL) {
        if (!DerivedAnnotatedFormula(Target->AnnotatedFormula)) {
            if ((Status = GetStatus(Target->AnnotatedFormula,NULL)) == 
conjecture || Status == negated_conjecture) {
                FoundAConjecture = 1;
            }
            if (AnnotatedFormulaInTreesTHM(RootListHead,Target->
AnnotatedFormula,0) == NULL) {
                QPRINTF(OptionValues,2)("WARNING: Leaf %s is not used\n",
GetName(Target->AnnotatedFormula,FormulaName));
                OKSoFar = 0;
            }
        }
        Target = Target->Next;
    } 

    if (!FoundAConjecture) {
        QPRINTF(OptionValues,2)(
"WARNING: (Negated) leaf conjecture not found\n");
        OKSoFar = 0;
    }

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int WellFormedProofsByContradiction(OptionsType OptionValues,LISTNODE Head,
SIGNATURE Signature,int * NumberOfProofsByContradiction) {

    LISTNODE Target;
    int OKSoFar;
    String FormulaName;
    String AllParentNames;
    int NumberOfParents;
    StringParts ParentNames;
    ANNOTATEDFORMULA FalseParent;
    ANNOTATEDFORMULA AncestorParent;
    ANNOTATEDFORMULA SwapParent;
    char * AncestorName;
    String ProcessedTag;

    OKSoFar = 1;
    *NumberOfProofsByContradiction = 0;
    Target = Head;
    while (OKSoFar && Target != NULL) {
        GetName(Target->AnnotatedFormula,FormulaName);
        GetNodeParentNames(Target->AnnotatedFormula,AllParentNames);
        NumberOfParents = Tokenize(AllParentNames,ParentNames,"\n");
        if (NumberOfParents == 2) {
            FalseParent = GetAnnotatedFormulaFromListByName(Head,
ParentNames[0]);
            AncestorParent = GetAnnotatedFormulaFromListByName(Head,
ParentNames[1]);
            if (FalseAnnotatedFormula(FalseParent)) {
                AncestorName = ParentNames[1];
            } else if (FalseAnnotatedFormula(AncestorParent)) {
                SwapParent = FalseParent;
                FalseParent = AncestorParent;
                AncestorParent = SwapParent;
                AncestorName = ParentNames[0];
            } else {
                FalseParent = NULL;
            }
            if (FalseParent != NULL) {
//----Shit, must allow non thm inferences in this trail due to splits.
//----This PbC is still unclear of course.
                if (!IsAncestor(Head,AncestorParent,FalseParent,0)) {
                    QPRINTF(OptionValues,2)(
"FAILURE: %s has a badly formed proof by contradiction\n",FormulaName);
                    OKSoFar = 0;
                } else {
                    (*NumberOfProofsByContradiction)++;
                    sprintf(ProcessedTag,"proved_by_contradiction(%s)",
AncestorName);
                    AddUsefulInformationToAnnotatedFormula(
Target->AnnotatedFormula,Signature,ProcessedTag);
                }
            }
        }
        Target = Target->Next;
    }

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int ExplicitSplitsIndependent(OptionsType OptionValues,LISTNODE Head,
ROOTLIST RootListHead,int * NumberOfExplicitSplits) {

    int OKSoFar;
    LISTNODE Target;
    ROOTLIST InnerTarget;
    String FormulaName;
    int OccurenceNumber;
    String UsefulInfo;
    StringParts SplitChildrenNames;
    ANNOTATEDFORMULA SplitChild1;
    ANNOTATEDFORMULA SplitChild2;
    String InnerName;
    TREENODE RefutationRoot;
    ROOTLIST FalseList;

    RefutationRoot = GetFalseRootNode(RootListHead);
    FalseList = GetFalseNodes(RootListHead,Head);
    OKSoFar = 1;
    *NumberOfExplicitSplits = 0;
    Target = Head;
    while (OKSoFar && Target != NULL) {
        OccurenceNumber = 1;
//----Look for explicitly split formulae. A formula may be split multiple times.
        while (GetUsefulInfoTerm(Target->AnnotatedFormula,"explicit_split_to",
OccurenceNumber,UsefulInfo) != NULL) {
            OccurenceNumber++;
            GetName(Target->AnnotatedFormula,FormulaName);
            (*NumberOfExplicitSplits)++;
            ExtractTermArguments(UsefulInfo);
            if (Tokenize(UsefulInfo,SplitChildrenNames,",") != 2) {
                CodingError("Wrong number of split children");
            }
            SplitChild1 = GetAnnotatedFormulaFromListByName(Head,
SplitChildrenNames[0]);
            SplitChild2 = GetAnnotatedFormulaFromListByName(Head,
SplitChildrenNames[1]);
//----No formula may have both split children as an ancestor. Check all roots.
            InnerTarget = RootListHead;
            while (OKSoFar && InnerTarget != NULL) {
                if (AnnotatedFormulaInTreeTHM(InnerTarget->TheTree,SplitChild1,
0) != NULL && AnnotatedFormulaInTreeTHM(InnerTarget->TheTree,SplitChild2,0) !=
NULL) {
                    OKSoFar = 0;
                    QPRINTF(OptionValues,2)(
"FAILURE: %s has both split siblings %s and %s as ancestors\n",
GetName(InnerTarget->TheTree->AnnotatedFormula,InnerName),SplitChildrenNames[0],
SplitChildrenNames[1]);
                }
                InnerTarget = InnerTarget->Next;
            }
//----If child is an ancestor of the refutation node, not going through non-thm
//----Both sides must have a false descendant
            if (OKSoFar && OptionValues.CheckRefutation &&
((AnnotatedFormulaInTreeTHM(RefutationRoot,SplitChild1,1) != NULL &&
AnnotatedFormulaInTreesTHM(FalseList,SplitChild2,1) == NULL) ||
(AnnotatedFormulaInTreeTHM(RefutationRoot,SplitChild2,1) != NULL && 
AnnotatedFormulaInTreesTHM(FalseList,SplitChild1,1) == NULL))) {
                OKSoFar = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: One of the split siblings %s and %s is not refuted\n",
SplitChildrenNames[0],SplitChildrenNames[1]);
            }
        }
        Target = Target->Next;
    }

    FreeRootList(&FalseList,0);
    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int IsPredicateDefinition(ANNOTATEDFORMULA PossibleAnnotatedDefn,
String PredicateDefined) {

    FORMULA PossibleDefn;

    PossibleDefn = PossibleAnnotatedDefn->AnnotatedFormulaUnion.
AnnotatedTSTPFormula.FormulaWithVariables->Formula;

    if (PossibleDefn->Type == binary && 
PossibleDefn->FormulaUnion.BinaryFormula.Connective == equivalence &&
PossibleDefn->FormulaUnion.BinaryFormula.LHS->Type == atom &&
IsNewPredicate(GetSymbol(PossibleDefn->FormulaUnion.BinaryFormula.LHS->
FormulaUnion.Atom))) {
        strcpy(PredicateDefined,GetSymbol(PossibleDefn->FormulaUnion.
BinaryFormula.LHS->FormulaUnion.Atom));
        return(1);
    } else {
        return(0);
    }
}
//-----------------------------------------------------------------------------
int StructuralVerification(OptionsType * OptionValues,LISTNODE Head,
SIGNATURE Signature) {

    int OKSoFar;
    int NumberOfInstances;
    ROOTLIST RootListHead;

    OKSoFar = 1;

    if (!GlobalInterrupted && OKSoFar) {
        if (UniquelyNamed(Head)) {
            QPRINTF((*OptionValues),2)(
"SUCCESS: Derivation has unique formula names\n");
        } else {
            QPRINTF((*OptionValues),2)(
"FAILURE: Derivation has duplicate formula names\n");
            OKSoFar = 0;
        }
    }

    if (!GlobalInterrupted && OKSoFar) {
        if (AllParentsExist(*OptionValues,Head)) {
            QPRINTF((*OptionValues),2)(
"SUCCESS: All derived formulae have parents and inference information\n");
        } else {
            OKSoFar = 0;
        }
    }

    if ((RootListHead = BuildRootList(Head)) == NULL) {
        QPRINTF((*OptionValues),2)(
"FAILURE: Cannot build explicit proof tree\n");
        OKSoFar = 0;
    }

    if (!GlobalInterrupted && OKSoFar) {
        if (!CyclicRootList(RootListHead)) {
            QPRINTF((*OptionValues),2)("SUCCESS: Derivation is acyclic\n");
        } else {
            QPRINTF((*OptionValues),2)("FAILURE: Derivation is cyclic\n");
            OKSoFar = 0;
        }
    }

//----Check a refutation has a false root (at least one)
    if (!GlobalInterrupted && OKSoFar) {
        if (OptionValues->CheckRefutation) {
            if (CheckFalseRootNode(*OptionValues,RootListHead) != NULL) {
                QPRINTF((*OptionValues),2)(
"SUCCESS: Derivation could be a refutation\n");
            } else {
                OKSoFar = 0;
            }
        } else if (OptionValues->AutoMode && 
GetFalseRootNode(RootListHead) != NULL) {
            OptionValues->CheckRefutation = 1;
            QPRINTF((*OptionValues),1)(
"AUTOSET: Has false root, will check as a refutation\n");
            QPRINTF((*OptionValues),2)(
"SUCCESS: Derivation could be a refutation\n");
        }
    }

//----Check all formulae are used, and a conjecture exists
    if (!GlobalInterrupted && OKSoFar && OptionValues->CheckRefutation) {
        if (UsesFormulae(*OptionValues,Head,RootListHead)) {
            QPRINTF((*OptionValues),2)(
"SUCCESS: Derivation uses all formulae and has a (negated) conjecture\n");
        } 
    }

//----Check that all nodes that have a false parent have two parents, and
//----the second is an ancestor of the false
    if (!GlobalInterrupted && OKSoFar && OptionValues->CheckRefutation) {
        if (OptionValues->NoExpensiveChecks) {
            QPRINTF((*OptionValues),2)(
"WARNING: Suppressed check of well formed proofs by contradiction\n");
        } else if (WellFormedProofsByContradiction(*OptionValues,Head,Signature,
&NumberOfInstances)) {
//----Report only if there are some
            if (NumberOfInstances > 0) {
                QPRINTF((*OptionValues),2)(
"SUCCESS: Derivation has well formed proofs by contradiction\n");
            }
        } else {
            OKSoFar = 0;
        }
    }

//----Check that explicit splits are independent
    if (!GlobalInterrupted && OKSoFar) {
        if (OptionValues->NoExpensiveChecks) {
            QPRINTF((*OptionValues),2)(
"WARNING: Suppressed expensive check of explicit splits\n");
        } else if (ExplicitSplitsIndependent(*OptionValues,Head,RootListHead,
&NumberOfInstances)) {
//----Report only if there are some
            if (NumberOfInstances > 0) {
                QPRINTF((*OptionValues),2)(
"SUCCESS: All explicit splits are independent and refuted\n");
            }
        } else {
            OKSoFar = 0;
        }
    }

    FreeRootList(&RootListHead,1);

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
void GetSplitDefinitionNames(OptionsType OptionValues,LISTNODE Head,
ANNOTATEDFORMULA AnnotatedFormula,String SplitDefinitionNames) {

    String UsefulInfo;
    int NumberOfSiblings;
    StringParts SiblingNames;
    int SiblingNumber;
    String SiblingInfo;

    strcpy(SplitDefinitionNames,"");
//----Get the list of split offsrping
    if (GetUsefulInfoTerm(AnnotatedFormula,"psuedo_split_to",1,UsefulInfo) == 
NULL) {
        CodingError("Missing psuedo_split_to term");
    }
    ExtractTermArguments(UsefulInfo);
    NumberOfSiblings = Tokenize(UsefulInfo,SiblingNames,",");
//----For each offspring, if it has an esplit defn, get the name
    for (SiblingNumber = 0;SiblingNumber < NumberOfSiblings;SiblingNumber++) {
        if (GetUsefulInfoTerm(GetAnnotatedFormulaFromListByName(Head,
SiblingNames[SiblingNumber]),"esplit_defn",1,SiblingInfo) != NULL) {
            ExtractTermArguments(SiblingInfo);
            strcat(SplitDefinitionNames,SiblingInfo);
            strcat(SplitDefinitionNames,"\n");
        }
    }
}
//-----------------------------------------------------------------------------
LISTNODE GetListOfLeaves(OptionsType OptionValues,LISTNODE Head) {

    LISTNODE Leaves;

    Leaves = NULL;
    while (Head != NULL) {
        if (!DerivedAnnotatedFormula(Head->AnnotatedFormula)) {
            AddListNode(&Leaves,Leaves,Head->AnnotatedFormula);
        }
        Head = Head->Next;
    }
    return(Leaves);
}
//-----------------------------------------------------------------------------
int UserSemanticsVerification(OptionsType OptionValues,LISTNODE Head) {

    LISTNODE Leaves;
    LISTNODE LeafAxioms;
    int OKSoFar;

    OKSoFar = 1;

//----Check axioms are satisfiable, or at least not unsatisfiable
    Leaves = GetListOfLeaves(OptionValues,Head);
    LeafAxioms = GetListOfAnnotatedFormulaeWithType(Leaves,axiom_like);
    if (LeafAxioms == NULL || GDVCheckSatisfiable(OptionValues,LeafAxioms,
"axioms","sat",1) == 1) {
        QPRINTF(OptionValues,2)("SUCCESS: Leaf axioms are satisfiable\n");
    } else {
        QPRINTF(OptionValues,2)("WARNING: Leaf axioms are not satisfiable\n");
    }
    FreeListOfAnnotatedFormulae(&LeafAxioms);
    FreeListOfAnnotatedFormulae(&Leaves);

    return(OKSoFar);
}
//-----------------------------------------------------------------------------
int RuleSpecificVerification(OptionsType OptionValues,LISTNODE Head,
SIGNATURE Signature) {

    LISTNODE Target;
    int AllOKSoFar;
    String UsefulInfo;
    int UserCheckParentRelevance;
    String FormulaName;
    int NumberOfParents;
    StringParts ParentNames;
    String ListParentNames;
    LISTNODE ParentAnnotatedFormulae;
    LISTNODE Parent;
    String SplitDefinitionNames;
    ANNOTATEDFORMULA SplitParent;
    int OccurenceNumber;

    Target = Head;
    AllOKSoFar = 1;
    while (!GlobalInterrupted && (AllOKSoFar || OptionValues.ForceContinue) &&
Target != NULL) {
        GetName(Target->AnnotatedFormula,FormulaName);
//DEBUG PrintAnnotatedTSTPNode(stdout,Target->AnnotatedFormula,tptp,1);

//----esplit parent to be inferred from its children, with relevance check
        if (GetUsefulInfoTerm(Target->AnnotatedFormula,"psuedo_split_to",1,
UsefulInfo) != NULL) {
            UserCheckParentRelevance = OptionValues.CheckParentRelevance;
            OptionValues.CheckParentRelevance = 1;
//----Get the split children's names
            ExtractTermArguments(UsefulInfo);
            NumberOfParents = Tokenize(UsefulInfo,ParentNames,",");
            MakePrintableList(ParentNames,NumberOfParents,ListParentNames);
//----Make list of split children
            MakeListOfParents(OptionValues,Head,FormulaName,
ParentNames,NumberOfParents,&ParentAnnotatedFormulae);
            if (!CorrectlyInferred(OptionValues,Target->AnnotatedFormula,
FormulaName,ParentAnnotatedFormulae,ListParentNames,"thm",FormulaName,"")) {
                AllOKSoFar = 0;
            } 
//----Free the parents list
            FreeListOfAnnotatedFormulae(&ParentAnnotatedFormulae);
            OptionValues.CheckParentRelevance = UserCheckParentRelevance;
        }

//----esplit child, to be inferred from the parent and the definitions
//----This is done only if esplit definition were not expected, and therefore
//----created by GDV and the right tags inserted to make all this possible.
//----Hopefully in the future this will be unused.
        if (GetUsefulInfoTerm(Target->AnnotatedFormula,"psuedo_split_from",1,
UsefulInfo) != NULL) {
//----Get the split parent's name
            ExtractTermArguments(UsefulInfo);
//----Get the split definitions
            if ((SplitParent = GetAnnotatedFormulaFromListByName(Head,
UsefulInfo)) == NULL) {
                CodingError("Missing split parent");
            } 
            GetSplitDefinitionNames(OptionValues,Head,SplitParent,
SplitDefinitionNames);
            strcat(UsefulInfo,"\n");
            strcat(UsefulInfo,SplitDefinitionNames);
            NumberOfParents = Tokenize(UsefulInfo,ParentNames,"\n");
            MakePrintableList(ParentNames,NumberOfParents,ListParentNames);
//----Make list of split children
            MakeListOfParents(OptionValues,Head,FormulaName,ParentNames,
NumberOfParents,&ParentAnnotatedFormulae);
            if (!CorrectlyInferred(OptionValues,Target->AnnotatedFormula,
FormulaName,ParentAnnotatedFormulae,ListParentNames,"thm",FormulaName,"")) {
                AllOKSoFar = 0;
            } else {
                AddVerifiedTag(Target->AnnotatedFormula,Signature,
"psuedo_split");
            }
//----Free the parents list
            FreeListOfAnnotatedFormulae(&ParentAnnotatedFormulae);
        }

//----sr_split parent negation to be inferred from its negated children, 
//----with relevance check
        OccurenceNumber = 1;
        while (GetUsefulInfoTerm(Target->AnnotatedFormula,"explicit_split_to",
OccurenceNumber,UsefulInfo) != NULL) {
            OccurenceNumber++;
            UserCheckParentRelevance = OptionValues.CheckParentRelevance;
            OptionValues.CheckParentRelevance = 1;
//----Get the split children's names
            ExtractTermArguments(UsefulInfo);
            NumberOfParents = Tokenize(UsefulInfo,ParentNames,",");
            MakePrintableList(ParentNames,NumberOfParents,ListParentNames);
//----Make list of split children
            MakeListOfParents(OptionValues,Head,FormulaName,
ParentNames,NumberOfParents,&ParentAnnotatedFormulae);
            NegateListOfAnnotatedTSTPNodes(ParentAnnotatedFormulae,0);
            Negate(Target->AnnotatedFormula,0);
            if (!CorrectlyInferred(OptionValues,Target->AnnotatedFormula,
FormulaName,ParentAnnotatedFormulae,ListParentNames,"thm",FormulaName,
"(Negated formulae for sr_split)")) {
                AllOKSoFar = 0;
            } else {
                Parent = ParentAnnotatedFormulae;
                while (Parent != NULL) {
                    AddVerifiedTag(Parent->AnnotatedFormula,Signature,
"explicit_split");
                    Parent = Parent->Next;
                }
            }
            Negate(Target->AnnotatedFormula,1);
            NegateListOfAnnotatedTSTPNodes(ParentAnnotatedFormulae,1);
//----Free the parents list
            FreeListOfAnnotatedFormulae(&ParentAnnotatedFormulae);
            OptionValues.CheckParentRelevance = UserCheckParentRelevance;
        }

//----Proofs by contradiction
        if (GetUsefulInfoTerm(Target->AnnotatedFormula,
"proved_by_contradiction",1,UsefulInfo) != NULL) {
//----Get the parent's name
            ExtractTermArguments(UsefulInfo);
            NumberOfParents = 1;
            ParentNames[0] = UsefulInfo;
            MakeListOfParents(OptionValues,Head,FormulaName,ParentNames,
NumberOfParents,&ParentAnnotatedFormulae);
            NegateListOfAnnotatedTSTPNodes(ParentAnnotatedFormulae,0);
            if (!CorrectlyInferred(OptionValues,Target->AnnotatedFormula,
FormulaName,ParentAnnotatedFormulae,UsefulInfo,"thm",FormulaName,
"(Negated parent for PbC)")) {
                AllOKSoFar = 0;
            } else {
                AddVerifiedTag(Target->AnnotatedFormula,Signature,
"proved_by_contradiction");
            }
            NegateListOfAnnotatedTSTPNodes(ParentAnnotatedFormulae,1);
//----Free the parents list
            FreeListOfAnnotatedFormulae(&ParentAnnotatedFormulae);
        }

        Target = Target->Next;
    }

    return(AllOKSoFar);
}
//-----------------------------------------------------------------------------
void SplitIntoSatisfiableLists(OptionsType OptionValues, LISTNODE Head,
LISTNODE * PositiveHead,LISTNODE * NegativeHead,LISTNODE * NeitherHead) {

    int PositiveOrNegative;

    *PositiveHead = NULL;
    *NegativeHead = NULL;
    *NeitherHead = NULL;
    while (Head != NULL) {
        PositiveOrNegative = 0;
        if (AnnotatedFormulaTrueInInterpretation(Head->AnnotatedFormula,
positive)) {
            AddListNode(PositiveHead,*PositiveHead,Head->AnnotatedFormula);
            PositiveOrNegative = 1;
        }
        if (AnnotatedFormulaTrueInInterpretation(Head->AnnotatedFormula,
negative)) {
            AddListNode(NegativeHead,*NegativeHead,Head->AnnotatedFormula);
            PositiveOrNegative = 1;
        }
        if (!PositiveOrNegative) {
            AddListNode(NeitherHead,*NeitherHead,Head->AnnotatedFormula);
        }
        Head = Head->Next;
    }

//----Check if any left over
    if (*NeitherHead == NULL) {
        QPRINTF(OptionValues,0)(
"WOOHOOO: All problem formulae are positive or negative\n");
    } else if (GDVCheckSatisfiable(OptionValues,*NeitherHead,"neither",
"sat",0) == 1) {
        QPRINTF(OptionValues,0)(
"WOOHOOO: All problem formulae are in a satisfiable set\n");
    } else {
        QPRINTF(OptionValues,2)(
"WARNING: Some problem formulae are not in a satisfiable set\n");
//----Free the neither list (making the head NULL) if no use
        FreeListOfAnnotatedFormulae(NeitherHead);
    }
}
//-----------------------------------------------------------------------------
int LeafVerification(OptionsType OptionValues,LISTNODE Head,SIGNATURE
Signature) {

    LISTNODE ProblemHead;
    LISTNODE SatisfiableHead;
    LISTNODE PositiveHead;
    LISTNODE NegativeHead;
    LISTNODE NeitherHead;
    LISTNODE Target;
    String FormulaName;
    String FileBaseName;
    String UsefulInfo;
    LISTNODE ProblemTarget;
    String ProblemTargetName;
    int AllOKSoFar;
    int ThisOneOK;
    String PredicateDefined;

//----Get the problem file into memory
    if ((ProblemHead = ParseFileOfFormulae(OptionValues.ProblemFileName,NULL,
Signature,1,NULL)) == NULL) {
        QPRINTF(OptionValues,1)(
"ERROR: Could not parse problem file %s\n",OptionValues.ProblemFileName);
        QPRINTF(OptionValues,2)("FAILURE: Leaf nodes unverified\n");
        return(0);
    }

    if (OptionValues.Quietness == 0) {
        printf("Problem file contents:\n");
        PrintListOfAnnotatedTSTPNodes(stdout,Signature,ProblemHead,tptp,1);
    }
//----Convert the problem into FOF for semantic checking
    UniversalFOFifyList(ProblemHead);
    if (OptionValues.Quietness == 0) {
        printf("Problem file contents as FOF:\n");
        PrintListOfAnnotatedTSTPNodes(stdout,Signature,ProblemHead,tptp,1);
    }

//----Check if the entire input is satisfiable
    if (GDVCheckSatisfiable(OptionValues,ProblemHead,"problem","sat",0) == 
1) {
        QPRINTF(OptionValues,0)("WOOHOOO: Input problem is satisfiable\n");
        SatisfiableHead = ProblemHead;
        PositiveHead = NULL;
        NegativeHead = NULL;
        NeitherHead = NULL;
    } else {
        SatisfiableHead = NULL;
//----Split problem into satisfiable lists
        SplitIntoSatisfiableLists(OptionValues,ProblemHead,&PositiveHead,
&NegativeHead,&NeitherHead);
    }

//----For each derivation node, check if the same as a problem node, or
//----can be inferred from one of the satisfiable lists
    Target = Head;
    AllOKSoFar = 1;
    while (!GlobalInterrupted && (AllOKSoFar || OptionValues.ForceContinue) &&
Target != NULL) {
        if (!DerivedAnnotatedFormula(Target->AnnotatedFormula) &&
GetUsefulInfoTerm(Target->AnnotatedFormula,"verified",1,UsefulInfo) == NULL) {
            GetName(Target->AnnotatedFormula,FormulaName);
//DEBUG printf("Starting leaf named %s\n",FormulaName);
            strcpy(UsefulInfo,"");
            GetSourceTerm(Target->AnnotatedFormula,UsefulInfo);
//----Don't verify definitions inserted by GDV
            if (strstr(UsefulInfo,"creator(gdv") == UsefulInfo) {
                QPRINTF(OptionValues,1)(
"WARNING: %s is a leaf created by GDV, and not verified\n",FormulaName);
                ThisOneOK = 1;
//----Verify introduced leaves by their type.
            } else if (strstr(UsefulInfo,"introduced(") == UsefulInfo) {
//DEBUG printf("Checking introduced leaf %s\n",FormulaName);
                ThisOneOK = 0;
                ExtractTermArguments(UsefulInfo);
                if (strstr(UsefulInfo,"definition") == UsefulInfo) {
                    if (IsPredicateDefinition(Target->AnnotatedFormula,
PredicateDefined)) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a definition of %s\n",FormulaName,PredicateDefined);
                        ThisOneOK = 1;
                    } 
                } else if (strstr(UsefulInfo,"axiom_of_choice") == UsefulInfo) {
                } else if (strstr(UsefulInfo,"tautology") == UsefulInfo) {
                    strcpy(FileBaseName,FormulaName);
                    strcat(FileBaseName,"_is_tautology");
                    if (CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,NULL,NULL,"thm",FileBaseName,"")) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a tautology\n",FormulaName);
                        ThisOneOK = 1;
                    }
                }
                if (!ThisOneOK) {
                    if (strchr(UsefulInfo,',') != NULL) {
                        *strchr(UsefulInfo,',') = '\0';
                    }
                    QPRINTF(OptionValues,2)(
"FAILURE: %s is an ill-formed %s\n",FormulaName,UsefulInfo);
                    AllOKSoFar = 0;
                }
//----Regular leaves
            } else {
                ThisOneOK = 0;
//----Look for an original that has been copied
                ProblemTarget = ProblemHead;
                while (!ThisOneOK && ProblemTarget != NULL) {
                    if (SameFormulaInAnnotatedFormulae(Target->AnnotatedFormula,
ProblemTarget->AnnotatedFormula,1)) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a copy of %s\n",FormulaName,
GetName(ProblemTarget->AnnotatedFormula,ProblemTargetName));
                        ThisOneOK = 1;
                    }
                    ProblemTarget = ProblemTarget->Next;
                }

//----If not a copy, try some inferencing
                OptionValues.CheckParentRelevance = 0;
                if (!ThisOneOK && SatisfiableHead != NULL) {
                    strcpy(FileBaseName,FormulaName);
                    strcat(FileBaseName,"_from_problem");
                    if (CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,SatisfiableHead,NULL,"thm",FileBaseName,
"")) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a thm of all the input formulae\n",FormulaName);
                        ThisOneOK = 1;
                    }
                }
                if (!ThisOneOK && PositiveHead != NULL) {
                    strcpy(FileBaseName,FormulaName);
                    strcat(FileBaseName,"_from_positive");
                    if (CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,PositiveHead,NULL,"thm",FileBaseName,"")) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a thm of the positive input formulae\n",FormulaName);
                        ThisOneOK = 1;
                    }
                }
                if (!ThisOneOK && NegativeHead != NULL) {
                    strcpy(FileBaseName,FormulaName);
                    strcat(FileBaseName,"_from_negative");
                    if (CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,NegativeHead,NULL,"thm",FileBaseName,"")) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a thm of the negative input formulae\n",FormulaName);
                        ThisOneOK = 1;
                    }
                }
                if (!ThisOneOK && NeitherHead != NULL) {
                    strcpy(FileBaseName,FormulaName);
                    strcat(FileBaseName,"_from_neither");
                    if (CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,NeitherHead,NULL,"thm",FileBaseName,"")) {
                        QPRINTF(OptionValues,2)(
"SUCCESS: %s is a thm of the neither input formulae\n",FormulaName);
                        ThisOneOK = 1;
                    }
                }

                if (ThisOneOK) {
                    AddVerifiedTag(Target->AnnotatedFormula,Signature,"leaf");
                } else {
                    QPRINTF(OptionValues,2)(
"FAILURE: %s cannot be shown to be a thm of the input formulae\n",FormulaName);
                }

                AllOKSoFar *= ThisOneOK;
            }
        }
        Target = Target->Next;
    }

//----Free the satisfiable lists and the problem list
    FreeListOfAnnotatedFormulae(&PositiveHead);
    FreeListOfAnnotatedFormulae(&NegativeHead);
    FreeListOfAnnotatedFormulae(&NeitherHead);
    FreeListOfAnnotatedFormulae(&ProblemHead);

    return(AllOKSoFar);
}
//-----------------------------------------------------------------------------
int DerivedVerification(OptionsType OptionValues,LISTNODE Head,
SIGNATURE Signature) {

    extern int GlobalInterrupted;
    LISTNODE Target;
    String VerifiedTag;
    int AllOKSoFar;
    String FormulaName;
    String InferenceRule;
    String AllParentNames;
    String ListParentNames;
    StringParts ParentNames;
    int NumberOfParents;
    LISTNODE ParentAnnotatedFormulae;
    String SZSStatus;

    Target = Head;
    AllOKSoFar = 1;
//----Work through list looking for derived nodes
    while (!GlobalInterrupted && (AllOKSoFar || OptionValues.ForceContinue) &&
Target != NULL) {
//DEBUG PrintAnnotatedTSTPNode(stdout,Target->AnnotatedFormula,tptp,1);
        if (DerivedAnnotatedFormula(Target->AnnotatedFormula) &&
GetUsefulInfoTerm(Target->AnnotatedFormula,"verified",1,VerifiedTag) == NULL) {
            GetName(Target->AnnotatedFormula,FormulaName);

            GetInferenceRule(Target->AnnotatedFormula,InferenceRule);
//----Get the parents' in various ways
            GetNodeParentNames(Target->AnnotatedFormula,AllParentNames);
            NumberOfParents = Tokenize(AllParentNames,ParentNames,"\n");
            MakePrintableList(ParentNames,NumberOfParents,ListParentNames);
            MakeListOfParents(OptionValues,Head,FormulaName,
ParentNames,NumberOfParents,&ParentAnnotatedFormulae);

//----Copied formula
            if (!strcmp(InferenceRule,"")) {
                if (SameFormulaInAnnotatedFormulae(
Target->AnnotatedFormula,ParentAnnotatedFormulae->AnnotatedFormula,1)) {
                    QPRINTF(OptionValues,2)(
"SUCCESS: %s is a copy of %s\n",FormulaName,ParentNames[0]);
                } else {
                    QPRINTF(OptionValues,2)(
"FAILURE: %s is not a copy of %s\n",FormulaName,ParentNames[0]);
                    AllOKSoFar = 0;
                }

//----Inferred formula
            } else {
//----Get SZS status
                if (GetInferenceStatus(Target->AnnotatedFormula,SZSStatus) ==
NULL) {
                    QPRINTF(OptionValues,1)(
"WARNING: Cannot get SZS status for %s, assuming thm\n",FormulaName);
                    strcpy(SZSStatus,"thm");
                }
                if (!CorrectlyInferred(OptionValues,
Target->AnnotatedFormula,FormulaName,ParentAnnotatedFormulae,ListParentNames,
SZSStatus,FormulaName,"")) {
                    AllOKSoFar = 0;
                } else {
                    AddVerifiedTag(Target->AnnotatedFormula,Signature,
SZSStatus);
                }
            }
//----Free the parents list
            FreeListOfAnnotatedFormulae(&ParentAnnotatedFormulae);
        }
        Target = Target->Next;
    }

    return(AllOKSoFar);
}
//-----------------------------------------------------------------------------
void GetProblemFileName(OptionsType * OptionValues,ANNOTATEDFORMULA 
AnnotatedFormula,char * ProblemFileName) {

    String PossibleFileName;

//----If no name given look in first formula
    if (strcmp(ProblemFileName,"")) {
        strcpy(PossibleFileName,ProblemFileName);
    } else if (GetFileSourceNameAndNode(AnnotatedFormula,PossibleFileName) != 
NULL) {
        *strchr(PossibleFileName,'\n') = '\0';
        if (PossibleFileName[0] == '\'') {
            PossibleFileName[strlen(PossibleFileName)-1] = '\0';
            strcpy(PossibleFileName,PossibleFileName+1);
        }
    } else {
        strcpy(PossibleFileName,"");
    }

//----Check that it's a readable file
    if (strcmp(PossibleFileName,"") && access(PossibleFileName,R_OK) == 0) {
        strcpy(ProblemFileName,PossibleFileName);
        if (OptionValues->AutoMode) {
            OptionValues->VerifyLeaves = 1;
            QPRINTF((*OptionValues),1)(
"AUTOSET: Found readable problem file, will check leaves\n");
        }
    } 
} 
//-----------------------------------------------------------------------------
int main(int argc,char * argv[]) {

    extern int GlobalInterrupted;
    OptionsType OptionValues;
    LISTNODE Head;
    SIGNATURE Signature;
    int OKSoFar;

    GlobalInterrupted = 0;
    if (signal(SIGQUIT,GlobalInterruptHandler) == SIG_ERR ||
signal(SIGQUIT,GlobalInterruptHandler) == SIG_ERR) {
        perror("Setting interrupt handler");
        exit(EXIT_FAILURE);
    }

    if (!SystemOnTPTPAvailable()) {
        exit(EXIT_FAILURE);
    }

    if (!ProcessCommandLine(argc,argv,&OptionValues)) {
        QPRINTF(OptionValues,4)("ERROR: Invalid command line arguments\n");
        exit(EXIT_FAILURE);
    }

//----Read the derivation file
    Signature = NewSignature();
    SetNeedForNonLogicTokens(0);
    if ((Head = ParseFileOfFormulae(OptionValues.DerivationFileName,NULL,
Signature,1,NULL)) == NULL) {
        QPRINTF(OptionValues,4)(
"ERROR: Could not parse %s\n",OptionValues.DerivationFileName);
        exit(EXIT_FAILURE);
    }
    if (OptionValues.Quietness == 0) {
        printf("Derivation file contents:\n");
        PrintListOfAnnotatedTSTPNodes(stdout,Signature,Head,tptp,1);
    }
//----Get problem file name sorted out
    GetProblemFileName(&OptionValues,Head->AnnotatedFormula,OptionValues.
ProblemFileName);

//----Create working directory
    if (!GlobalInterrupted) {
        if (!CreateDirectory(OptionValues.KeepFilesDirectory,
OptionValues.DerivationFileName)) {
            QPRINTF(OptionValues,4)(
"ERROR: Could not create working directory %s\n",
OptionValues.KeepFilesDirectory);
            exit(EXIT_FAILURE);
        }
    }

//----Start verification
    OKSoFar = 1;

//----Structural completion - failure cannot be forced past
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Start structural completion\n");
        if (!StructuralCompletion(OptionValues,&Head,Signature)) {
            OKSoFar = 0;
            if (OptionValues.ForceContinue) {
                OptionValues.ForceContinue = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: Structural completion failure, cannot be forced to continue\n");
            }
        }
    }
//DEBUG PrintListOfAnnotatedTSTPNodes(stdout,Head,0,1);

//----Structural verification - failure cannot be forced past
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Start structural verification\n");
        if (!StructuralVerification(&OptionValues,Head,Signature)) {
            OKSoFar = 0;
            if (OptionValues.ForceContinue) {
                OptionValues.ForceContinue = 0;
                QPRINTF(OptionValues,2)(
"FAILURE: Structural failure, cannot be forced to continue\n");
            }
        }
    }
//DEBUG PrintListOfAnnotatedTSTPNodes(stdout,Head,0,1);

//----Convert to FOF for semantic parts
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Convert to FOF\n");
        UniversalFOFifyList(Head);
    }
//DEBUG PrintListOfAnnotatedTSTPNodes(stdout,Head,0,1);

//----User semantic parts
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Start user semantics verification\n");
        OKSoFar *= UserSemanticsVerification(OptionValues,Head);
    }

//----Inference rule specific parts
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Start rule specific verification\n");
        OKSoFar *= RuleSpecificVerification(OptionValues,Head,Signature);
    }
//DEBUG PrintListOfAnnotatedTSTPNodes(stdout,Head,0,1);

//----Leaf verification
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue) &&
OptionValues.VerifyLeaves) {
        QPRINTF(OptionValues,0)("Start leaf verification\n");
        OKSoFar *= LeafVerification(OptionValues,Head,Signature);
    }

//----Derived verification
    if (!GlobalInterrupted && (OKSoFar || OptionValues.ForceContinue)) {
        QPRINTF(OptionValues,0)("Start derived verification\n");
        OKSoFar *= DerivedVerification(OptionValues,Head,Signature);
    }

//----Delete working directory if not longer needed
    if (!OptionValues.KeepFiles) {
        QPRINTF(OptionValues,0)("Clean up files\n");
        EmptyAndDeleteDirectory(OptionValues.KeepFilesDirectory);
    }

//----Free memory
    FreeListOfAnnotatedFormulae(&Head);
    FreeSignature(&Signature);
    if (GlobalInterrupted) {
        OKSoFar = 0;
    }

    QPRINTF(OptionValues,2)("CPUTIME: %.2f\n",GetTotalCPUTime());
    QPRINTF(OptionValues,3)("%s\n",OKSoFar?"SUCCESS":"FAILURE");

    return(EXIT_SUCCESS);
}
//-----------------------------------------------------------------------------
