(* ========================================================================= *)
(* HOL Types                                                                 *)
(* ========================================================================= *)

(*Indexnames can be quickly renamed by adding an offset to the integer part,
  for resolution.*)
type indexname = string * int

(* Types are classified by sorts. *)
type cclass = string
type sort  = cclass list
type arity = string * sort list * sort

type typ =
    Type  of string * typ list |
    TFree of string * sort |
    TVar  of indexname * sort

let bt_i = Type("ind",[])
let bt_o = Type("bool", [])
let bt_x = TVar(("X",0), ["tvar"])
let bt_type = Type("type",[])

(* type hol_type = *)
(*     Basetype of string *)
(*   | TypeVar of string *)
(*   | Funtype of hol_type * hol_type *)

let typevar name =
  TVar((name,0) , ["sort"])

(* let basetype type name = *)
(*   Type(type, [name]) *)

let abstr_type argtype bodytype =
  Type("fun", [argtype;bodytype])

(* let dest_basetype = function *)
(*     Basetype s -> s *)
(*   | _ -> failwith "dest_basetype" *)


let appl_type funtype argtype =
  match funtype with
    Type("fun", [type1;type2]) ->
        assert(type1=argtype);
        type2
    | _ -> failwith "appl_type: first argument not a function type"


let arg_type = function
    Type("fun", t1::_) -> t1
  | _ -> failwith "arg_type: argument not a function type"


let result_type = function
    Type("fun", t1::t2::_) -> t2
  | _ -> failwith "arg_type: argument not a function type"


let rec all_arg_types' ls = function
    Type("fun", [argtype;restype]) -> all_arg_types' (argtype::ls) restype
  | TVar(_, _) -> []
  | TFree(_, _) -> []
  
let all_arg_types t = List.rev (all_arg_types' [] t)


let is_funtype = function
    Type("fun", _) -> true
  | _  -> false


let is_basetype = function
    Type("bool", _) -> true
  | Type("ind", _) -> true
  | _ -> false


let is_applicable type1 type2 =
  match type1 with
      Type("fun", argtype::_) -> type2=argtype
    | _ -> false


let rec to_string = function
    Type("fun", [t1;t2]) -> (add_pars t1)^" -> "^(add_pars t2)
  | Type("ind", _) -> "i" 
  | Type("bool", _) -> "o" 
  | TFree(nam, sort) -> nam
  | TVar((nam,i), sort) -> nam
  | _ -> "not matched "
and add_pars ty =
  if is_basetype ty then to_string ty
  else "("^(to_string ty)^")"


