(* ========================================================================= *)
(* HOL Types                                                                 *)
(* ========================================================================= *)


(** Module Hol_type implements HOL types.
    @author Arnaud
    @since 07-06-06*)

type indexname = string * int

(* Types are classified by sorts. *)
type cclass = string
type sort  = cclass list
type arity = string * sort list * sort

type typ =
    Type  of string * typ list |
    TFree of string * sort |
    TVar  of indexname * sort

val bt_i : typ
val bt_o : typ
val bt_x : typ
val bt_type : typ

(* type hol_type = *)
(*     Basetype of string *)
(*   | TypeVar of string *)
(*   | Funtype of hol_type * hol_type *)
(** The abstract type of HOL types.*)
	

(** {6 Constructors} *)

(* val basetype : string -> typ *)
(** basetype constructor *)

val typevar : string -> typ
(** typevar constructor *)

val abstr_type : typ -> typ -> typ
(** type of an abstraction with bound var of type1 and abstracted term of type2 *)

(** {6 Destructors} *)

(* val dest_basetype : typ -> string *)
(** basetype destructor *)

val appl_type : typ -> typ -> typ
(** type of an application of fun term of type1 to an argument of type2 *)
(* isn't this the same as result_type ? *)

val arg_type : typ -> typ
(** type of the required argument of an application*)

val result_type : typ -> typ
(** result type of an application *)

val all_arg_types : typ -> typ list
(** types of all arguments of a funtype *)


(** {6 Predicates} *)

val is_funtype : typ -> bool
(** Checks whether the given type is a function type. *)

val is_basetype : typ -> bool
(* val is_basetype : typ -> bool *)
(** Checks whether the given type is a basetype. *)

(* typeclash? *)
val is_applicable : typ -> typ -> bool
(** [is_applicable ty1 ty2] checks if [ty2] is an argument type of [ty1]. *)


(** {6 Pretty-printing} *)

(* val to_string : typ -> string *)
val to_string : typ -> string
(** Returns a string representation of the given type. *)

