(* ========================================================================= *)
(* Positions and related data structures                                     *)
(* ========================================================================= *)


(** Module Position implements term positons and related data structures.
    @author Frank
    @since 19-05-06*)

(** To Do: replace entry lists and list operations by sets (list->set, append->union) *)


type relpos = Abstr | Func | Arg

type position = relpos list

type 'a positiontable = Empty |
                        Table of 'a positiontable *
                                  'a positiontable *
                                  'a positiontable *
                                  'a * bool

let new_positiontable () = Empty


let rec insert_at_pos table pos d =
  match table with
    Empty -> if List.length pos = 0
             then Table(Empty,Empty,Empty,d,true)
             else (match List.hd pos with
              Abstr -> Table(insert_at_pos Empty (List.tl pos) d,Empty,Empty,d,false) | 
              Func  -> Table(Empty,insert_at_pos Empty (List.tl pos) d,Empty,d,false) | 
              Arg   -> Table(Empty,Empty,insert_at_pos Empty (List.tl pos) d,d,false)) |
    Table(abstr,func,arg,d2,flag) ->
             if List.length pos = 0
             then Table(abstr,func,arg,d,true)
             else match List.hd pos with
              Abstr -> Table(insert_at_pos abstr (List.tl pos) d,func,arg,d2,flag) | 
              Func  -> Table(abstr,insert_at_pos func (List.tl pos) d,arg,d2,flag) | 
              Arg   -> Table(abstr,func,insert_at_pos arg (List.tl pos) d,d2,flag) 



let rec data_at_pos table pos =
  match table with
    Empty -> raise Not_found |
    Table(abstr,func,arg,d,flag) ->
             if List.length pos = 0
             then if flag then d else raise Not_found
             else match List.hd pos with
              Abstr -> data_at_pos abstr (List.tl pos) | 
              Func  -> data_at_pos func (List.tl pos) | 
              Arg   -> data_at_pos arg (List.tl pos) 
   


(*
let rec all_occurrences table =
  match !table with
    Empty -> [] |
    Table(abstr,func,arg,l) ->
      all_occurrences abstr @
      all_occurrences func @
      all_occurrences arg @
      !l

let rec subterms_at table pos =
  match !table with
    Empty -> [] |
    Table(abstr,func,arg,l) ->
      if List.length pos = 0
      then !l
      else
        match List.hd pos with
          Abstr -> subterms_at abstr (List.tl pos) |
          Func  -> subterms_at func (List.tl pos) |
          Arg   -> subterms_at arg (List.tl pos)

let rec subterms_below table pos =
  match !table with
    Empty -> [] |
    Table(abstr,func,arg,l) ->
      if List.length pos = 0
      then all_occurrences table
      else
        match List.hd pos with
          Abstr -> subterms_below abstr (List.tl pos) |
          Func  -> subterms_below func (List.tl pos) |
          Arg   -> subterms_below arg (List.tl pos)

let rec subterms_above table pos =
  match !table with
    Empty -> [] |
    Table(abstr,func,arg,l) ->
      if List.length pos = 0
      then []
      else
        !l@
        match List.hd pos with
          Abstr -> subterms_above abstr (List.tl pos) |
          Func  -> subterms_above func (List.tl pos) |
          Arg   -> subterms_above arg (List.tl pos)


let insert_superterm_at table pos v = insert_subterm_at table (List.rev pos) v

let superterms_at table pos = subterms_at table (List.rev pos)

let superterms_no_closer_than table pos = subterms_below table (List.rev pos)

let superterms_no_farther_than table pos = subterms_above table (List.rev pos)

*)