(* ========================================================================= *)
(* User readable terms                                                       *)
(* ========================================================================= *)

open Hol_type

(** The user-readable terms have a simple structure which should not
    be hidden in an ADT.
    The insertion into the termset is easier to implement if the term
    structure is public. *) 
type term =
    Const of string * typ |
    Free of string * typ |
    Var of indexname * typ |
    Bound of int |
    Typ of typ |
    Abstr of string * typ * term |
    Appl of term * term
(* type term = *)
(*     Symbol of string | *)
(*     Const of string * hol_type | *)
(*     Var of string * hol_type | *)
(*     Bound of int | *)
(*     Typ of hol_type | *)
(*     Appl of term * term | *)
(*     Abstr of term * hol_type * term *)
	

val term_for_type : Hol_type.typ -> term  

val type_of : term -> Hol_type.typ 

val type_of1 : Hol_type.typ list -> term -> Hol_type.typ 

val is_symbol : term -> bool
(** Checks whether the given term is a symbol. *)

(* val get_symbol : term -> string *)
(** Returns the string name of the symbol represented by the given term. Raises
    [Failure "not a symbol"] if the term is not a symbol. *)

val is_appl : term -> bool
(** Checks whether the given term is an application. *)

val is_abstr : term -> bool
(** Checks whether the given term is an abstraction. *)

val beta_normalize : term -> term
(** Returns the beta-normal form of the given term. *)

val multi_abstr : (string * typ) list -> term -> term
(** given [(x1,ty1),...,(xn,tyn)] and t, returns Abstr(x1,ty1,Abstr(x2,ty2,...t...)). *)

val multi_quantified : term -> (string * typ) list -> term -> term
(** given Q, [(x1,ty1),...,(xn,tyn)] and t, returns Appl(Q,Abstr(x1,ty1,Appl(Q,Abstr(x2,ty2,...t...))))
    so it essentially turns [forall [(x1,ty1),...,(xn,tyn)] t] into forall(lambda x1:ty1. forall(lambda x2:ty2. ...t...)). *)

val to_string : term -> string
(** Returns a string representation of the given term. *)

