(* ========================================================================= *)
(* Sets of terms in shared representation                                    *)
(* ========================================================================= *)

(** Module Termset implements sets of terms in term graph representation.
    @author Arnaud
    @since 07-06-06*)

open Hol_type
open Term
open Signature


type id = int
(** must this be an abstract type? *)

type termset
(** abstract type of term sets *)

val new_termset : unit -> termset
(** returns an empty term set *)

val create : termset -> term -> id
(** inserts a term into the term graph (which is modified in place)
    and returns the id of the node which represents the term in the
    term graph *)

val retrieve : termset -> id -> term
(** returns the term a the node referenced by the id *)

val appl_with_argument : termset -> id -> id -> id

val appl_with_function : termset -> id -> id -> id

val abstr_with_type : termset -> id -> hol_type -> id

val appl_exists : termset -> id -> id -> bool

val abstr_exists : termset -> id -> hol_type -> bool


val node_type : signature -> termset -> id -> hol_type

val eta_expand : signature -> termset -> id -> id
(** Eta-expands the given term. *)


val to_string : termset -> string
(** Returns a string representation of the given term. *)

