
{
  open Htparser
  exception Eof
}

let digit =                   ['0'-'9']
let lower =                   ['a'-'z']
let upper =                   ['A'-'Z']
let alphanum =                ((lower)|(upper)|(digit)|['_'])
let any_char =                (_|'\n')
let ddollar =                 ['$']['$']
let unsigned_integer =        (digit)(digit)*
let signed_integer =          ['+''-'](unsigned_integer)
let decimal_part =            ['.'](digit)(digit)*
let real =                    ((signed_integer)|(unsigned_integer))(decimal_part)
let upper_word =              (upper)(alphanum)*
let lower_word =              (lower)(alphanum)*
let atomic_system_word =      (ddollar)(lower)(alphanum)*
let system_comment_one =      ['%'][' ']*(ddollar)(_)*
let system_comment_multi =    ['/']['*'][' ']*(ddollar)([^'*']*['*']['*']*[^'/''*'])*[^'*']*['*']['*']*['/']
let system_comment =          (system_comment_one)|(system_comment_multi)
let comment_one =             ['%'](_)*
let comment_multi =           ['/']['*']([^'*']*['*']['*']*[^'/''*'])*[^'*']*['*']['*']*['/']
let comment =                 (comment_one)|(comment_multi)
let single_quoted =           ['\'']([^'\\''\'']|['\\']['\'']|['\\']['\\'])*['\'']
let distinct_object =         ['"']([^'\\''"']|['\\']['"']|['\\']['\\'])*['"']


rule token = parse
  | '&'    { AMPERSAND }
  | '@'    { AT_SIGN }
  | '^'    { CARET }
  | "cnf"  { CNF }
  | ':'    { COLON }
  | ','    { COMMA }
  | '='    { EQUALS }
  | '!'    { EXCLAMATION }
  | "fof"  { FOF }
  | "hof"  { HOF }
  | ":="   { GETS }
  | '>'    { GREATER }
  | "<="   { IF }
  | "<=>"  { IFF }
  | "=>"   { IMPLIES }
  | "definition"   { DEFINITION }
  | "conjecture"   { CONJECTURE }
  | "axiom"   { AXIOM }
  | "include" { INCLUDE }
  | "lambda"    { LAMBDA }
  | '['    { LBRKT }
  | '('    { LPAREN }
  | "->"    { MAP_TO }
  | "--"    { MMINUS }
  | "~&"    { NAMPERSAND }
  | "!="    { NEQUALS }
  | "<~>"    { NIFF }
  | "~|"    { NVLINE }
  | '.'    { PERIOD }
  | "++"    { PPLUS }
  | '?'    { QUESTION }
  | ']'    { RBRKT }
  | ')'    { RPAREN }
  | '~'    { TILDE }
  | "$false"    { TOK_FALSE }
  | "$true"    { TOK_TRUE }
  | "$i"    { TOK_I }
  | "$o"    { TOK_O }
  | "$type"    { TOK_TYPE }
  | '|'    { VLINE }
  | real as str     { Real(float_of_string str) }
  | signed_integer as str   { Signed_integer(int_of_string str) }
  | unsigned_integer as str { Unsigned_integer(int_of_string str) }
  | decimal_part as str      { Decimal_part(int_of_string str) }
  | upper_word as str      { Upper_word(str) }
  | lower_word as str     { Lower_word(str) }
  | atomic_system_word as str     { Atomic_system_word(str) }
  | single_quoted as str      { Single_quoted(str) }
  | distinct_object as str     { Distinct_object(str) }
  | comment   { token lexbuf }
  | [' ' '\t' '\n']    { token lexbuf }
  | ['\000'-'\040']|['\177']    { token lexbuf }
  | ['\041'-'\176']    { token lexbuf }
  | eof { EOF }

