open Term 
open Hol_type 
open Signature 
open Clause 



let curry f x y = f (x, y)
let uncurry f (x, y) = f x y

let include_combS = ref false

let newSig = Signature.new_signature ()
let symSig = ref newSig

let init sign = 
  symSig := sign

let include_min_comb =  ref false
let include_min_comb =  ref false
    
let in_min_comb count_comb = 
  if count_comb then include_min_comb := true else include_min_comb := false
    
let in_combS count_comb = 
  if count_comb then include_combS := true else include_combS := false

let bigK x = (fun _ -> x)

let const_typargs = ref (bigK [] : (string*typ -> typ list))



(**********************************************************************)
(* convert a Term.term with lambdas into a Term.term with combinators *) 
(**********************************************************************)

let rec is_free bn c = 
  match (bn, c) with 
      (Bound(a), n) -> (a = n)
    | (Abstr(x,_,b), n) -> (is_free b (n + 1))
    | (Appl(p, q), n) -> ((is_free p n) || (is_free q n))
    | (_,_) -> false
	
	  
exception LAM2COMB of term

exception BND of term
exception TYPE of string * term

let rec decre_bndVar t =
  match t with 
      (Bound n) -> Bound (n-1)
    | Appl(p, q) -> Appl((decre_bndVar p),(decre_bndVar q))
    | t -> 
	match t with
	    Const(_,_) -> t
	  | Free(_,_) -> t
	  | Var(_,_) -> t
	  | Abstr(_,_,_) -> raise (BND t) (*should not occur*)
	      



(*******************************************)
let rec lam2comb te bnds count_comb =
  match (te, bnds, count_comb) with
    (Abstr(x,tp,Bound 0), _, count_comb) ->
      let tpI = (abstr_type tp tp) in 
      let _ = in_min_comb count_comb
      in 
      Const("COMBI",tpI) 
  | (Abstr(x,tp,Bound n), bnds, count_comb) -> 
      let (Bound n') = decre_bndVar (Bound n) in 
      let tb = (List.nth bnds n') in 
      let tK = (abstr_type tb (abstr_type tp tb)) in 
      let _ = in_min_comb count_comb 
      in
      Appl(Const("COMBK",tK), (Bound n'))
	
  | (Abstr(x,t1,Const(c,t2)), _, count_comb) ->
      let tK = (abstr_type t2 (abstr_type t1 t2)) in
      let _ = in_min_comb count_comb  
      in 
      Appl(Const("COMBK",tK), Const(c,t2))
	
  | (Abstr(x,t1,Free(v,t2)), _, count_comb) ->
      let tK = (abstr_type t2 (abstr_type t1 t2)) in
      let _ = in_min_comb count_comb
      in
      Appl(Const("COMBK",tK), Free(v,t2))
	
  | (Abstr(x,t1,Var(ind,t2)), _, count_comb) ->
      let tK = (abstr_type t2 (abstr_type t1 t2)) in
      let _ = in_min_comb count_comb 
      in
      Appl(Const("COMBK",tK), Var(ind,t2))
	
  | (Abstr(x,t1,Appl(p,(Bound 0))), bnds, count_comb) ->
    let nfreeP = not(is_free p 0) in 
    let tr = (Term.type_of1 (t1::bnds) p) 
    in
    if nfreeP then (decre_bndVar p)
    else (
      let tI = Type("fun",[t1;t1]) in
      let p' = lam2comb (Abstr(x,t1,p)) bnds count_comb in 
      let tp' = Term.type_of1 bnds p' in 
      let tS = Type("fun",[tp';Type("fun",[tI;tr])]) in 
      let _ = in_min_comb count_comb in 
      let _ = in_combS count_comb 
      in
	(print_string ("lam2comb new appl "^(Term.to_string (Appl(Appl(Const("COMBS",tS),p'), Const("COMBI",tI))))^"\n"));
	Appl(Appl(Const("COMBS",tS),p'), Const("COMBI",tI))
     )
  | (Abstr(x,t1,Appl(p,q)), bnds, count_comb) ->
      (print_string ("lam2comb case Appl(p,q) "^(Term.to_string (Appl(p,q)))^"\n"));
      let (nfreeP,nfreeQ) = (not(is_free p 0),not(is_free q 0)) in 
      let tpq = (Term.type_of1 (t1::bnds) (Appl(p,q)))
      in
	if(nfreeP && nfreeQ) then (
	    let tK = abstr_type tpq (abstr_type t1 tpq) in 
	    let pq' = decre_bndVar(Appl(p, q)) in 
	    let _ = in_min_comb count_comb 
	    in 
	      Appl(Const("COMBK",tK), pq')
	  )
	else (
	    if nfreeP then (
		let q' = lam2comb (Abstr(x,t1,q)) bnds count_comb in 
		let p' = decre_bndVar p in 
		let tp = Term.type_of1 bnds p' in 
		let tq' = Term.type_of1 bnds  q' in
		let tB = Type("fun",[tp;Type("fun",[tq';Type("fun",[t1;tpq])])]) in
		let _ = in_min_comb count_comb
		in
		  Appl(Appl(Const("COMBB",tB), p'), q') 
	      )
	    else (
	  if nfreeQ then (
	    let p' = lam2comb (Abstr(x,t1,p)) bnds count_comb in
	    let q' = decre_bndVar q in
	    let tq = Term.type_of1 bnds q' in
	    let tp' = Term.type_of1 bnds p' in
	    let tC = Type("fun",[tp';Type("fun",[tq;Type("fun",[t1;tpq])])]) in 
	    let _ = in_min_comb count_comb
	    in
	      Appl(Appl(Const("COMBC",tC), p'), q') 
	   )
	  else(
	    let p' = lam2comb (Abstr(x,t1,p)) bnds count_comb in
	    let q' = lam2comb (Abstr(x,t1,q)) bnds count_comb  in
	    let tp' = Term.type_of1 bnds p' in
	    let tq' = Term.type_of1 bnds q' in
	    let tS = Type("fun",[tp';Type("fun",[tq';Type("fun",[t1;tpq])])]) in
	    let _ = in_min_comb count_comb in
	    let _ = in_combS count_comb
	    in
	      Appl(Appl(Const("COMBS",tS), p'), q')
	    )))
  | (t,_, _) -> raise (LAM2COMB t)
      
(*********************)
	
let rec to_comb te bnds count_comb = 
  match (te, bnds, count_comb) with
    (Abstr(x,tp,b), bnds, count_comb) ->
(*       (print_string ("to_comb lam "^(Term.to_string (Abstr(x,tp,b)))^"\n")); *)
      let b' = (to_comb b (tp::bnds) count_comb)
      in
(*       (print_string ("to_comb new body "^(Term.to_string b')^"\n")); *)
      let combTerm = lam2comb (Abstr(x,tp,b')) bnds count_comb in 
      (print_string ("comb_term "^(Term.to_string combTerm)^"\n"));
      combTerm
  | (Appl(p, q), bnds, count_comb) -> 
(*       (print_string ("to_comb appl "^(Term.to_string (Appl(p,q)))^"\n")); *)
      let new_term = (Appl((to_comb p bnds count_comb), (to_comb q bnds count_comb))) in 
      (print_string ("to_comb new appl "^(Term.to_string new_term)^"\n"));
      new_term
  | (t, _, _) -> 
(*       (print_string ("to_comb_term "^(Term.to_string t)^"\n")); *)
      t
	
	
let comb_of t count_comb = 
  to_comb t [] count_comb

let is_bool_type = function 
    (bt_o) -> true
  | (_) -> false
	
let const_prefix = "c_"
let tconst_prefix = "tc_"
let class_prefix = "class_"
            
let lookup_const c =
  c
(* case Symtab.lookup const_trans_table c of *)
(*         SOME c' => c' *)
(*       | NONE => ascii_of c; *)

let lookup_type_const c = 
  c
(* case Symtab.lookup type_const_trans_table c of *)
(*         SOME c' => c' *)
(*       | NONE => ascii_of c; *)
    
let isAlphaNum c = 
  ('A' <= c && c <= 'Z') ||
  ('a' <= c && c <= 'z') ||
  ('0' <= c && c <= '9')


let a_minus_space = (Char.code 'A') - (Char.code ' ')

let ascii_of_c c =
  if isAlphaNum c then Char.escaped c
  else if c = '_' then "__"
  else if ' ' <= c && c <= '/'
  then "_" ^ Char.escaped (Char.chr (Char.code c + a_minus_space))
  else "_" ^ (string_of_int (Char.code c))
	       
(* let  ascii_of = String.translate ascii_of_c *)
    
let make_fixed_type_const c = tconst_prefix ^ lookup_type_const c

let make_type_class clas = class_prefix ^ ascii_of_c clas


    
(******************************************************)
(* data types for typed combinator expressions        *)
(******************************************************)
	
type type_level = T_FULL | T_PARTIAL | T_CONST | T_NONE
  
let typ_level = ref T_CONST
let full_types () = (typ_level:=T_FULL)
let partial_types () = (typ_level:=T_PARTIAL)
let const_types_only () = (typ_level:=T_CONST)
let no_types () = (typ_level:=T_NONE)
    
type kind = Axiom | Conjecture
let name_of_kind = function 
    Axiom -> "axiom"
  | Conjecture -> "conjecture"

(**** Isabelle FOL clauses ****)

type pred_name = string
type axiom_name = string

type polarity = bool
type indexname = string
type clause_id = int

type ctyp_var = Clause.typ_var

type csort = Clause.sort
type ctype_literal = Clause.type_literal

(*FIXME: give the constructors more sensible names*)

type ctyp = Clause.fol_type
	
type combterm = 
    CombConst of string * ctyp * ctyp list
  | CombFree of string * ctyp
  | CombVar of string * ctyp
  | CombApp of combterm * combterm * ctyp
  | Bool of combterm
	
type literal = Literal of polarity * combterm
    
type clauserec = {clause_id: clause_id;
		  axiom_name: axiom_name;
		  th: thm;
		  kind: kind;
		  literals: literal list;
		  ctypes_sorts: (ctyp_var * csort) list; 
		  ctvar_type_literals: ctype_literal list; 
		  ctfree_type_literals: ctype_literal list}
      
type clause = Clause of clauserec 

let string_of_kind (Clause cls) = name_of_kind (cls.kind)
let get_axiomName (Clause cls) = cls.axiom_name
let get_clause_id (Clause cls) = cls.clause_id
    
let get_literals (Clause cls) = cls.literals


let rec type_of = function 
    (Type(a, ts)) ->
      let (folTypes,ts) = types_of ts in 
      let t = make_fixed_type_const a
      in
      (Clause.mk_fol_type("Comp",t,folTypes),ts)
	
  | (TFree(a,s)) ->
      let t = Clause.make_fixed_type_var a 
      in
      ((Clause.mk_fol_type("Fixed",t,[])),[Clause.mk_typ_var_sort (TFree(a,s))])
	
  | (TVar(v,s)) ->
      let t = Clause.make_schematic_type_var v
      in
      (Clause.mk_fol_type("Var",t,[]),[Clause.mk_typ_var_sort (TVar(v,s))])
and types_of ts =
    let foltyps_ts = (List.map type_of ts) in 
    let (folTyps,ts) = Clause.unzip foltyps_ts
    in
	(folTyps,Clause.union_all ts)
    
let domain_type = function 
    Type("fun", [t;_]) -> t
let range_type  = function 
    (Type("fun", [_;t])) -> t

let rec simp_type_of = function 
    (Type (a, ts)) ->
      let typs = List.map simp_type_of ts in 
      let t = Clause.make_fixed_type_const a
      in
      Clause.mk_fol_type("Comp",t,typs)
  | (TFree (a,s)) -> Clause.mk_fol_type("Fixed",Clause.make_fixed_type_var a,[])
  | (TVar (v,s)) -> Clause.mk_fol_type("Var",Clause.make_schematic_type_var v,[])

let comb_typ = function 
    ("COMBI",t) ->
      let t' = domain_type t
      in
      [simp_type_of t']
	
  | ("COMBK",t) ->
      let a = domain_type t in
      let b = domain_type (range_type t)
      in
      List.map simp_type_of [a;b]
	
  | ("COMBS",t) ->
      let t' = domain_type t in 
      let a = domain_type t' in
      let b = domain_type (range_type t') in
      let c = range_type (range_type t')
      in 
      List.map simp_type_of [a;b;c]
	
  |  ("COMBB",t) ->
      let ab = domain_type t in
      let ca = domain_type (range_type t) in
      let a = domain_type ab in
      let b = range_type ab in
      let c = domain_type ca
      in
      List.map simp_type_of [a;b;c]
	
  |  ("COMBC",t) ->
      let t1 = domain_type t in
      let a = domain_type t1 in
      let b = domain_type (range_type t1) in
      let c = range_type (range_type t1)
      in
      List.map simp_type_of [a;b;c]
	
let const_type_of = function 
    ("COMBI",t) ->
      let (tp,ts) = type_of t in 
      let i_var = comb_typ ("COMBI",t)
      in
      (tp,ts,i_var)
	
  | ("COMBK",t) ->
      let (tp,ts) = type_of t in 
      let k_var = comb_typ ("COMBK",t)
      in
      (tp,ts,k_var)
	
  |  ("COMBS",t) ->
    let (tp,ts) = type_of t in
    let s_var = comb_typ ("COMBS",t)
    in
    (tp,ts,s_var)
      
  |  ("COMBB",t) ->
    let (tp,ts) = type_of t in
    let b_var = comb_typ ("COMBB",t)
    in
    (tp,ts,b_var)
      
  |  ("COMBC",t) ->
    let (tp,ts) = type_of t in
    let c_var = comb_typ ("COMBC",t)
    in
    (tp,ts,c_var)
      
  |  (c,t) ->
    let (tp,ts) = type_of t in
    let tvars = !const_typargs(c,t) in
    let tvars' = List.map simp_type_of tvars
    in
    (tp,ts,tvars')
      
      
let is_bool_type = function 
    (Type("bool",[])) -> true
  | (_) -> false



(* convert a Term.term (with combinators) into a combterm, also accummulate sort info *)
let rec combterm_of = function 
    (Const(c,t)) ->
      let (tp,ts,tvar_list) = const_type_of (c,t) in
      let is_bool = is_bool_type t in
      let c' = CombConst(Clause.make_fixed_const c,tp,tvar_list) in
      let c'' = if is_bool then Bool(c') else c'
      in
      (c'',ts)
	
  |  (Free(v,t)) ->
    let (tp,ts) = type_of t in
    let is_bool = is_bool_type t in
    let v' = if Clause.isMeta v then CombVar(Clause.make_schematic_var(v,0),tp)
    else CombFree(Clause.make_fixed_var v,tp) in
    let v'' = if is_bool then Bool(v') else v'
    in
    (v'',ts)
      
  | (Var(v,t)) ->
    let (tp,ts) = type_of t in
    let is_bool = is_bool_type t in
    let v' = CombVar(Clause.make_schematic_var v,tp) in
    let v'' = if is_bool then Bool(v') else v'
    in
    (v'',ts)
      
  | (Appl(p, q)) ->
    let (p',tsP) = combterm_of p in
    let (q',tsQ) = combterm_of q in
    let tp = (Term.type_of (Appl(p,q))) in
    let tp' = simp_type_of tp in
    let is_bool = is_bool_type tp in
    let t' = CombApp(p',q',tp') in
    let t'' = if is_bool then Bool(t') else t'
    in
    (t'', Clause.union_string tsP tsQ)
      
let rec predicate_of (term, pol) = 
  match (term, pol) with  
    (Appl(Const("Not",_), p), polarity) ->
      (predicate_of (p, (not polarity)))
  | (term, polarity) -> (combterm_of term, polarity)
      
      
    
let rec occurs s t = 
  match (s, t) with 
    (a, (CombVar(b,_))) -> a = b
  | (a, (CombApp(t1,t2,_))) -> (occurs a t1) || (occurs a t2)
  | (_, _) -> false
	
let too_general_terms = function 
    (CombVar(v,_), t) -> not (occurs v t)
  | (_) -> false

let too_general_lit = function 
    (Literal(true,(Bool(CombApp(CombApp(CombConst("equal",tp,tps),t1,tp1),t2,tp2))))) -> 
      too_general_terms (t1,t2) || too_general_terms (t2,t1)
  |  (_) -> false

(* forbid a clause that contains hBOOL(V) *)
let rec too_general = function
    ([]) -> false
  | (lit::lits) -> 
      match lit with 
	Literal(_,Bool(CombVar(_,_))) -> true
      | (_) -> too_general lits


let rec literals_of_term1 args term = 
  match (args, term) with
    (args, Appl(Const("Trueprop",_), p)) -> literals_of_term1 args p
  | (args, Appl(Appl(Const("op |",_), p), q)) -> 
      let args' = literals_of_term1 args p
      in
      literals_of_term1 args' q
	
  | ((lits,ts), p) ->
      let ((pred,ts'),pol) = (predicate_of (p, true)) in
      let lits' = Literal(pol,pred)::lits
      in
      (lits', (Clause.union_string ts ts'))
	
	

let literals_of_term p = literals_of_term1 ([],[]) p

(* making axiom and conjecture clauses *)
exception MAKE_CLAUSE

let isFalse = function 
    (Literal(pol,Bool(CombConst(c,_,_)))) ->
      (pol && c = "c_False") ||
      (not pol && c = "c_True")
  | _ -> false


let isTrue = function 
    (Literal (pol,Bool(CombConst(c,_,_)))) -> 
      (pol && c = "c_True") ||
      (not pol && c = "c_False")
    | _ -> false

let isTaut (Clause cls) =
  List.exists isTrue cls.literals  

(**********************************************************************)
(* convert clause into ATP specific formats:                          *)
(* TPTP used by Vampire and E                                         *)
(* DFG used by SPASS                                                  *)
(**********************************************************************)

let type_wrapper = "typeinfo"

let wrap_type (c,t) = 
  match !typ_level with
    T_FULL -> type_wrapper ^ (Clause.paren_pack [c;t])
  | _ -> c
    

let bool_tp = Clause.make_fixed_type_const "bool"

let app_str = "hAPP"

let bool_str = "hBOOL"

exception STRING_OF_COMBTERM of int

let string_of_ctyp = Clause.string_of_fol_type

(* convert literals of clauses into strings *)
let rec string_of_combterm1_aux b t = 
  match (b, t) with 
    (_, (CombConst(c,tp,_))) -> 
      let tp' = string_of_ctyp tp in 
      let c' = if c = "equal" then "fequal" else c
      in
      (wrap_type (c',tp'),tp')
	
  | (_, (CombFree(v,tp))) ->
    let tp' = string_of_ctyp tp
    in
    (wrap_type (v,tp'),tp')
      
  |  (_, (CombVar(v,tp))) ->
    let tp' = string_of_ctyp tp
    in
	(wrap_type (v,tp'),tp')
    
  | (is_pred, (CombApp(t1,t2,tp))) ->
      let (s1,tp1) = string_of_combterm1_aux is_pred t1 in 
      let (s2,tp2) = string_of_combterm1_aux is_pred t2 in 
      let tp' = Clause.string_of_fol_type tp in 
      let r =
	match  !typ_level with 
	  T_FULL -> type_wrapper ^  (Clause.paren_pack [(app_str ^ (Clause.paren_pack [s1;s2]));tp'])
	| T_PARTIAL -> app_str ^ (Clause.paren_pack [s1;s2;tp1])
	| T_NONE -> app_str ^ (Clause.paren_pack [s1;s2])
	| T_CONST -> raise (STRING_OF_COMBTERM 1) (*should not happen, if happened may be a bug*)
      in
      (r,tp')

    
  |  (is_pred, (Bool(CombApp(CombApp(CombConst("equal",tp,tps),t1,tp1),t2,tp2)))) ->
    if is_pred then 
      let (s1,_) = string_of_combterm1_aux false t1 in 
      let (s2,_) = string_of_combterm1_aux false t2
      in
      ("equal" ^ (Clause.paren_pack [s1;s2]),bool_tp)
	
    else
      let (t,_) = string_of_combterm1_aux false (CombApp(CombApp(CombConst("equal",tp,tps),t1,tp1),t2,tp2))
      in
      (t,bool_tp)
	
  |  (is_pred, (Bool(t))) ->
    let (t',_) = string_of_combterm1_aux false t in
    let r = if is_pred then bool_str ^ (Clause.paren_pack [t'])
    else t'
    in
    (r,bool_tp)
      

let string_of_combterm1 is_pred term = fst (string_of_combterm1_aux is_pred term)

let rec string_of_combterm2 a b = 
  match (a,b) with  
    (_, (CombConst(c,tp,tvars))) ->
      let tvars' = List.map string_of_ctyp tvars in
      let c' = if c = "equal" then "fequal" else c
      in
      c' ^ (Clause.paren_pack tvars')
	     
  | (_, (CombFree(v,tp))) -> v
  | (_, (CombVar(v,tp))) -> v
  | (is_pred, (CombApp(t1,t2,tp))) ->
      let s1 = string_of_combterm2 is_pred t1 in
      let s2 = string_of_combterm2 is_pred t2
      in
      app_str ^ (Clause.paren_pack [s1;s2])
    
  | (is_pred, (Bool(CombApp(CombApp(CombConst("equal",tp,tps),t1,tp1),t2,tp2)))) ->
    if is_pred then 
      let s1 = string_of_combterm2 false t1 in
      let s2 = string_of_combterm2 false t2
      in
      ("equal" ^ (Clause.paren_pack [s1;s2]))
    else
      string_of_combterm2 false (CombApp(CombApp(CombConst("equal",tp,tps),t1,tp1),t2,tp2))
 
  | (is_pred, (Bool(t))) ->
      let t' = string_of_combterm2 false t
      in
      if is_pred then bool_str ^ (Clause.paren_pack [t'])
      else t'
	  



let string_of_combterm is_pred term = 
    match  !typ_level with 
      T_CONST -> string_of_combterm2 is_pred term
    | _ -> string_of_combterm1 is_pred term
	  
	  
let string_of_clausename (cls_id,ax_name) = 
    Clause.clause_prefix ^  ax_name ^ "_" ^ (string_of_int cls_id)

let string_of_type_clsname (cls_id,ax_name,idx) = 
    string_of_clausename (cls_id,ax_name) ^ "_tcs" ^ (string_of_int idx)


(* tptp format *)

let tptp_literal (Literal(pol,pred)) =
  let pred_string = string_of_combterm true pred in
  let pol_str = if pol then "++" else "--"
    in
  pol_str ^ pred_string
	      

 
let tptp_type_lits (Clause cls) = 
  let lits = List.map tptp_literal (cls.literals) in
  let ctvar_lits_strs =
    match !typ_level with
      T_NONE -> []
    | _ -> (List.map Clause.tptp_of_typeLit (cls.ctvar_type_literals))
  in
  let ctfree_lits = 
    match !typ_level with
      T_NONE -> []
    | _ -> (List.map Clause.tptp_of_typeLit (cls.ctfree_type_literals )) 
  in
  (ctvar_lits_strs @ lits, ctfree_lits)
    
let clause2tptp cls =
  let (lits,ctfree_lits) = tptp_type_lits cls in 
  let cls_id = get_clause_id cls in
  let ax_name = get_axiomName cls in
  let knd = string_of_kind cls in
  let lits_str = Clause.bracket_pack lits in
  let cls_str = Clause.gen_tptp_cls(cls_id,ax_name,knd,lits_str)
  in
  (cls_str,ctfree_lits)
    


let make_clause(clause_id,axiom_name,kind,thm,is_user) =
  try (
      let term = Clause.prop_of thm in 
      let term' = (comb_of term is_user) in
	(print_string ("comb of term "^(Term.to_string term')^"\n"));  
	let (lits,ctypes_sorts) = literals_of_term term' in
	let (ctvar_lits,ctfree_lits) = Clause.add_typs_aux ctypes_sorts
	in
	  if List.for_all isFalse lits
	  then (print_string "Problem too trivial for resolution (empty clause)";      raise MAKE_CLAUSE)
	  else 
	    if (too_general lits) then 
	    (*       (print_string "Omitting " ^ axiom_name ^ ": clause contains universal predicates";  *)
	    raise MAKE_CLAUSE
	  else
	    if (!typ_level <> T_FULL) && kind=Axiom && (List.for_all too_general_lit lits)
	  then 
(* 	(print_string "Omitting " ^ axiom_name ^ ": equalities are too general";  *)
	    raise MAKE_CLAUSE
	  else
	    Clause({clause_id = clause_id;
		    axiom_name = axiom_name;
		    th = thm;
		    kind = kind;
		    literals = lits;
		    ctypes_sorts = ctypes_sorts;
		    ctvar_type_literals = ctvar_lits;
		    ctfree_type_literals = ctfree_lits}))
  with TYPE(msg, te) -> 
    (print_string ("ERROR: " ^msg^" "^(Term.to_string te)^"\n")); 
    raise MAKE_CLAUSE
      
   
let make_axiom_clause thm (ax_name,cls_id,is_user) = make_clause(cls_id,ax_name,Axiom,thm,is_user)
 
let rec make_axiom_clauses thms ls = 
  match (thms, ls) with 
    ([], user_lemmas) -> []
  | (((thm,(name,id))::thms), user_lemmas) -> 
      let is_user = List.mem name user_lemmas in 
      let cls = 
	try 
	  Some (make_axiom_clause thm (name,id,is_user))  
	with
	  MAKE_CLAUSE -> None
      in 
      let clss = make_axiom_clauses thms user_lemmas
      in
      match cls with 
	None -> clss
      | Some(cls') -> if isTaut cls' then clss else (name,cls')::clss
    

let make_conjecture_clause n thm = make_clause(n,"conjecture",Conjecture,thm,true)
    
let rec make_conjecture_clauses_aux n xs = 
  match (n, xs) with 
    (_, []) -> []
  | (n, (t::ts)) ->
      (make_conjecture_clause n t) :: (make_conjecture_clauses_aux (n+1) ts)
				     
let make_conjecture_clauses = make_conjecture_clauses_aux 0

let prefix = "/home/jzimmer/import/MathServe/thirdParty/leo/src/atp-inputs/"

let read_in fs = List.map (fun filename  -> 
  let file = open_in filename in
  let size = in_channel_length file in
  let buf = String.create size in 
  really_input file buf 0 size;
  close_in file; 
  buf) 
    fs

let get_helper_clauses_tptp = 
  let tlevel = match !typ_level with
    T_FULL -> prefix^"full_"
  | T_PARTIAL -> prefix^"par_"
  | T_CONST -> prefix^"const_"
  | T_NONE -> prefix^"u_"
  in
  let helpers = 
    if !include_combS then
      (print_string "Include combinator S";  ["helper1.tptp";"comb_inclS.tptp"])
    else
      if !include_min_comb then
	(print_string "Include min combinators" ;   ["helper1.tptp";"comb_noS.tptp"])
      else (print_string "No combinator is used";  ["helper1.tptp"])
  in
  let t_helpers = List.map (fun x  -> tlevel^x) helpers 
  in
  read_in t_helpers
    

let tptp_write_file thms filename (axclauses,classrel_clauses,arity_clauses) user_lemmas=
    let clss = make_conjecture_clauses thms in
    let (clnames,axclauses') = Clause.unzip (make_axiom_clauses axclauses user_lemmas) in 
    let (tptp_clss,tfree_litss) = Clause.unzip (List.map clause2tptp clss) in
    let tfree_clss = List.map Clause.tptp_tfree_clause (List.fold_left Clause.union_string [] tfree_litss) in
    let out = open_out filename in 
    let helper_clauses = (get_helper_clauses_tptp)
    in
(* TODO     List.app (curry TextIO.output out o #1 o clause2tptp) axclauses'  *)
(*     List.app (curry TextIO.output out o (match o with x,_ -> x) clause2tptp) axclauses' ; *)
    Clause.writeln_strs out tfree_clss ;
    Clause.writeln_strs out tptp_clss ;
    (print_string "clauses generated: \n");
    List.iter (fun s -> print_string s) tfree_clss;
    List.iter (fun s -> print_string s) tptp_clss;
(*     List.app (curry TextIO.output out o Clause.tptp_classrelClause) classrel_clauses ; *)
(*     List.app (curry TextIO.output out o Clause.tptp_arity_clause) arity_clauses ; *)
(*     List.app (curry TextIO.output out) helper_clauses ; *)
(*     TextIO.closeOut out ; *)
    close_out out ;
    clnames
      
