type order = LESS | EQUAL | GREATER

module type KEY =
sig
  type key 
  val ord: key * key -> order
end

module type TABLE =
sig
  type key
  type 'a table
  exception DUP of key
  exception DUPS of key list
  exception SAME
  exception UNDEF of key
  val empty: 'a table
  val is_empty: 'a table -> bool
  val curry : ('a * 'b -> 'c) -> 'a -> 'b -> 'c
  val uncurry : ('a -> 'b -> 'c) -> 'a * 'b -> 'c
  val ( |> ) : 'a -> ('a -> 'b) -> 'b
  val ( |-> ) : 'a * 'b -> ('a -> 'b -> 'c) -> 'c
  val ( |>> ) : 'a * 'b -> ('a -> 'c) -> 'c * 'b
  val ( ||> ) : 'a * 'b -> ('b -> 'c) -> 'a * 'c
  val ( |>>> ) : 'a * 'b -> ('a -> 'c * 'd) -> 'c * ('b * 'd)
  val ( ||>> ) : 'a * 'b -> ('b -> 'c * 'd) -> ('a * 'c) * 'd
(*   val map: ('a -> 'b) -> 'a table -> 'b table *)
(*   val map': (key -> 'a -> 'b) -> 'a table -> 'b table *)
  val fold: (key * 'b -> 'a -> 'a) -> 'b table -> 'a -> 'a
(*   val fold_map: (key * 'b -> 'a -> 'c * 'a) -> 'b table -> 'a -> 'c table * 'a *)
(*   val foldl: ('a * (key * 'b) -> 'a) -> 'a * 'b table -> 'a *)
  val dest: 'a table -> (key * 'a) list
(*   val keys: 'a table -> key list *)
  val min_key: 'a table -> key option
  val max_key: 'a table -> key option
(*   val exists: (key * 'a -> bool) -> 'a table -> bool *)
(*   val forall: (key * 'a -> bool) -> 'a table -> bool *)
(*   val defined: 'a table -> key -> bool *)
(*   val lookup: 'a table -> key -> 'a option *)
  val update: (key * 'a) -> 'a table -> 'a table
  val update_new: (key * 'a) -> 'a table -> 'a table
  val default: key * 'a -> 'a table -> 'a table
  val map_entry: key -> ('a -> 'a) -> 'a table -> 'a table
(*   val map_default: (key * 'a) -> ('a -> 'a) -> 'a table -> 'a table *)
(*   val make: (key * 'a) list -> 'a table                                (\*exception DUPS*\) *)
(*   val extend: 'a table * (key * 'a) list -> 'a table                   (\*exception DUPS*\) *)
(*   val join: (key -> 'a * 'a -> 'a) (\*exception DUP/SAME*\) -> *)
(*     'a table * 'a table -> 'a table                                    (\*exception DUPS*\) *)
(*   val merge: ('a * 'a -> bool) -> 'a table * 'a table -> 'a table      (\*exception DUPS*\) *)
(*   val delete: key -> 'a table -> 'a table                              (\*exception UNDEF*\) *)
(*   val delete_safe: key -> 'a table -> 'a table *)
(*   val member: ('b * 'a -> bool) -> 'a table -> key * 'b -> bool *)
(*   val insert: ('a * 'a -> bool) -> key * 'a -> 'a table -> 'a table    (\*exception DUP*\) *)
(*   val remove: ('b * 'a -> bool) -> key * 'b -> 'a table -> 'a table *)
(*   val lookup_list: 'a list table -> key -> 'a list *)
(*   val update_list: (key * 'a) -> 'a list table -> 'a list table *)
(*   val insert_list: ('a * 'a -> bool) -> key * 'a -> 'a list table -> 'a list table *)
(*   val remove_list: ('b * 'a -> bool) -> key * 'b -> 'a list table -> 'a list table *)
(*   val make_list: (key * 'a) list -> 'a list table *)
(*   val dest_list: 'a list table -> (key * 'a) list *)
(*   val merge_list: ('a * 'a -> bool) -> *)
(*     'a list table * 'a list table -> 'a list table *)
end

module TableFun(Key: KEY): TABLE = struct

type key = Key.key 

type 'a table =
  Empty |
  Branch2 of 'a table * (key * 'a) * 'a table |
  Branch3 of 'a table * (key * 'a) * 'a table * (key * 'a) * 'a table

exception DUP of key
exception DUPS of key list
 
(* let I x = x *)
(* let K x = fn _ => x *)
let curry f x y = f (x, y)
let uncurry f (x, y) = f x y


let (|>) x f = f x
let (|->) (x, y) f = f x y
let (|>>) (x, y) f = (f x, y)
let (||>) (x, y)  f = (x, f y)
let (|>>>) (x, y) f = let (x', z) = (f x) in (x', (y, z)) 
let (||>>) (x, y) f = let (z, y') = (f y) in ((x, z), y') 

(*reverse composition*)
(* fun (#>) f g = g o f *)
(* fun (#->) f g = uncurry g o f; *)


let empty = Empty

let is_empty = function 
    Empty -> true
  |  _ -> false

let pair a = 
  let pairc b = (a,b) 
  in pairc

(* map and fold combinators *)

let map_table f = 
  let rec map = function 
      Empty -> Empty
    | Branch2(left, (k, x), right) ->
        Branch2 (map left, (k, f k x), map right)
    | (Branch3(left, (k1, x1), mid, (k2, x2), right)) ->
        Branch3(map left, (k1, f k1 x1), map mid, (k2, f k2 x2), map right)
  in map 
    
let fold_table f =
    let rec fold t x = 
      match (t, x) with 
        (Empty, x) -> x
      | ((Branch2(left, p, right)), x) -> fold right (f p (fold left x))
      | ((Branch3(left, p1, mid, p2, right)), x) ->
          fold right (f p2 (fold mid (f p1 (fold left x))))
    in fold 

let fold_map_table f =
  let rec fold_map t x =
    match (t, x) with
      (Empty, s) -> (Empty, s)
    | (Branch2(left, (k, x), right), s) ->
	let myfn ((l, e), r) = pair (Branch2(l, (k, e), r)) in 
        s
    |> fold_map left
    ||>> f (k,x)
    ||>> fold_map right
    |-> myfn
	  
    | (Branch3(left, (k1, x1), mid, (k2, x2), right), s) ->
	let myfn ((((l, e1), m), e2), r) = pair (Branch3(l, (k1, e1), m, (k2, e2), r)) in
        s
    |> fold_map left
    ||>> f (k1, x1)
    ||>> fold_map mid
    ||>> f (k2, x2)
    ||>> fold_map right
    |-> myfn 
  in fold_map
   
let cons a b = a::b
let dest tab = List.rev (fold_table cons tab [])  
(* let keys tab = List.rev (fold_table (cons o #1) tab []) *)

exception TRUE

(* let exists pred tab = *)
(*   try  *)
(*     let myfn p () = if pred p then raise TRUE else () in  *)
(*     (fold_table tab () false) *)
(*   with TRUE -> true *)
(* (\*       (fn p => fn () => if pred p then raise TRUE else ()) *\) *)
(* let forall pred = not o exists (not o pred) *)

let rec min_key = function 
    Empty -> None
  | (Branch2(Empty, (k, _), _)) -> Some k
  | (Branch3(Empty, (k, _), _, _, _)) -> Some k
  | (Branch2(left, _, _)) -> min_key left
  | (Branch3(left, _, _, _, _)) -> min_key left

let rec max_key = function 
    Empty-> None
  |  (Branch2(_, (k, _), Empty)) -> Some k
  |  (Branch3(_, _, _, (k, _), Empty)) -> Some k
  |  (Branch2(_, _, right)) -> max_key right
  |  (Branch3(_, _, _, _, right)) -> max_key right

(* modify *)

type 'a growth =
  Stay of 'a table |
  Sprout of 'a table * (key * 'a) * 'a table

exception SAME

let modify key f tab =
  let rec modfy = function 
      Empty -> Sprout(Empty, (key, f None), Empty)
    | Branch2(left, (k, x), right) ->
        (match Key.ord (key, k) with
          LESS ->
          (match modfy left  with
            Stay left' -> Stay (Branch2(left', (k, x), right))
	  | Sprout(left1, q, left2) -> Stay (Branch3(left1, q, left2, (k, x), right)))
	| EQUAL -> Stay (Branch2(left, (k, f (Some x)), right))
        | GREATER ->
              (match modfy right with
                Stay right' -> Stay (Branch2(left, (k, x), right'))
              | Sprout(right1, q, right2) ->
                Stay (Branch3(left, (k, x), right1, q, right2))))
    | (Branch3(left, (k1, x1), mid, (k2, x2), right)) ->
        (match Key.ord (key, k1) with
          LESS ->
          (match modfy left with
            Stay left' -> Stay (Branch3(left', (k1, x1), mid, (k2, x2), right))
          | Sprout(left1, q, left2) ->
            Sprout(Branch2(left1, q, left2), (k1, x1), Branch2(mid, (k2, x2), right)))
        | EQUAL -> Stay (Branch3(left, (k1, f (Some x1)), mid, (k2, x2), right))
        | GREATER ->
          (match Key.ord (key, k2) with
            LESS ->
            (match modfy mid with
              Stay mid' -> Stay (Branch3(left, (k1, x1), mid', (k2, x2), right))
              | Sprout(mid1, q, mid2) ->
                Sprout(Branch2(left, (k1, x1), mid1), q, Branch2(mid2, (k2, x2), right)))
          | EQUAL -> Stay (Branch3(left, (k1, x1), mid, (k2, f (Some x2)), right))
          | GREATER ->
            (match modfy right with
              Stay right' -> Stay (Branch3(left, (k1, x1), mid, (k2, x2), right'))
            | Sprout(right1, q, right2) ->
              Sprout(Branch2(left, (k1, x1), mid), (k2, x2), Branch2(right1, q, right2)))))
	  
  in
    try 
      match (modfy tab) with
	  Stay tab' -> tab'
      | Sprout(a,b,c) -> Branch2(a,b,c)
    with SAME -> tab
      
      
let update (key, x) tab = modify key (fun _ -> x) tab
  
let update_new (key, x) tab = modify key (fun y -> match y with None -> x | Some _ -> raise (DUP key)) tab
      
let default (key, x) tab = let myfn = function 
    None -> x | Some _ -> raise SAME in 
			     modify key myfn tab
let map_entry key f = let myfn = function 
    None -> raise SAME | Some x -> f x in 
  modify key myfn 
let map_default (key, x) f = let myfn =function 
    None -> f x | Some x -> f x in 
			       modify key myfn
				 
(* delete *)

exception UNDEF of key

let compare a b = 
  match (a,b) with 
    (None, (k2, _)) -> LESS
  | ((Some k1), (k2, _))  -> Key.ord (k1, k2)

let if_eq u v w = 
  match (u,v,w) with 
    (EQUAL, x, y) -> x
  | (_, x, y) -> y

(* simultaneous modifications *)

let reject_dups t s = 
  match (t, s) with 
      (tab, []) -> tab
    | (_, dups) -> raise (DUPS (List.rev dups))
	
let fold = fold_table

let extend (table, args) =
  let add (key, x) (tab, dups) =
    try (update_new (key, x) tab, dups) 
    with DUP dup -> (tab, dup::dups)
  in
    reject_dups (fold add args (table, [])) 

let make entries = extend (empty, entries)

(* structure Inttab = TableFun(type key = int let ord = int_ord) *)

end

