(in-package "USER")

(export 'run-mathweb)

(defvar counter 0 "Counter for unique names of instantiated operators")
(defvar all-solutions nil "All solutions of different runs.")

;;; extended 13.10.2004: now searches for all plans for queries derived
;;; from the original query by changing the type of the output parameter
;;; (last object in objects list) of the query.
(defun run-mathweb ()
  (if (> (sys:command-line-argument-count) 2)
      (let* ((domain-file-name (sys:command-line-argument 1))
	     (time-bound-float (read-from-string (sys:command-line-argument
                                                  2)))
             (depth-bound (read-from-string (sys:command-line-argument 3)))
	     (output-file-name (sys:command-line-argument 4))
	     )
	(format t "~%domain-file-name ~S" domain-file-name)
	(format t "~%time-bound-float ~S ~S" time-bound-float (single-float-p time-bound-float))
	(format t "~%outputfile ~S" output-file-name)
	
	(load domain-file-name)
	(load-domain)
	(setf prodigy4::*dont-ask-and-find-all* :different)
	(pset :linear t)

	(setf all-solutions nil)
	
	(let* ((current (current-problem))
	       (prob-name (p4::problem-name current))
	       (rev-objs (reverse (p4::problem-objects current)))
	       (output (if (listp rev-objs)
			   (first rev-objs)
			 null))
	       (sdfsd (setf final output))
	       (oname (if (listp output)
			 (first output)))
	       (type (if (and (listp output) (listp (cdr output)))
			 (second output)))
	       (sdfsd (setf finalt type))
	       (subtypes (if type
			     (p4::all-subtypes type
					       p4::*current-problem-space*)
			   nil))
	       (initial (p4::problem-state current))
	       (goal (p4::problem-goal current))
	       (new-time-bound (if (> (length subtypes) 0)
				   (/ time-bound-float
				      (+ (length subtypes)
					 1))
				 time-bound-float))
	       )
	  (format t "time-bound: ~A" new-time-bound)

	  (run :output-level 1 :time-bound new-time-bound :multiple-sols t
	       ;;:search-default :breadth-first
	       :depth-bound depth-bound)
	  (if (> (length prodigy4::*all-solutions*) 1)
	      (setf all-solutions (append
				   (reverse (rest prodigy4::*all-solutions*))
				   all-solutions)))
	  (dolist (stype subtypes)
	    (format t "subtype: ~A" stype)
	    (let* ((typename (p4::type-name stype))
		   (new-out (list oname typename))
		   (new-objs (reverse (cons new-out (rest rev-objs))))
		   (new-prob
		    (p4::make-problem :name prob-name
				      :objects new-objs
				      :goal goal
				      :state initial
				      :plist nil))
		   )
	      (setf (current-problem) new-prob)
	      (run :output-level 1 :time-bound new-time-bound :multiple-sols t
		   ;; :search-default :breadth-first
		   :depth-bound depth-bound)
	      (if (> (length prodigy4::*all-solutions*) 1)
		  (setf all-solutions (append
				       all-solutions
				       (reverse (rest prodigy4::*all-solutions*))
				       )))
	      )
	    ) ;; end dolist
	  ) ;; end (let ((current
	
	;; 	(if (> (length prodigy4::*all-solutions*) 1)
	;; 	    (let* ((sols (rest prodigy4::*all-solutions*))
	       ;; 		   )
	(with-open-file (stream output-file-name :direction :output :if-exists :supersede)
			(format stream "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>~%")
			(format stream "~%<prodigy>~%")
			(mapcar #'(lambda (plan)
				    (let* ((ops (rest plan))
					   )
				      (format stream "~%<plan>~%")
				      (mapcar #'(lambda (op)
						  (print-inst-op op stream)
						  )
					      ops)
				      (format stream "</plan>~%")
				      ))
				all-solutions)
			(format stream "~%</prodigy>~%")
			)
	(exit))
    t))  

(defun print-inst-op (inst-op stream)
  (declare (type instantiated-op inst-op))

  (let* ((op (p4::instantiated-op-op inst-op))
	 (vars (p4::rule-vars op))
	 (values (p4::instantiated-op-values inst-op)))
    (format stream " <operator name=\"~A\">~%" (p4::operator-name op))
    (dolist (argument (cdr (p4::rule-params op)))
      (let ((position (position argument vars)))
	(if (>= position (length values))
	    (error "Trying to print a parameter that is not in the
bindings for operator ~S - perhaps some of the parameters are not in
the preconditions?" op))
	(let ((value (elt values position)))
	  (format stream " ")
	  (if (listp value)
	      (format stream "   <arg name=\"~A\"/>~%" argument)
	    (print-literal value stream)))))
    (format stream " </operator>~%"))
  (when (and user::*analogical-replay*
	     (p4::instantiated-op-binding-node-back-pointer inst-op))
    (let ((guidance (getf (p4::nexus-plist
			   (p4::instantiated-op-binding-node-back-pointer inst-op))
			  :guided)))
      (if guidance (format stream " ~S" guidance)))))


(defun print-literal (x stream)
  (format stream "   <arg name=\"~A\"/>~%"
	  (if (p4::prodigy-object-p x) (p4::prodigy-object-name x) x)))

